#!/usr/bin/perl
#
# Script to compare user-generated CSPP_NUCAPS output files with test
# output files provided ...
#
# Uses only Perl modules that are part of the v5.006 standard install.
#
# Pre-requisites:
# CSPP_NUCAPS distribution in directory $CSPP_NUCAPS_HOME
# (needed to find a version of h5diff supplied with CSPP_NUCAPS)
#
# START-HELP (N.B. This tag is used in printing help, do not touch!)
#
#  Options:
#  --version             show program's version information and exit
#  ?, -h, --help         show this help message and exit
#  -n, --nothing         glob & check matching files, but do not run h5diff.
#
#  Mandatory Arguments:
#    At a minimum these arguments must be specified [i.e. no default].
#
#    -s MISSION, --sat=MISSION
#                        The mission satellite data for NUCAPS algorithm.
#                        Possible values (following NUCAPS documentation) ...
#                        'npp', 'j01'.
#    -i INPUT_DIR, --input_dir=INPUT_DIR
#                        Path to verification files directory. The expectation
#                        is that there will be as many verification files as 
#                        WORK_DIR files of the specified MISSION, beyond that
#                        no checking is done to ensure one-to-one matching of
#                        file dates/times, so beware.
#                        e.g. -i ../output
#
#  Extra Options:
#    These options may be used to customize behaviour of this program.
#
#    -w WORK_DIR, --work=WORK_DIR
#                        The directory in which all activity will occur,
#                        this is where user output files are sought.
#                        [default: current directory].
#    -m, --machineprec   Set deltas for variables to machine precision
#    -v, --verbose       Each occurrence increases verbosity 1 level from
#                        [default: ERROR only] -v=WARNING -vv=INFO -vvv=DEBUG
#
# END-HELP (N.B. This tag is used in printing help, do not touch!)
# 


# Pragmata
# ````````
use 5.006;
use strict;
use warnings;


# Load Perl modules - standard modules only
# `````````````````````````````````````````
use File::Spec;
use Data::Dumper;
use Getopt::Long;
use File::Basename;
use Encode qw(decode);
use Cwd qw(abs_path getcwd);
use List::Util qw(sum min max);
use I18N::Langinfo qw(langinfo CODESET);


# Global variables are all UPPERCASE throughout
# `````````````````````````````````````````````
our $MAIN = basename($0);
defined $ENV{CSPP_NUCAPS_HOME} or die "$MAIN\:: CSPP_NUCAPS_HOME not set.\nAbort";


# $H5DUMP points to supplied static-compiled h5dump (version 1.8.11)
# ``````````````````````````````````````````````````````````````````
our $H5DIFF = File::Spec -> catfile($ENV{'CSPP_NUCAPS_HOME'}, 'bin', 'h5diff');
(-e $H5DIFF) or die "$MAIN\:: h5diff $H5DIFF not found.\nAbort";
(-x $H5DIFF) or die "$MAIN\:: h5diff $H5DIFF not executable.\nAbort";


# $H5DUMP points to supplied static-compiled h5dump (version 1.8.11)
# ``````````````````````````````````````````````````````````````````
our $NCDUMP = File::Spec -> catfile($ENV{'CSPP_NUCAPS_HOME'}, 'bin', 'ncdump');
(-e $NCDUMP) or die "$MAIN\:: ncdump $NCDUMP not found.\nAbort";
(-x $NCDUMP) or die "$MAIN\:: ncdump $NCDUMP not executable.\nAbort";


# List the variables to verify in each file type
# ``````````````````````````````````````````````
# I am simplifying to a list of variable names without reference to the
# file type in which they occur - there is a lot of overlap in the NUCAPS
# output files.  I hope that the variable type and scale factor is consistent
# for the names used in multiple files types else there will be repercussions!
# This does mean that I will have to suppress warnings if varname is not found
# in any specific file since now there is no requirement that it be in there
# UNLESS it IS in the reference file against which a file is being compared.
my @varnames = qw(AD Ascending_Descending ATMS_Brightness_Temps ATMS_Channels
  ATMS_Frequencies ATMS_Latitude ATMS_Longitude ATMS_QF1 ATMS_QF2
  ATMS_Satellite_Azimuth ATMS_Satellite_Height ATMS_Satellite_Zenith
  ATMS_Solar_Azimuth ATMS_Solar_Zenith CH4 CH4_MR Cloud_Emis Cloud_Fraction
  Cloud_Freq Cloud_Height Cloud_Refl Cloud_Top_Fraction Cloud_Top_Pressure CO
  CO_MR CO2 CrIS_Channels CrIS_FORs CrIS_FOVs CrIS_Frequencies CrIS_Latitude
  CrIS_Longitude CrIS_QF1 CrIS_QF2 CrIS_QF3 CrIS_QF4 CrIS_QF5 CrIS_QF6
  CrIS_Radiances CrIS_RR CrIS_RR_Channels CrIS_RR_Frequencies
  CrIS_RR_Quality_Flag CrIS_View_Angle Effective_Pressure FG_H2O FG_H2O_MR
  FG_IR_Emis_Freq FG_IR_Surface_Emis FG_Mean_CO2 FG_O3 FG_O3_MR
  FG_Skin_Temperature FG_Temperature FLUX H2O H2O_MR HNO3 HNO3_MR
  Ice_Liquid_Flag IR_Emis_Freq IR_Surface_Emis IR_Surface_Refl
  Ispare_Field IT_H2O_MR Land_Fraction Land_Fraction_Error LAT Latitude
  Liquid_H2O Liquid_H2O_MR LON Longitude Mean_CO2 MIT_H2O MIT_MW_Emis
  MIT_Skin_Temperature MIT_Temperature MW_Emis MW_Frequency MW_Surface_Class
  MW_Surface_Emis N_Smw_Per_FOV N2O N2O_MR ncemis_Per_FOV ncld_Per_FOV
  nemis_Per_FOV O3 O3_MR Orbit_Number PC_Scores Pressure Principal_Components
  QA Quality_Flag quality_information Rspare_Field Satellite_Azimuth
  Satellite_Height Satellite_Zenith SATHEIGHT SATZEN Scan_Line SCPosition
  Skin_Temperature SO2 SO2_MR Solar_Azimuth Solar_Zenith Stability
  Subset_CrIS_FORs Subset_CrIS_FOVs Surface_Pressure Temperature Time TIME
  Topography View_Angle VIEWANG);


# Get the command line options, and display help or revision if asked
# ```````````````````````````````````````````````````````````````````
my $revision = "Verification Perl script for CSPP_NUCAPS";
my $pwd = getcwd();
my %options = get_commandline($revision, $pwd);


# These are used for fileglobbing when INPUTFILES is a directory name
# ```````````````````````````````````````````````````````````````````
# I have simplified this down to just those .nc files that include the
# satellite id 'npp' or 'j01'.  This menas that files named like ...
#   NUCAPS_C0300_ALLFOVS_PCS3B_20180716_1238159_1238457.nc  and
#   NUCAPS_CCR_20180716_1238159_1238457.nc 
# ... will not get caught in the fileglob.  Too bad.  If this is a
# problem come back and rework this later.
my %filemask = (npp => 'NUCAPS*_npp_*.nc',
                j01 => 'NUCAPS*_j01_*.nc');


# Make lists of files to compare just for MISSION $options{sat}
# `````````````````````````````````````````````````````````````
my @user_files = (sort(glob("$options{work}/$filemask{$options{sat}}")));
my @veri_files = (sort(glob("$options{input_dir}/$filemask{$options{sat}}")));
($options{verbose} > 1) and do {
  print "usermask:  $options{work}/$filemask{$options{sat}}\n";
  print "verimask:  $options{input_dir}/$filemask{$options{sat}}\n"; };

# Same number of each or fail
# ```````````````````````````
($#user_files == $#veri_files) or
  die "$MAIN\:: Number of user-generated and test files do not match\nFAIL";


# If --nothing, time to bailout
# `````````````````````````````
($options{nothing}) and exit(0);
my $verbose = $options{verbose};
my $machine = $options{machineprec};


# Verify IMG files
# ````````````````
for (my $i=0; $i<=$#user_files; $i++) {
  my $passed = compare_files($user_files[$i], $veri_files[$i], $verbose, $machine, @varnames);
  my $user = basename($user_files[$i]);
  ($passed) and print "PASSED :  $user\n";
  ($passed) or  print "FAIL-FAIL-FAIL-FAIL-FAIL\nFAILED :  $user\n";
}


# All done, exit
# ``````````````
exit(0);


###############################################################################
#                                                                             #
#         User-defined functions - they are listed alphabetically             #
#                                                                             #
#   NOTE: My editor can collapse code between matching {} braces, this        #
#   explains the strange formatting - I can collapse each function to see     #
#   just the function name and an example of how to call it; and expand       #
#   to see the more detailed documentation and to edit.      JED Jul 2018     #
#                                                                             #
###############################################################################

# -----------------
  sub compare_files
# =================
# $passed = compare_files($test_file, $veri_file, $verbose, $machine, @variables);
  {
    # $test_file is the file you are checking
    # $veri_file is the file it should match all variables, but not have to
    #   match global attributes like processing date/time.
    # $verbose is 0,1,2 or 3 (higher number, more diagnostic output).
    # $machine is 0 or 1 (1 means override the variable --delta settings and
    #   revert to using --use-system-epsilon for all variables).
    # @variables is the list of variables to verify.
    # $passed is return code: 1 is PASS, 0 is FAIL.
    # Uses: global variable $H5DIFF which is path/name of the h5diff executable.
    my ($test_file, $veri_file, $verbosity, $machine) = (shift, shift, shift, shift);
    my  @variables = @_;
    my $passed = 1;
    my $user = basename($test_file);
    ($verbose >= 2) and print "Verifying $user\n";
    my $files = "$test_file $veri_file";
    my $precision = "--use-system-epsilon";
    # If a variable is not listed in this hash then apply "--use-system-epsilon"
    my %prec_lut = (
      '--delta=2' => [qw/
        Mean_CO2/],
      '--delta=0.2' => [qw/
        FG_Skin_Temperature FG_Temperature MIT_Skin_Temperature MIT_Temperature
        Skin_Temperature/],
      '--delta=0.5' => [qw/
        Temperature/],
      '--delta=0.01' => [qw/
        IR_Surface_Emis IR_Surface_Refl MIT_MW_Emis MW_Emis MW_Surface_Emis FLUX VIEWANG SATHEIGHT/],
      '--delta=0.001' => [qw/
        ATMS_Satellite_Height Satellite_Height/],
      '--delta=0.0001' => [qw/
        ATMS_Latitude ATMS_Longitude ATMS_Satellite_Azimuth ATMS_Satellite_Zenith
        ATMS_Solar_Azimuth ATMS_Solar_Zenith CrIS_Latitude CrIS_Longitude
        CrIS_View_Angle Satellite_Azimuth Satellite_Zenith Solar_Azimuth
        Solar_Zenith View_Angle/],
      '--delta=0.00001' => [qw/
        ATMS_Brightness_Temps/],
      '--delta=0.000001' => [qw/
        CrIS_Radiances Land_Fraction Land_Fraction_Error SCPosition Topography/],
      '--relative=0.1' => [qw/
        CH4 CH4_MR CO2 FG_H2O FG_H2O_MR FG_IR_Surface_Emis FG_O3 FG_O3_MR
        HNO3 HNO3_MR MIT_H2O MIT_H2O_HR N2O N2O_MR O3
        O3_MR SO2 SO2_MR/],
      '--relative=0.2' => [qw/
        CO CO_MR H2O H2O_MR Liquid_H2O Liquid_H2O_MR/]);
#   For CrIS_Radiances of FOR (i.e. not FOV) as appear in NUCAPS-CCR-AR files,
#   I needs combination of relative and absolute differences to make a fair
#   comparison.  The valid_range is -5 to 150.  A fair comparison could be
#   --delta=0.001 for < 5 and --relative=0.001 for > 5.  But this takes a lot
#   more coding, and ultimately it is the retrieved geophysical variables that
#   I care about.  So, unless --machineprec is set, I am going to skip this
#   field.  I am also going to skip Rspare_Field, Ispare_Field and Stability
#   since I cannot say what deviation is acceptable for these fields.
#   Again, -m  or --machineprec will bring them all back into play.
    foreach my $field (@variables) {
      ($machine) or do {
        (grep {$field eq $_} qw/Rspare_Field Ispare_Field Stability Cloud_Height
           Cloud_Fraction Cloud_Top_Pressure Cloud_Top_Fraction/) and next;
        (grep {$field eq $_} qw/CrIS_Radiances/) and do {
          (index($user, 'NUCAPS-CCR') < 0) or next;
        };
      };
      my $novar = system("$NCDUMP -v $field $veri_file 1>/dev/null 2>/dev/null");
      ($novar) and next;
      my $prec = $precision;
      foreach my $key (sort keys %prec_lut) {
        #print "key :  $key\n";
        (grep {$field eq $_} @{$prec_lut{$key}}) and $prec = $key;
      }
      ($machine) and $prec = $precision;
      my $cmd = "$H5DIFF -q $prec $files $field";
      #print "$cmd\n";
      my $status = system($cmd);
      $cmd = "$H5DIFF -r $prec $files $field |";
      ($verbose == 3) and print "Field $field\n";
      open(my $fh, $cmd) or die "$MAIN\:: Could not run command $cmd\nAbort";
      while (<$fh>) { chomp;
        /differences found/ or next;
        my @parts = split(/\s+/, $_);
        ($parts[0] == 0) and next;
        $passed = 0;
        print "$_ for $field ;  h5diff return code $status\n";
      }
      close($fh);
    }
    return $passed;
  }

# -------------------
  sub get_commandline
# ===================
# %options = get_commandline($revision, $pwd);
  {
    # If things go bad in here, jump to usage() which terminates with exit(9).
    # Note that I had to hack in the Python-like -vv & -vvv support.
    # This routine assumes standard module Getopt::Long has been loaded.
    # abs_path() is dangerous for the purpose of setting the work (and,
    # equivalently, the output) directory - see this post:
    # http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=257568
    my ($rev, $pwd) = (shift, shift);
    my $name = (caller(0))[3];
    $name =~ s/main/$MAIN/g;
    my $codeset = langinfo(CODESET);
    @ARGV = map{decode $codeset, $_} @ARGV;
    foreach (@ARGV) { s/\?/-\?/g; }
    my %options = ();
    GetOptions(\%options,
      'version|V',
      'help|?',
      'nothing|n',
      'input_dir|i=s',
      'sat|s=s',
      'work|w=s',
      'machineprec|m',
      'verbose|v+', 'vv', 'vvv') or usage();
    defined($options{version}) and die "$rev\n";
    defined($options{help}) and usage();
    $options{nothing} = defined($options{nothing}) ? $options{nothing} : 0;
    defined($options{input_dir}) or usage();
    defined($options{sat}) or usage();
    (grep {$_ eq $options{sat}} qw/npp j01/) or usage();
    (defined($options{work})) or $options{work} = $pwd;
    $options{machineprec} = defined($options{machineprec}) ? $options{machineprec} : 0;
    $options{verbose} = defined($options{verbose}) ? $options{verbose} : 0;
    $options{vv} = defined($options{vv}) ? $options{vv} : 0;
    $options{vvv} = defined($options{vvv}) ? $options{vvv} : 0;
    $options{verbose} = $options{verbose} + 2 * $options{vv};
    $options{verbose} = $options{verbose} + 3 * $options{vvv};
    $options{verbose} = min($options{verbose}, 3);
    delete @options{'vv', 'vvv'};
    return %options;
  }

# ---------
  sub usage
# =========
# usage();
  {
    my $name = (caller(0))[3];
    $name =~ s/main/$MAIN/g;
    open(my $fh, "<$0");
    while (<$fh>) { chomp;
      if ( /START-HELP/ ) {
        while (<$fh>) { chomp;
          ( /END-HELP/ ) and do { close($fh); exit(9); };
          my $help = (split(/\#/, $_))[1]; # remove leading hash
          $help and print "$help\n" or print "\n";
        }
      }
    }
    die "$name\: Unprecedented error!\nAbort";
  }


###############################################################################
#                                                                             #
#       R E F E R E N C E      R E F E R E N C E      R E F E R E N C E       #
#                                                                             #
#      Variable names and their attributes from imager and sounder files      #
#                                                                             #
###############################################################################

###############################################################################
# NUCAPS-EDR   NUCAPS-EDR   NUCAPS-EDR   NUCAPS-EDR   NUCAPS-EDR   NUCAPS-EDR #
###############################################################################
#  variables:
#    char quality_information ;
#      quality_information:long_name = "total number of retrievals, percentage of
#        optimal retrievals, percentage of bad retrievals" ;
#      quality_information:total_number_retrievals = 120 ;
#      quality_information:percentage_optimal_retrievals = 74.16666f ;
#      quality_information:percentage_bad_retrievals = 25.83333f ;
#    int CrIS_FORs(Number_of_CrIS_FORs) ;
#      CrIS_FORs:long_name = "The index of the CrIS FOR" ;
#      CrIS_FORs:parameter_type = "NUCAPS data" ;
#      CrIS_FORs:units = "1" ;
#      CrIS_FORs:valid_range = 1, 120 ;
#      CrIS_FORs:_FillValue = -9999 ;
#    double Time(Number_of_CrIS_FORs) ;
#      Time:long_name = "UTC Milliseconds since Jan 1, 1970" ;
#      Time:standard_name = "time" ;
#      Time:units = "msec since 1970-01-01T00:00:00Z" ;
#      Time:calendar = "gregorian" ;
#      Time:parameter_type = "NUCAPS data" ;
#      Time:valid_range = 950000000000., 2500000000000. ;
#      Time:_FillValue = -9999. ;
#    float Latitude(Number_of_CrIS_FORs) ;
#      Latitude:long_name = "Retrieval latitude values for each CrIS FOR" ;
#      Latitude:standard_name = "latitude" ;
#      Latitude:units = "degrees_north" ;
#      Latitude:parameter_type = "NUCAPS data" ;
#      Latitude:valid_range = -90.f, 90.f ;
#      Latitude:_FillValue = -9999.f ;
#    float Longitude(Number_of_CrIS_FORs) ;
#      Longitude:long_name = "Retrieval longitude values for each CrIS FOR" ;
#      Longitude:standard_name = "longitude" ;
#      Longitude:units = "degrees_east" ;
#      Longitude:parameter_type = "NUCAPS data" ;
#      Longitude:valid_range = -180.f, 180.f ;
#      Longitude:_FillValue = -9999.f ;
#    float View_Angle(Number_of_CrIS_FORs) ;
#      View_Angle:long_name = "View angle for each CrIS FOR" ;
#      View_Angle:units = "degrees" ;
#      View_Angle:parameter_type = "NUCAPS data" ;
#      View_Angle:coordinates = "Time Latitude Longitude" ;
#      View_Angle:valid_range = -60.f, 60.f ;
#      View_Angle:_FillValue = -9999.f ;
#    float Satellite_Height(Number_of_CrIS_FORs) ;
#      Satellite_Height:long_name = "Satellite height above each CrIS FOR" ;
#      Satellite_Height:standard_name = "altitude" ;
#      Satellite_Height:units = "km" ;
#      Satellite_Height:parameter_type = "NUCAPS data" ;
#      Satellite_Height:coordinates = "Time Latitude Longitude" ;
#      Satellite_Height:valid_range = 800.f, 900.f ;
#      Satellite_Height:_FillValue = -9999.f ;
#    float FG_Mean_CO2(Number_of_CrIS_FORs) ;
#      FG_Mean_CO2:long_name = "First guess column averaged CO2 (ppm) per CrIS FOR" ;
#      FG_Mean_CO2:standard_name = "mole_fraction_of_carbon_dioxide_in_air" ;
#      FG_Mean_CO2:units = "1" ;
#      FG_Mean_CO2:parameter_type = "NUCAPS data" ;
#      FG_Mean_CO2:coordinates = "Time Latitude Longitude" ;
#      FG_Mean_CO2:valid_range = 0.f, 1000.f ;
#      FG_Mean_CO2:_FillValue = -9999.f ;
#    float Mean_CO2(Number_of_CrIS_FORs) ;
#      Mean_CO2:long_name = "Column averaged CO2 (ppm) per CrIS FOR" ;
#      Mean_CO2:standard_name = "mole_fraction_of_carbon_dioxide_in_air" ;
#      Mean_CO2:units = "1" ;
#      Mean_CO2:parameter_type = "NUCAPS data" ;
#      Mean_CO2:coordinates = "Time Latitude Longitude" ;
#      Mean_CO2:valid_range = 0.f, 1000.f ;
#      Mean_CO2:_FillValue = -9999.f ;
#    float Solar_Zenith(Number_of_CrIS_FORs) ;
#      Solar_Zenith:long_name = "Solar zenith angles for each CrIS FOR" ;
#      Solar_Zenith:standard_name = "solar_zenith_angle" ;
#      Solar_Zenith:units = "degrees" ;
#      Solar_Zenith:parameter_type = "NUCAPS data" ;
#      Solar_Zenith:coordinates = "Time Latitude Longitude" ;
#      Solar_Zenith:valid_range = 0.f, 180.f ;
#      Solar_Zenith:_FillValue = -9999.f ;
#    short Ascending_Descending(Number_of_CrIS_FORs) ;
#      Ascending_Descending:long_name = "1=Descending, 0=Ascending" ;
#      Ascending_Descending:parameter_type = "NUCAPS data" ;
#      Ascending_Descending:units = "1" ;
#      Ascending_Descending:coordinates = "Time Latitude Longitude" ;
#      Ascending_Descending:valid_range = 0s, 1s ;
#      Ascending_Descending:_FillValue = -9999s ;
#    float Topography(Number_of_CrIS_FORs) ;
#      Topography:long_name = "Surface height" ;
#      Topography:units = "meters" ;
#      Topography:parameter_type = "NUCAPS data" ;
#      Topography:coordinates = "Time Latitude Longitude" ;
#      Topography:valid_range = 0.f, 10000.f ;
#      Topography:_FillValue = -9999.f ;
#    float Land_Fraction(Number_of_CrIS_FORs) ;
#      Land_Fraction:long_name = "Land fraction" ;
#      Land_Fraction:parameter_type = "NUCAPS data" ;
#      Land_Fraction:units = "1" ;
#      Land_Fraction:coordinates = "Time Latitude Longitude" ;
#      Land_Fraction:valid_range = 0.f, 1.f ;
#      Land_Fraction:_FillValue = -9999.f ;
#    float Surface_Pressure(Number_of_CrIS_FORs) ;
#      Surface_Pressure:long_name = "Surface pressure" ;
#      Surface_Pressure:units = "mb" ;
#      Surface_Pressure:parameter_type = "NUCAPS data" ;
#      Surface_Pressure:coordinates = "Time Latitude Longitude" ;
#      Surface_Pressure:valid_range = 0.f, 10000.f ;
#      Surface_Pressure:_FillValue = -9999.f ;
#    float Skin_Temperature(Number_of_CrIS_FORs) ;
#      Skin_Temperature:long_name = "Skin temperature" ;
#      Skin_Temperature:standard_name = "surface_temperature" ;
#      Skin_Temperature:units = "Kelvin" ;
#      Skin_Temperature:parameter_type = "NUCAPS data" ;
#      Skin_Temperature:coordinates = "Time Latitude Longitude" ;
#      Skin_Temperature:valid_range = 0.f, 1000.f ;
#      Skin_Temperature:_FillValue = -9999.f ;
#    float MIT_Skin_Temperature(Number_of_CrIS_FORs) ;
#      MIT_Skin_Temperature:long_name = "Skin temperature from MIT retrieval" ;
#      MIT_Skin_Temperature:standard_name = "surface_temperature" ;
#      MIT_Skin_Temperature:units = "Kelvin" ;
#      MIT_Skin_Temperature:parameter_type = "NUCAPS data" ;
#      MIT_Skin_Temperature:coordinates = "Time Latitude Longitude" ;
#      MIT_Skin_Temperature:valid_range = 0.f, 1000.f ;
#      MIT_Skin_Temperature:_FillValue = -9999.f ;
#    float FG_Skin_Temperature(Number_of_CrIS_FORs) ;
#      FG_Skin_Temperature:long_name = "Skin temperature from the first guess" ;
#      FG_Skin_Temperature:standard_name = "surface_temperature" ;
#      FG_Skin_Temperature:units = "Kelvin" ;
#      FG_Skin_Temperature:parameter_type = "NUCAPS data" ;
#      FG_Skin_Temperature:coordinates = "Time Latitude Longitude" ;
#      FG_Skin_Temperature:valid_range = 0.f, 1000.f ;
#      FG_Skin_Temperature:_FillValue = -9999.f ;
#    short MW_Surface_Class(Number_of_CrIS_FORs) ;
#      MW_Surface_Class:long_name = "Microwave surface class" ;
#      MW_Surface_Class:parameter_type = "NUCAPS data" ;
#      MW_Surface_Class:units = "1" ;
#      MW_Surface_Class:coordinates = "Time Latitude Longitude" ;
#      MW_Surface_Class:valid_range = 0s, 10s ;
#      MW_Surface_Class:_FillValue = -9999s ;
#    float MW_Surface_Emis(Number_of_CrIS_FORs) ;
#      MW_Surface_Emis:long_name = "Microwave surface emissivity" ;
#      MW_Surface_Emis:standard_name = "surface_microwave_emissivity" ;
#      MW_Surface_Emis:parameter_type = "NUCAPS data" ;
#      MW_Surface_Emis:units = "1" ;
#      MW_Surface_Emis:coordinates = "Time Latitude Longitude" ;
#      MW_Surface_Emis:valid_range = 0.f, 1.f ;
#      MW_Surface_Emis:_FillValue = -9999.f ;
#    int N_Smw_Per_FOV(Number_of_CrIS_FORs) ;
#      N_Smw_Per_FOV:long_name = "Number of MW spectral points CrIS FOR" ;
#      N_Smw_Per_FOV:parameter_type = "NUCAPS data" ;
#      N_Smw_Per_FOV:units = "1" ;
#      N_Smw_Per_FOV:coordinates = "Time Latitude Longitude" ;
#      N_Smw_Per_FOV:valid_range = 1, 16 ;
#      N_Smw_Per_FOV:_FillValue = -9999 ;
#    int nemis_Per_FOV(Number_of_CrIS_FORs) ;
#      nemis_Per_FOV:long_name = "Number of surface emis hinge points per CrIS FOR" ;
#      nemis_Per_FOV:parameter_type = "NUCAPS data" ;
#      nemis_Per_FOV:units = "1" ;
#      nemis_Per_FOV:coordinates = "Time Latitude Longitude" ;
#      nemis_Per_FOV:valid_range = 1, 100 ;
#      nemis_Per_FOV:_FillValue = -9999 ;
#    int ncemis_Per_FOV(Number_of_CrIS_FORs) ;
#      ncemis_Per_FOV:long_name = "Number of cloud emis hinge points per CrIS FOR" ;
#      ncemis_Per_FOV:parameter_type = "NUCAPS data" ;
#      ncemis_Per_FOV:units = "1" ;
#      ncemis_Per_FOV:coordinates = "Time Latitude Longitude" ;
#      ncemis_Per_FOV:valid_range = 1, 100 ;
#      ncemis_Per_FOV:_FillValue = -9999 ;
#    int ncld_Per_FOV(Number_of_CrIS_FORs) ;
#      ncld_Per_FOV:long_name = "Number of cloud layers per CrIS FOR" ;
#      ncld_Per_FOV:parameter_type = "NUCAPS data" ;
#      ncld_Per_FOV:units = "1" ;
#      ncld_Per_FOV:coordinates = "Time Latitude Longitude" ;
#      ncld_Per_FOV:valid_range = 1, 8 ;
#      ncld_Per_FOV:_FillValue = -9999 ;
#    int Quality_Flag(Number_of_CrIS_FORs) ;
#      Quality_Flag:long_name = "Quality flags for retrieval" ;
#      Quality_Flag:parameter_type = "NUCAPS data" ;
#      Quality_Flag:coordinates = "Time Latitude Longitude" ;
#      Quality_Flag:valid_range = 0, 31 ;
#      Quality_Flag:_FillValue = -9999 ;
#      Quality_Flag:flag_values = 0, 1, 2, 4, 8, 9, 16, 17, 24, 25 ;
#      Quality_Flag:flag_meanings = "accepted reject_physical reject_MIT
#        reject_NOAA_reg reject_iMIT reject_phy_and_iMIT reject_iNOAA
#        reject_phy_and_iNOAA reject_iMIT_and_iNOAA reject_phy_and_iMIT_and_iNOAA" ;
#    int Ispare_Field(Number_of_CrIS_FORs, Number_of_Ispares) ;
#      Ispare_Field:long_name = "Ispare diagnostics array from retrieval" ;
#      Ispare_Field:parameter_type = "NUCAPS data" ;
#      Ispare_Field:units = "1" ;
#      Ispare_Field:coordinates = "Time Latitude Longitude" ;
#      Ispare_Field:valid_range = 0, 10000 ;
#      Ispare_Field:_FillValue = -9999 ;
#    float Rspare_Field(Number_of_CrIS_FORs, Number_of_Rspares) ;
#      Rspare_Field:long_name = "Rspare diagnostics array from retrieval" ;
#      Rspare_Field:parameter_type = "NUCAPS data" ;
#      Rspare_Field:units = "1" ;
#      Rspare_Field:coordinates = "Time Latitude Longitude" ;
#      Rspare_Field:valid_range = 0.f, 10000.f ;
#      Rspare_Field:_FillValue = -9999.f ;
#    float Cloud_Top_Pressure(Number_of_CrIS_FORs, Number_of_Cloud_Layers) ;
#      Cloud_Top_Pressure:long_name = "Cloud top pressure" ;
#      Cloud_Top_Pressure:units = "mb" ;
#      Cloud_Top_Pressure:parameter_type = "NUCAPS data" ;
#      Cloud_Top_Pressure:coordinates = "Time Latitude Longitude" ;
#      Cloud_Top_Pressure:valid_range = 0.f, 10000.f ;
#      Cloud_Top_Pressure:_FillValue = -9999.f ;
#    float Cloud_Top_Fraction(Number_of_CrIS_FORs, Number_of_Cloud_Layers) ;
#      Cloud_Top_Fraction:long_name = "Cloud top fractional coverage" ;
#      Cloud_Top_Fraction:parameter_type = "NUCAPS data" ;
#      Cloud_Top_Fraction:units = "1" ;
#      Cloud_Top_Fraction:coordinates = "Time Latitude Longitude" ;
#      Cloud_Top_Fraction:valid_range = 0.f, 1.f ;
#      Cloud_Top_Fraction:_FillValue = -9999.f ;
#    float Pressure(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      Pressure:long_name = "Pressure" ;
#      Pressure:units = "mb" ;
#      Pressure:parameter_type = "NUCAPS data" ;
#      Pressure:coordinates = "Time Latitude Longitude" ;
#      Pressure:valid_range = 0.f, 2000.f ;
#      Pressure:_FillValue = -9999.f ;
#    float Effective_Pressure(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      Effective_Pressure:long_name = "Effective_Pressure" ;
#      Effective_Pressure:units = "mb" ;
#      Effective_Pressure:parameter_type = "NUCAPS data" ;
#      Effective_Pressure:coordinates = "Time Latitude Longitude" ;
#      Effective_Pressure:valid_range = 0.f, 2000.f ;
#      Effective_Pressure:_FillValue = -9999.f ;
#    float Temperature(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      Temperature:long_name = "Temperature" ;
#      Temperature:standard_name = "air_temperature" ;
#      Temperature:units = "Kelvin" ;
#      Temperature:parameter_type = "NUCAPS data" ;
#      Temperature:coordinates = "Time Latitude Longitude Pressure" ;
#      Temperature:valid_range = 0.f, 1000.f ;
#      Temperature:_FillValue = -9999.f ;
#    float MIT_Temperature(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      MIT_Temperature:long_name = "Temperature from MIT retrieval" ;
#      MIT_Temperature:standard_name = "air_temperature" ;
#      MIT_Temperature:units = "Kelvin" ;
#      MIT_Temperature:parameter_type = "NUCAPS data" ;
#      MIT_Temperature:coordinates = "Time Latitude Longitude Pressure" ;
#      MIT_Temperature:valid_range = 0.f, 1000.f ;
#      MIT_Temperature:_FillValue = -9999.f ;
#    float FG_Temperature(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      FG_Temperature:long_name = "Temperature from the first guess" ;
#      FG_Temperature:standard_name = "air_temperature" ;
#      FG_Temperature:units = "Kelvin" ;
#      FG_Temperature:parameter_type = "NUCAPS data" ;
#      FG_Temperature:coordinates = "Time Latitude Longitude Pressure" ;
#      FG_Temperature:valid_range = 0.f, 1000.f ;
#      FG_Temperature:_FillValue = -9999.f ;
#    float H2O(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      H2O:long_name = "water vapor layer column density (molecules/cm2)" ;
#      H2O:units = "1" ;
#      H2O:parameter_type = "NUCAPS data" ;
#      H2O:coordinates = "Time Latitude Longitude Pressure" ;
#      H2O:valid_range = 0.f, 1.e+08f ;
#      H2O:_FillValue = -9999.f ;
#    float MIT_H2O(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      MIT_H2O:long_name = "water vapor layer column density from MIT retrieval (molecules/cm2)" ;
#      MIT_H2O:units = "1" ;
#      MIT_H2O:parameter_type = "NUCAPS data" ;
#      MIT_H2O:coordinates = "Time Latitude Longitude Pressure" ;
#      MIT_H2O:valid_range = 0.f, 1.e+08f ;
#      MIT_H2O:_FillValue = -9999.f ;
#    float FG_H2O(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      FG_H2O:long_name = "water vapor layer column density from the first guess (molecules/cm2)" ;
#      FG_H2O:units = "1" ;
#      FG_H2O:parameter_type = "NUCAPS data" ;
#      FG_H2O:coordinates = "Time Latitude Longitude Pressure" ;
#      FG_H2O:valid_range = 0.f, 1.e+08f ;
#      FG_H2O:_FillValue = -9999.f ;
#    float H2O_MR(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      H2O_MR:long_name = "water vapor mixing ratio" ;
#      H2O_MR:units = "kg/kg" ;
#      H2O_MR:parameter_type = "NUCAPS data" ;
#      H2O_MR:coordinates = "Time Latitude Longitude Effective_Pressure" ;
#      H2O_MR:valid_range = 0.f, 1.e+08f ;
#      H2O_MR:_FillValue = -9999.f ;
#    float MIT_H2O_MR(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      MIT_H2O_MR:long_name = "water vapor mixing ratio from MIT retrieval" ;
#      MIT_H2O_MR:units = "kg/kg" ;
#      MIT_H2O_MR:parameter_type = "NUCAPS data" ;
#      MIT_H2O_MR:coordinates = "Time Latitude Longitude Effective_Pressure" ;
#      MIT_H2O_MR:valid_range = 0.f, 1.e+08f ;
#      MIT_H2O_MR:_FillValue = -9999.f ;
#    float FG_H2O_MR(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      FG_H2O_MR:long_name = "water vapor mixing ratio from the first guess" ;
#      FG_H2O_MR:units = "kg/kg" ;
#      FG_H2O_MR:parameter_type = "NUCAPS data" ;
#      FG_H2O_MR:coordinates = "Time Latitude Longitude Effective_Pressure" ;
#      FG_H2O_MR:valid_range = 0.f, 1.e+08f ;
#      FG_H2O_MR:_FillValue = -9999.f ;
#    float O3(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      O3:long_name = "Ozone layer column density (molecules/cm2)" ;
#      O3:units = "1" ;
#      O3:parameter_type = "NUCAPS data" ;
#      O3:coordinates = "Time Latitude Longitude Pressure" ;
#      O3:valid_range = 0.f, 1.e+08f ;
#      O3:_FillValue = -9999.f ;
#    float FG_O3(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      FG_O3:long_name = "Ozone layer column density from first guess (molecules/cm2)" ;
#      FG_O3:units = "1" ;
#      FG_O3:parameter_type = "NUCAPS data" ;
#      FG_O3:coordinates = "Time Latitude Longitude Pressure" ;
#      FG_O3:valid_range = 0.f, 1.e+08f ;
#      FG_O3:_FillValue = -9999.f ;
#    float O3_MR(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      O3_MR:long_name = "Ozone mixing ratio (ppbv)" ;
#      O3_MR:units = "1" ;
#      O3_MR:parameter_type = "NUCAPS data" ;
#      O3_MR:coordinates = "Time Latitude Longitude Effective_Pressure" ;
#      O3_MR:valid_range = 0.f, 1.e+08f ;
#      O3_MR:_FillValue = -9999.f ;
#    float FG_O3_MR(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      FG_O3_MR:long_name = "Ozone mixing ratio from first guess (ppbv)" ;
#      FG_O3_MR:units = "1" ;
#      FG_O3_MR:parameter_type = "NUCAPS data" ;
#      FG_O3_MR:coordinates = "Time Latitude Longitude Effective_Pressure" ;
#      FG_O3_MR:valid_range = 0.f, 1.e+08f ;
#      FG_O3_MR:_FillValue = -9999.f ;
#    float Liquid_H2O(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      Liquid_H2O:long_name = "Liquid water layer column density (molecules/cm2)" ;
#      Liquid_H2O:units = "1" ;
#      Liquid_H2O:parameter_type = "NUCAPS data" ;
#      Liquid_H2O:coordinates = "Time Latitude Longitude Pressure" ;
#      Liquid_H2O:valid_range = 0.f, 1.e+08f ;
#      Liquid_H2O:_FillValue = -9999.f ;
#    float Liquid_H2O_MR(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      Liquid_H2O_MR:long_name = "Liquid water mixing ratio" ;
#      Liquid_H2O_MR:standard_name = "cloud_liquid_water_mixing_ratio" ;
#      Liquid_H2O_MR:units = "kg/kg" ;
#      Liquid_H2O_MR:parameter_type = "NUCAPS data" ;
#      Liquid_H2O_MR:coordinates = "Time Latitude Longitude Effective_Pressure" ;
#      Liquid_H2O_MR:valid_range = 0.f, 1.e+08f ;
#      Liquid_H2O_MR:_FillValue = -9999.f ;
#    short Ice_Liquid_Flag(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      Ice_Liquid_Flag:long_name = "Ice liquid flag 0=water, 1=ice" ;
#      Ice_Liquid_Flag:parameter_type = "NUCAPS data" ;
#      Ice_Liquid_Flag:units = "1" ;
#      Ice_Liquid_Flag:coordinates = "Time Latitude Longitude Pressure" ;
#      Ice_Liquid_Flag:valid_range = 0s, 1s ;
#      Ice_Liquid_Flag:_FillValue = -9999s ;
#    float CO(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      CO:long_name = "Carbon monoxide layer column density (molecules/cm2)" ;
#      CO:units = "1" ;
#      CO:parameter_type = "NUCAPS data" ;
#      CO:coordinates = "Time Latitude Longitude Pressure" ;
#      CO:valid_range = 0.f, 1.e+08f ;
#      CO:_FillValue = -9999.f ;
#    float CO_MR(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      CO_MR:long_name = "Carbon monoxide mixing ratio (ppbv)" ;
#      CO_MR:units = "1" ;
#      CO_MR:parameter_type = "NUCAPS data" ;
#      CO_MR:coordinates = "Time Latitude Longitude Effective_Pressure" ;
#      CO_MR:valid_range = 0.f, 1.e+08f ;
#      CO_MR:_FillValue = -9999.f ;
#    float CH4(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      CH4:long_name = "Methane layer column density (molecules/cm2)" ;
#      CH4:units = "1" ;
#      CH4:parameter_type = "NUCAPS data" ;
#      CH4:coordinates = "Time Latitude Longitude Pressure" ;
#      CH4:valid_range = 0.f, 1.e+08f ;
#      CH4:_FillValue = -9999.f ;
#    float CH4_MR(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      CH4_MR:long_name = "Methane mixing ratio (ppbv)" ;
#      CH4_MR:units = "1" ;
#      CH4_MR:parameter_type = "NUCAPS data" ;
#      CH4_MR:coordinates = "Time Latitude Longitude Effective_Pressure" ;
#      CH4_MR:valid_range = 0.f, 1.e+08f ;
#      CH4_MR:_FillValue = -9999.f ;
#    float CO2(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      CO2:long_name = "Carbon dioxide dry mixing ratio (ppm)" ;
#      CO2:units = "1" ;
#      CO2:parameter_type = "NUCAPS data" ;
#      CO2:coordinates = "Time Latitude Longitude Pressure" ;
#      CO2:valid_range = 0.f, 1000.f ;
#      CO2:_FillValue = -9999.f ;
#    float HNO3(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      HNO3:long_name = "Nitric Acid layer column density (molecules/cm2)" ;
#      HNO3:units = "1" ;
#      HNO3:parameter_type = "NUCAPS data" ;
#      HNO3:coordinates = "Time Latitude Longitude Pressure" ;
#      HNO3:valid_range = 0.f, 1.e+08f ;
#      HNO3:_FillValue = -9999.f ;
#    float HNO3_MR(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      HNO3_MR:long_name = "Nitric Acid mixing ratio (ppbv)" ;
#      HNO3_MR:units = "1" ;
#      HNO3_MR:parameter_type = "NUCAPS data" ;
#      HNO3_MR:coordinates = "Time Latitude Longitude Effective_Pressure" ;
#      HNO3_MR:valid_range = 0.f, 1.e+08f ;
#      HNO3_MR:_FillValue = -9999.f ;
#    float N2O(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      N2O:long_name = "Nitrous Oxide layer column density (molecules/cm2)" ;
#      N2O:units = "1" ;
#      N2O:parameter_type = "NUCAPS data" ;
#      N2O:coordinates = "Time Latitude Longitude Pressure" ;
#      N2O:valid_range = 0.f, 1.e+08f ;
#      N2O:_FillValue = -9999.f ;
#    float N2O_MR(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      N2O_MR:long_name = "Nitrous Oxide mixing ratio (ppbv)" ;
#      N2O_MR:units = "1" ;
#      N2O_MR:parameter_type = "NUCAPS data" ;
#      N2O_MR:coordinates = "Time Latitude Longitude Effective_Pressure" ;
#      N2O_MR:valid_range = 0.f, 1.e+08f ;
#      N2O_MR:_FillValue = -9999.f ;
#    float SO2(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      SO2:long_name = "Sulfur Dioxide layer column density (molecules/cm2)" ;
#      SO2:units = "1" ;
#      SO2:parameter_type = "NUCAPS data" ;
#      SO2:coordinates = "Time Latitude Longitude Pressure" ;
#      SO2:valid_range = 0.f, 1.e+08f ;
#      SO2:_FillValue = -9999.f ;
#    float SO2_MR(Number_of_CrIS_FORs, Number_of_P_Levels) ;
#      SO2_MR:long_name = "Sulfur Dioxide mixing ratio (ppbv)" ;
#      SO2_MR:units = "1" ;
#      SO2_MR:parameter_type = "NUCAPS data" ;
#      SO2_MR:coordinates = "Time Latitude Longitude Effective_Pressure" ;
#      SO2_MR:valid_range = 0.f, 1.e+08f ;
#      SO2_MR:_FillValue = -9999.f ;
#    float MW_Frequency(Number_of_CrIS_FORs, Number_of_MW_Spectral_Pts) ;
#      MW_Frequency:long_name = "Microwave frequency" ;
#      MW_Frequency:units = "cm-1" ;
#      MW_Frequency:parameter_type = "NUCAPS data" ;
#      MW_Frequency:coordinates = "Time Latitude Longitude" ;
#      MW_Frequency:valid_range = 0.f, 10000.f ;
#      MW_Frequency:_FillValue = -9999.f ;
#    float MW_Emis(Number_of_CrIS_FORs, Number_of_MW_Spectral_Pts) ;
#      MW_Emis:long_name = "Microwave emissivity" ;
#      MW_Emis:parameter_type = "NUCAPS data" ;
#      MW_Emis:units = "1" ;
#      MW_Emis:coordinates = "Time Latitude Longitude" ;
#      MW_Emis:valid_range = 0.f, 1.f ;
#      MW_Emis:_FillValue = -9999.f ;
#    float MIT_MW_Emis(Number_of_CrIS_FORs, Number_of_MW_Spectral_Pts) ;
#      MIT_MW_Emis:long_name = "Microwave emissivity from MIT retrieval" ;
#      MIT_MW_Emis:parameter_type = "NUCAPS data" ;
#      MIT_MW_Emis:units = "1" ;
#      MIT_MW_Emis:coordinates = "Time Latitude Longitude" ;
#      MIT_MW_Emis:valid_range = 0.f, 1.f ;
#      MIT_MW_Emis:_FillValue = -9999.f ;
#    float IR_Emis_Freq(Number_of_CrIS_FORs, Number_of_Surf_Emis_Hinge_Pts) ;
#      IR_Emis_Freq:long_name = "IR emissivity hinge point frequencies" ;
#      IR_Emis_Freq:units = "cm-1" ;
#      IR_Emis_Freq:parameter_type = "NUCAPS data" ;
#      IR_Emis_Freq:coordinates = "Time Latitude Longitude" ;
#      IR_Emis_Freq:valid_range = 0.f, 10000.f ;
#      IR_Emis_Freq:_FillValue = -9999.f ;
#    float FG_IR_Emis_Freq(Number_of_CrIS_FORs, Number_of_Surf_Emis_Hinge_Pts) ;
#      FG_IR_Emis_Freq:long_name = "IR emissivity hinge point frequencies from the first guess" ;
#      FG_IR_Emis_Freq:units = "cm-1" ;
#      FG_IR_Emis_Freq:parameter_type = "NUCAPS data" ;
#      FG_IR_Emis_Freq:coordinates = "Time Latitude Longitude" ;
#      FG_IR_Emis_Freq:valid_range = 0.f, 10000.f ;
#      FG_IR_Emis_Freq:_FillValue = -9999.f ;
#    float IR_Surface_Emis(Number_of_CrIS_FORs, Number_of_Surf_Emis_Hinge_Pts) ;
#      IR_Surface_Emis:long_name = "IR surface emissivity" ;
#      IR_Surface_Emis:parameter_type = "NUCAPS data" ;
#      IR_Surface_Emis:units = "1" ;
#      IR_Surface_Emis:coordinates = "Time Latitude Longitude" ;
#      IR_Surface_Emis:valid_range = 0.f, 1.f ;
#      IR_Surface_Emis:_FillValue = -9999.f ;
#    float FG_IR_Surface_Emis(Number_of_CrIS_FORs, Number_of_Surf_Emis_Hinge_Pts) ;
#      FG_IR_Surface_Emis:long_name = "IR surface emissivity from the first guess" ;
#      FG_IR_Surface_Emis:parameter_type = "NUCAPS data" ;
#      FG_IR_Surface_Emis:units = "1" ;
#      FG_IR_Surface_Emis:coordinates = "Time Latitude Longitude" ;
#      FG_IR_Surface_Emis:valid_range = 0.f, 1.f ;
#      FG_IR_Surface_Emis:_FillValue = -9999.f ;
#    float IR_Surface_Refl(Number_of_CrIS_FORs, Number_of_Surf_Emis_Hinge_Pts) ;
#      IR_Surface_Refl:long_name = "IR surface reflectance" ;
#      IR_Surface_Refl:units = "1" ;
#      IR_Surface_Refl:parameter_type = "NUCAPS data" ;
#      IR_Surface_Refl:coordinates = "Time Latitude Longitude" ;
#      IR_Surface_Refl:valid_range = 0.f, 100.f ;
#      IR_Surface_Refl:_FillValue = -9999.f ;
#    float Stability(Number_of_CrIS_FORs, Number_of_Stability_Parameters) ;
#      Stability:long_name = "Stability parameters" ;
#      Stability:units = "1" ;
#      Stability:parameter_type = "NUCAPS data" ;
#      Stability:coordinates = "Time Latitude Longitude" ;
#      Stability:valid_range = 0.f, 1000000.f ;
#      Stability:_FillValue = -9999.f ;
#    float Cloud_Freq(Number_of_CrIS_FORs, Number_of_Cloud_Layers, Number_of_Cloud_Emis_Hing_Pts) ;
#      Cloud_Freq:long_name = "Cloud IR frequencies" ;
#      Cloud_Freq:units = "cm-1" ;
#      Cloud_Freq:parameter_type = "NUCAPS data" ;
#      Cloud_Freq:coordinates = "Time Latitude Longitude" ;
#      Cloud_Freq:valid_range = 0.f, 10000.f ;
#      Cloud_Freq:_FillValue = -9999.f ;
#    float Cloud_Emis(Number_of_CrIS_FORs, Number_of_Cloud_Layers, Number_of_Cloud_Emis_Hing_Pts) ;
#      Cloud_Emis:long_name = "Cloud IR emissivity" ;
#      Cloud_Emis:parameter_type = "NUCAPS data" ;
#      Cloud_Emis:units = "1" ;
#      Cloud_Emis:coordinates = "Time Latitude Longitude" ;
#      Cloud_Emis:valid_range = 0.f, 1.f ;
#      Cloud_Emis:_FillValue = -9999.f ;
#    float Cloud_Refl(Number_of_CrIS_FORs, Number_of_Cloud_Layers, Number_of_Cloud_Emis_Hing_Pts) ;
#      Cloud_Refl:long_name = "Cloud IR reflectivity" ;
#      Cloud_Refl:units = "1" ;
#      Cloud_Refl:parameter_type = "NUCAPS data" ;
#      Cloud_Refl:coordinates = "Time Latitude Longitude" ;
#      Cloud_Refl:valid_range = 0.f, 100.f ;
#      Cloud_Refl:_FillValue = -9999.f ;

###############################################################################
# NUCAPS-CCR   NUCAPS-CCR   NUCAPS-CCR   NUCAPS-CCR   NUCAPS-CCR   NUCAPS-CCR #
###############################################################################
#  variables:
#    char quality_information ;
#      quality_information:long_name = "total number of retrievals, percentage of
#        optimal retrievals, percentage of bad retrievals" ;
#      quality_information:total_number_retrievals = 120 ;
#      quality_information:percentage_optimal_retrievals = 74.16666f ;
#      quality_information:percentage_bad_retrievals = 25.83333f ;
#    float CrIS_Frequencies(Number_of_CrIS_Frequencies) ;
#      CrIS_Frequencies:long_name = "Frequency at which the radiances are observed" ;
#      CrIS_Frequencies:units = "cm-1" ;
#      CrIS_Frequencies:parameter_type = "CrIS data" ;
#      CrIS_Frequencies:valid_range = 0.f, 10000.f ;
#      CrIS_Frequencies:_FillValue = -9999.f ;
#    int Scan_Line(Number_of_CrIS_FORs) ;
#      Scan_Line:long_name = "The number of the current scan line in the granule" ;
#      Scan_Line:parameter_type = "CrIS data" ;
#      Scan_Line:units = "1" ;
#      Scan_Line:coordinates = "Time CrIS_Latitude CrIS_Longitude" ;
#      Scan_Line:valid_range = 1, 4 ;
#      Scan_Line:_FillValue = -9999 ;
#    int CrIS_FORs(Number_of_CrIS_FORs) ;
#      CrIS_FORs:long_name = "The number of the current CrIS FOR for the current FOR" ;
#      CrIS_FORs:parameter_type = "CrIS data" ;
#      CrIS_FORs:units = "1" ;
#      CrIS_FORs:coordinates = "Time CrIS_Latitude CrIS_Longitude" ;
#      CrIS_FORs:valid_range = 1, 120 ;
#      CrIS_FORs:_FillValue = -9999 ;
#    byte Quality_Flag(Number_of_CrIS_FORs) ;
#      Quality_Flag:long_name = "CrIS quality flag number 1" ;
#      Quality_Flag:parameter_type = "CrIS data" ;
#      Quality_Flag:coordinates = "Time CrIS_Latitude CrIS_Longitude" ;
#      Quality_Flag:valid_range = 0b, 31b ;
#      Quality_Flag:_FillValue = -128b ;
#      Quality_Flag:flag_values = 0b, 1b, 2b, 4b, 8b, 9b, 16b, 17b, 24b, 25b ;
#      Quality_Flag:flag_meanings = "accepted reject_physical reject_MIT
#        reject_NOAA_reg reject_iMIT reject_phy_and_iMIT reject_iNOAA
#        reject_phy_and_iNOAA reject_iMIT_and_iNOAA reject_phy_and_iMIT_and_iNOAA" ;
#    double Time(Number_of_CrIS_FORs) ;
#      Time:long_name = "UTC Milliseconds since Jan 1, 1970" ;
#      Time:standard_name = "time" ;
#      Time:units = "msec since 1970-01-01T00:00:00Z" ;
#      Time:calendar = "gregorian" ;
#      Time:parameter_type = "CrIS data" ;
#      Time:valid_range = 950000000000., 2500000000000. ;
#      Time:_FillValue = -9999. ;
#    float CrIS_Latitude(Number_of_CrIS_FORs) ;
#      CrIS_Latitude:long_name = "CrIS Latitude values for each FOR" ;
#      CrIS_Latitude:standard_name = "latitude" ;
#      CrIS_Latitude:units = "degrees_north" ;
#      CrIS_Latitude:parameter_type = "CrIS data" ;
#      CrIS_Latitude:valid_range = -90.f, 90.f ;
#      CrIS_Latitude:_FillValue = -9999.f ;
#    float CrIS_Longitude(Number_of_CrIS_FORs) ;
#      CrIS_Longitude:long_name = "CrIS Longitude values for each FOR" ;
#      CrIS_Longitude:standard_name = "longitude" ;
#      CrIS_Longitude:units = "degrees_east" ;
#      CrIS_Longitude:parameter_type = "CrIS data" ;
#      CrIS_Longitude:valid_range = -180.f, 180.f ;
#      CrIS_Longitude:_FillValue = -9999.f ;
#    float CrIS_Radiances(Number_of_CrIS_FORs, Number_of_CrIS_Frequencies) ;
#      CrIS_Radiances:long_name = "CrIS Radiances for each FOR" ;
#      CrIS_Radiances:standard_name = "toa_outgoing_radiance_per_unit_wavenumber" ;
#      CrIS_Radiances:units = "mW/(m2 sr cm-1)" ;
#      CrIS_Radiances:parameter_type = "CrIS data" ;
#      CrIS_Radiances:coordinates = "Time CrIS_Latitude CrIS_Longitude CrIS_Frequencies" ;
#      CrIS_Radiances:valid_range = -5.f, 150.f ;
#      CrIS_Radiances:_FillValue = -9999.f ;
#    float CrIS_View_Angle(Number_of_CrIS_FORs) ;
#      CrIS_View_Angle:long_name = "CrIS View Angles for each FOR" ;
#      CrIS_View_Angle:units = "degrees" ;
#      CrIS_View_Angle:parameter_type = "CrIS data" ;
#      CrIS_View_Angle:coordinates = "Time CrIS_Latitude CrIS_Longitude" ;
#      CrIS_View_Angle:valid_range = -60.f, 60.f ;
#      CrIS_View_Angle:_FillValue = -9999.f ;
#    float Satellite_Height(Number_of_CrIS_FORs) ;
#      Satellite_Height:long_name = "Satellite height above each CrIS FOR" ;
#      Satellite_Height:standard_name = "altitude" ;
#      Satellite_Height:units = "km" ;
#      Satellite_Height:parameter_type = "CrIS data" ;
#      Satellite_Height:coordinates = "Time CrIS_Latitude CrIS_Longitude" ;
#      Satellite_Height:valid_range = 800.f, 900.f ;
#      Satellite_Height:_FillValue = -9999.f ;
#    float Solar_Zenith(Number_of_CrIS_FORs) ;
#      Solar_Zenith:long_name = "Solar zenith angles for each CrIS FOR" ;
#      Solar_Zenith:standard_name = "solar_zenith_angle" ;
#      Solar_Zenith:units = "degrees" ;
#      Solar_Zenith:parameter_type = "CrIS data" ;
#      Solar_Zenith:coordinates = "Time CrIS_Latitude CrIS_Longitude" ;
#      Solar_Zenith:valid_range = 0.f, 180.f ;
#      Solar_Zenith:_FillValue = -9999.f ;
#    int Ispare_Field(Number_of_CrIS_FORs, Number_of_Ispares) ;
#      Ispare_Field:long_name = "Ispare diagnostics array from retrieval" ;
#      Ispare_Field:parameter_type = "NUCAPS data" ;
#      Ispare_Field:units = "1" ;
#      Ispare_Field:coordinates = "Time Latitude Longitude" ;
#      Ispare_Field:valid_range = 0, 10000 ;
#      Ispare_Field:_FillValue = -9999 ;
#    float Rspare_Field(Number_of_CrIS_FORs, Number_of_Rspares) ;
#      Rspare_Field:long_name = "Rspare diagnostics array from retrieval" ;
#      Rspare_Field:parameter_type = "NUCAPS data" ;
#      Rspare_Field:units = "1" ;
#      Rspare_Field:coordinates = "Time Latitude Longitude" ;
#      Rspare_Field:valid_range = 0.f, 10000.f ;
#      Rspare_Field:_FillValue = -9999.f ;
#    short Ascending_Descending(Number_of_CrIS_FORs) ;
#      Ascending_Descending:long_name = "Orbital status for each CrIS FOR, 1=Descending, 0=Ascending" ;
#      Ascending_Descending:parameter_type = "CrIS data" ;
#      Ascending_Descending:units = "1" ;
#      Ascending_Descending:coordinates = "Time CrIS_Latitude CrIS_Longitude" ;
#      Ascending_Descending:valid_range = 0s, 1s ;
#      Ascending_Descending:_FillValue = -9999s ;

###############################################################################
#  NUCAPS-ALL-HR  NUCAPS-ALL-HR  NUCAPS-ALL-HR  NUCAPS-ALL-HR  NUCAPS-ALL-HR  #
###############################################################################
#  variables:
#    int CrIS_Channels(Number_of_CrIS_Channels) ;
#      CrIS_Channels:long_name = "Channel number for the radiance data" ;
#      CrIS_Channels:units = "none" ;
#      CrIS_Channels:scale_factor = 1. ;
#      CrIS_Channels:add_offset = 0. ;
#      CrIS_Channels:parameter_type = "CrIS data" ;
#      CrIS_Channels:valid_range = 1.f, 2223.f ;
#      CrIS_Channels:_FillValue = -9999 ;
#    float CrIS_Frequencies(Number_of_CrIS_Channels) ;
#      CrIS_Frequencies:long_name = "Frequency at which the radiances are observed" ;
#      CrIS_Frequencies:units = "wavenumber cm-1" ;
#      CrIS_Frequencies:scale_factor = 1. ;
#      CrIS_Frequencies:add_offset = 0. ;
#      CrIS_Frequencies:parameter_type = "CrIS data" ;
#      CrIS_Frequencies:valid_range = 0.f, 10000.f ;
#      CrIS_Frequencies:_FillValue = -9999.f ;
#    int ATMS_Channels(Number_of_ATMS_Channels) ;
#      ATMS_Channels:long_name = "Channel number for the radiance data" ;
#      ATMS_Channels:units = "none" ;
#      ATMS_Channels:scale_factor = 1. ;
#      ATMS_Channels:add_offset = 0. ;
#      ATMS_Channels:parameter_type = "ATMS data" ;
#      ATMS_Channels:valid_range = 1.f, 22.f ;
#      ATMS_Channels:_FillValue = -9999 ;
#    float ATMS_Frequencies(Number_of_ATMS_Channels) ;
#      ATMS_Frequencies:long_name = "Frequency at which the data is taken" ;
#      ATMS_Frequencies:units = "GHz" ;
#      ATMS_Frequencies:scale_factor = 1. ;
#      ATMS_Frequencies:add_offset = 0. ;
#      ATMS_Frequencies:parameter_type = "ATMS data" ;
#      ATMS_Frequencies:valid_range = 0.f, 16000.f ;
#      ATMS_Frequencies:_FillValue = -9999.f ;
#    int Subset_CrIS_FORs(Number_of_CrIS_FORs) ;
#      Subset_CrIS_FORs:long_name = "The index of the CrIS fields of regard to subset" ;
#      Subset_CrIS_FORs:units = "none" ;
#      Subset_CrIS_FORs:scale_factor = 1. ;
#      Subset_CrIS_FORs:add_offset = 0. ;
#      Subset_CrIS_FORs:parameter_type = "CrIS data" ;
#      Subset_CrIS_FORs:valid_range = 1.f, 1350.f ;
#      Subset_CrIS_FORs:_FillValue = -9999 ;
#      Subset_CrIS_FORs:coordinates = "Time Latitude Longitude" ;
#    int Subset_CrIS_FOVs(Number_of_CrIS_FOVs) ;
#      Subset_CrIS_FOVs:long_name = "The index of the CrIS fields of view to subset" ;
#      Subset_CrIS_FOVs:units = "none" ;
#      Subset_CrIS_FOVs:scale_factor = 1. ;
#      Subset_CrIS_FOVs:add_offset = 0. ;
#      Subset_CrIS_FOVs:parameter_type = "CrIS data" ;
#      Subset_CrIS_FOVs:valid_range = 1.f, 9.f ;
#      Subset_CrIS_FOVs:_FillValue = -9999 ;
#    int Scan_Line(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Scan_Line:long_name = "The number of the current scan line in the granule" ;
#      Scan_Line:units = "none" ;
#      Scan_Line:scale_factor = 1. ;
#      Scan_Line:add_offset = 0. ;
#      Scan_Line:parameter_type = "CrIS data" ;
#        Scan_Line:coordinates = "Time Latitude Longitude" ;
#      Scan_Line:valid_range = 1.f, 45.f ;
#      Scan_Line:_FillValue = -9999 ;
#    int CrIS_FORs(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_FORs:long_name = "The number of the current CrIS FOR for the current FOV" ;
#      CrIS_FORs:units = "none" ;
#      CrIS_FORs:scale_factor = 1. ;
#      CrIS_FORs:add_offset = 0. ;
#      CrIS_FORs:parameter_type = "CrIS data" ;
#      CrIS_FORs:coordinates = "Time Latitude Longitude" ;
#      CrIS_FORs:valid_range = 1.f, 1350.f ;
#      CrIS_FORs:_FillValue = -9999 ;
#    int CrIS_FOVs(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_FOVs:long_name = "The number of the current CrIS FOV on the field of regard" ;
#      CrIS_FOVs:units = "none" ;
#      CrIS_FOVs:scale_factor = 1. ;
#      CrIS_FOVs:add_offset = 0. ;
#      CrIS_FOVs:parameter_type = "Time Latitude Longitude" ;
#      CrIS_FOVs:valid_range = 1.f, 9.f ;
#      CrIS_FOVs:_FillValue = -9999 ;
#    byte CrIS_QF1(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_QF1:long_name = "CrIS quality flag number 1" ;
#      CrIS_QF1:units = "none" ;
#      CrIS_QF1:scale_factor = 1 ;
#      CrIS_QF1:add_offset = 0 ;
#      CrIS_QF1:parameter_type = "CrIS data" ;
#      CrIS_QF1:coordinates = "Time Latitude Longitude" ;
#      CrIS_QF1:valid_range = 0, 255 ;
#      CrIS_QF1:_FillValue = 95b ;
#    byte CrIS_QF2(Number_of_CrIS_FORs, Number_of_CrIS_FOVs, Number_of_CrIS_Bands) ;
#      CrIS_QF2:long_name = "CrIS quality flag number 2" ;
#      CrIS_QF2:units = "none" ;
#      CrIS_QF2:scale_factor = 1 ;
#      CrIS_QF2:add_offset = 0 ;
#      CrIS_QF2:parameter_type = "CrIS data" ;
#      CrIS_QF2:coordinates = "Time Latitude Longitude" ;
#      CrIS_QF2:valid_range = 0, 255 ;
#      CrIS_QF2:_FillValue = 95b ;
#    byte CrIS_QF3(Number_of_CrIS_FORs, Number_of_CrIS_FOVs, Number_of_CrIS_Bands) ;
#      CrIS_QF3:long_name = "CrIS quality flag number 3" ;
#      CrIS_QF3:units = "none" ;
#      CrIS_QF3:scale_factor = 1 ;
#      CrIS_QF3:add_offset = 0 ;
#      CrIS_QF3:parameter_type = "CrIS data" ;
#      CrIS_QF3:coordinates = "Time Latitude Longitude" ;
#      CrIS_QF3:valid_range = 0, 255 ;
#      CrIS_QF3:_FillValue = 95b ;
#    byte CrIS_QF4(Number_of_CrIS_FORs, Number_of_CrIS_FOVs, Number_of_CrIS_Bands) ;
#      CrIS_QF4:long_name = "CrIS quality flag number 4" ;
#      CrIS_QF4:units = "none" ;
#      CrIS_QF4:scale_factor = 1 ;
#      CrIS_QF4:add_offset = 0 ;
#      CrIS_QF4:parameter_type = "CrIS data" ;
#      CrIS_QF4:coordinates = "Time Latitude Longitude" ;
#      CrIS_QF4:valid_range = 0, 255 ;
#      CrIS_QF4:_FillValue = 95b ;
#    byte CrIS_QF5(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_QF5:long_name = "CrIS quality flag for geolocation" ;
#      CrIS_QF5:units = "none" ;
#      CrIS_QF5:scale_factor = 1 ;
#      CrIS_QF5:add_offset = 0 ;
#      CrIS_QF5:parameter_type = "CrIS data" ;
#      CrIS_QF5:coordinates = "Time Latitude Longitude" ;
#      CrIS_QF5:valid_range = 0, 255 ;
#      CrIS_QF5:_FillValue = 95b ;
#    byte CrIS_QF6(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_QF6:long_name = "CrIS quality flag for CCR" ;
#      CrIS_QF6:units = "none" ;
#      CrIS_QF6:scale_factor = 1 ;
#      CrIS_QF6:add_offset = 0 ;
#      CrIS_QF6:parameter_type = "CrIS data" ;
#      CrIS_QF6:coordinates = "Time Latitude Longitude" ;
#      CrIS_QF6:valid_range = 0, 255 ;
#      CrIS_QF6:_FillValue = 95b ;
#    byte ATMS_QF1(Number_of_CrIS_FORs, Number_of_ATMS_Channels) ;
#      ATMS_QF1:long_name = "ATMS Quality flag number 1" ;
#      ATMS_QF1:units = "none" ;
#      ATMS_QF1:scale_factor = 1 ;
#      ATMS_QF1:add_offset = 0 ;
#      ATMS_QF1:parameter_type = "ATMS data" ;
#      ATMS_QF1:coordinates = "Time Latitude Longitude" ;
#      ATMS_QF1:valid_range = 0, 255 ;
#      ATMS_QF1:_FillValue = 95b ;
#    byte ATMS_QF2(Number_of_CrIS_FORs) ;
#      ATMS_QF2:long_name = "ATMS Quality flag number 2" ;
#      ATMS_QF2:units = "none" ;
#      ATMS_QF2:scale_factor = 1 ;
#      ATMS_QF2:add_offset = 0 ;
#      ATMS_QF2:parameter_type = "ATMS data" ;
#      ATMS_QF2:coordinates = "Time Latitude Longitude" ;
#      ATMS_QF2:valid_range = 0, 255 ;
#      ATMS_QF2:_FillValue = 95b ;
#    float ATMS_Latitude(Number_of_CrIS_FORs) ;
#      ATMS_Latitude:long_name = "ATMS Latitude values for each FOR" ;
#      ATMS_Latitude:units = "degrees North" ;
#      ATMS_Latitude:scale_factor = 1. ;
#      ATMS_Latitude:add_offset = 0. ;
#      ATMS_Latitude:parameter_type = "ATMS data" ;
#      ATMS_Latitude:valid_range = 90.f, -90.f ;
#      ATMS_Latitude:_FillValue = -9999.f ;
#    float ATMS_Longitude(Number_of_CrIS_FORs) ;
#      ATMS_Longitude:long_name = "ATMS Longitude values for each FOR" ;
#      ATMS_Longitude:units = "degrees East" ;
#      ATMS_Longitude:scale_factor = 1. ;
#      ATMS_Longitude:add_offset = 0. ;
#      ATMS_Longitude:parameter_type = "ATMS data" ;
#      ATMS_Longitude:valid_range = -180.f, 179.99f ;
#      ATMS_Longitude:_FillValue = -9999.f ;
#    float ATMS_Satellite_Zenith(Number_of_CrIS_FORs) ;
#      ATMS_Satellite_Zenith:long_name = "Satellite zenith for FOR" ;
#      ATMS_Satellite_Zenith:units = "degrees" ;
#      ATMS_Satellite_Zenith:scale_factor = 1. ;
#      ATMS_Satellite_Zenith:add_offset = 0. ;
#      ATMS_Satellite_Zenith:parameter_type = "ATMS data" ;
#      ATMS_Satellite_Zenith:coordinates = "Time Latitude Longitude" ;
#      ATMS_Satellite_Zenith:valid_range = 0.f, 90.f ;
#      ATMS_Satellite_Zenith:_FillValue = -9999.f ;
#    float ATMS_Satellite_Azimuth(Number_of_CrIS_FORs) ;
#      ATMS_Satellite_Azimuth:long_name = "Satellite azimuth for FOR" ;
#      ATMS_Satellite_Azimuth:units = "degrees" ;
#      ATMS_Satellite_Azimuth:scale_factor = 1. ;
#      ATMS_Satellite_Azimuth:add_offset = 0. ;
#      ATMS_Satellite_Azimuth:parameter_type = "ATMS data" ;
#      ATMS_Satellite_Azimuth:coordinates = "Time Latitude Longitude" ;
#      ATMS_Satellite_Azimuth:valid_range = 0.f, 360.f ;
#      ATMS_Satellite_Azimuth:_FillValue = -9999.f ;
#    float ATMS_Solar_Zenith(Number_of_CrIS_FORs) ;
#      ATMS_Solar_Zenith:long_name = "Solar zenith for each FOR" ;
#      ATMS_Solar_Zenith:units = "degrees" ;
#      ATMS_Solar_Zenith:scale_factor = 1. ;
#      ATMS_Solar_Zenith:add_offset = 0. ;
#      ATMS_Solar_Zenith:parameter_type = "ATMS data" ;
#      ATMS_Solar_Zenith:coordinates = "Time Latitude Longitude" ;
#      ATMS_Solar_Zenith:valid_range = 0.f, 90.f ;
#      ATMS_Solar_Zenith:_FillValue = -9999.f ;
#    float ATMS_Solar_Azimuth(Number_of_CrIS_FORs) ;
#      ATMS_Solar_Azimuth:long_name = "Solar azimuth for each FOR" ;
#      ATMS_Solar_Azimuth:units = "degrees" ;
#      ATMS_Solar_Azimuth:scale_factor = 1. ;
#      ATMS_Solar_Azimuth:add_offset = 0. ;
#      ATMS_Solar_Azimuth:parameter_type = "ATMS data" ;
#      ATMS_Solar_Azimuth:coordinates = "Time Latitude Longitude" ;
#      ATMS_Solar_Azimuth:valid_range = 0.f, 360.f ;
#      ATMS_Solar_Azimuth:_FillValue = -9999.f ;
#    float ATMS_Satellite_Height(Number_of_CrIS_FORs) ;
#      ATMS_Satellite_Height:long_name = "Satellite height above each ATMS FOV" ;
#      ATMS_Satellite_Height:units = "km" ;
#      ATMS_Satellite_Height:scale_factor = 1. ;
#      ATMS_Satellite_Height:add_offset = 0. ;
#      ATMS_Satellite_Height:parameter_type = "ATMS data" ;
#      ATMS_Satellite_Height:coordinates = "Time Latitude Longitude" ;
#      ATMS_Satellite_Height:valid_range = 0.f, 1000.f ;
#      ATMS_Satellite_Height:_FillValue = -9999.f ;
#    double Time(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Time:long_name = "UTC Milliseconds since Jan 1, 1970" ;
#      Time:units = "msec" ;
#      Time:scale_factor = 1. ;
#      Time:add_offset = 0. ;
#      Time:parameter_type = "CrIS data" ;
#      Time:coordinates = "data" ;
#      Time:valid_range = 0.f, 1.577837e+12f ;
#      Time:_FillValue = -9999. ;
#    float CrIS_Latitude(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_Latitude:long_name = "CrIS Latitude values for each FOV" ;
#      CrIS_Latitude:units = "degrees North" ;
#      CrIS_Latitude:scale_factor = 1. ;
#      CrIS_Latitude:add_offset = 0. ;
#      CrIS_Latitude:parameter_type = "CrIS data" ;
#      CrIS_Latitude:valid_range = 90.f, -90.f ;
#      CrIS_Latitude:_FillValue = -9999.f ;
#    float CrIS_Longitude(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_Longitude:long_name = "CrIS Longitude values for each FOV" ;
#      CrIS_Longitude:units = "degrees East" ;
#      CrIS_Longitude:scale_factor = 1. ;
#      CrIS_Longitude:add_offset = 0. ;
#      CrIS_Longitude:parameter_type = "CrIS data" ;
#      CrIS_Longitude:valid_range = -180.f, 179.99f ;
#      CrIS_Longitude:_FillValue = -9999.f ;
#    float Cloud_Fraction(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Cloud_Fraction:long_name = "Cloud fraction" ;
#      Cloud_Fraction:units = "none" ;
#      Cloud_Fraction:scale_factor = 1. ;
#      Cloud_Fraction:add_offset = 0. ;
#      Cloud_Fraction:parameter_type = "VIIRS data" ;
#      Cloud_Fraction:coordinates = "Time Latitude Longitude" ;
#      Cloud_Fraction:valid_range = 0.f, 1.f ;
#      Cloud_Fraction:_FillValue = -9999.f ;
#    float Cloud_Height(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Cloud_Height:long_name = "Cloud height" ;
#      Cloud_Height:units = "m" ;
#      Cloud_Height:scale_factor = 1. ;
#      Cloud_Height:add_offset = 0. ;
#      Cloud_Height:parameter_type = "VIIRS data" ;
#      Cloud_Height:coordinates = "Time Latitude Longitude" ;
#      Cloud_Height:valid_range = 0.f, 100000.f ;
#      Cloud_Height:_FillValue = -9999.f ;
#    float Topography(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Topography:long_name = "Surface height" ;
#      Topography:units = "meters" ;
#      Topography:scale_factor = 1. ;
#      Topography:add_offset = 0. ;
#      Topography:parameter_type = "CrIS data" ;
#      Topography:coordinates = "Time Latitude Longitude" ;
#      Topography:valid_range = 0.f, 10000.f ;
#      Topography:_FillValue = -9999.f ;
#    float Land_Fraction(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Land_Fraction:long_name = "Land fraction" ;
#      Land_Fraction:units = "none" ;
#      Land_Fraction:scale_factor = 1. ;
#      Land_Fraction:add_offset = 0. ;
#      Land_Fraction:parameter_type = "CrIS data" ;
#      Land_Fraction:coordinates = "Time Latitude Longitude" ;
#      Land_Fraction:valid_range = 0.f, 1.f ;
#      Land_Fraction:_FillValue = -9999.f ;
#    float Land_Fraction_Error(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Land_Fraction_Error:long_name = "Land fraction error" ;
#      Land_Fraction_Error:units = "none" ;
#      Land_Fraction_Error:scale_factor = 1. ;
#      Land_Fraction_Error:add_offset = 0. ;
#      Land_Fraction_Error:parameter_type = "CrIS data" ;
#      Land_Fraction_Error:coordinates = "Time Latitude Longitude" ;
#      Land_Fraction_Error:valid_range = 0.f, 10000.f ;
#      Land_Fraction_Error:_FillValue = -9999.f ;
#    float CrIS_Radiances(Number_of_CrIS_FORs, Number_of_CrIS_FOVs, Number_of_CrIS_Channels) ;
#      CrIS_Radiances:long_name = "CrIS Radiances for each FOV" ;
#      CrIS_Radiances:units = "mW/m2/cm-1/sr" ;
#      CrIS_Radiances:scale_factor = 1. ;
#      CrIS_Radiances:add_offset = 0. ;
#      CrIS_Radiances:parameter_type = "CrIS data" ;
#      CrIS_Radiances:coordinates = "Time Latitude Longitude" ;
#      CrIS_Radiances:valid_range = 0.f, 40000.f ;
#      CrIS_Radiances:_FillValue = -9999.f ;
#    float ATMS_Brightness_Temps(Number_of_CrIS_FORs, Number_of_ATMS_Channels) ;
#      ATMS_Brightness_Temps:long_name = "ATMS BTs for each CrIS FOR" ;
#      ATMS_Brightness_Temps:units = "Kelvin" ;
#      ATMS_Brightness_Temps:scale_factor = 1. ;
#      ATMS_Brightness_Temps:add_offset = 0. ;
#      ATMS_Brightness_Temps:parameter_type = "ATMS data" ;
#      ATMS_Brightness_Temps:coordinates = "Time Latitude Longitude" ;
#      ATMS_Brightness_Temps:valid_range = 0.f, 40000.f ;
#      ATMS_Brightness_Temps:_FillValue = -9999.f ;
#    float CrIS_View_Angle(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_View_Angle:long_name = "CrIS View Angles for each FOV" ;
#      CrIS_View_Angle:units = "degrees" ;
#      CrIS_View_Angle:scale_factor = 1. ;
#      CrIS_View_Angle:add_offset = 0. ;
#      CrIS_View_Angle:parameter_type = "CrIS data" ;
#      CrIS_View_Angle:coordinates = "Time Latitude Longitude" ;
#      CrIS_View_Angle:valid_range = -90.f, 90.f ;
#      CrIS_View_Angle:_FillValue = -9999.f ;
#    float Satellite_Height(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Satellite_Height:long_name = "Satellite height above each CrIS FOV" ;
#      Satellite_Height:units = "km" ;
#      Satellite_Height:scale_factor = 1. ;
#      Satellite_Height:add_offset = 0. ;
#      Satellite_Height:parameter_type = "CrIS data" ;
#      Satellite_Height:coordinates = "Time Latitude Longitude" ;
#      Satellite_Height:valid_range = 0.f, 1000.f ;
#      Satellite_Height:_FillValue = -9999.f ;
#    float Satellite_Zenith(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Satellite_Zenith:long_name = "Satellite zenith angles for each CrIS FOV" ;
#      Satellite_Zenith:units = "degrees" ;
#      Satellite_Zenith:scale_factor = 1. ;
#      Satellite_Zenith:add_offset = 0. ;
#      Satellite_Zenith:parameter_type = "CrIS data" ;
#      Satellite_Zenith:coordinates = "Time Latitude Longitude" ;
#      Satellite_Zenith:valid_range = 0.f, 90.f ;
#      Satellite_Zenith:_FillValue = -9999.f ;
#    float Satellite_Azimuth(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Satellite_Azimuth:long_name = "Satellite azimuth angles for each CrIS FOV" ;
#      Satellite_Azimuth:units = "degrees" ;
#      Satellite_Azimuth:scale_factor = 1. ;
#      Satellite_Azimuth:add_offset = 0. ;
#      Satellite_Azimuth:parameter_type = "CrIS data" ;
#      Satellite_Azimuth:coordinates = "Time Latitude Longitude" ;
#      Satellite_Azimuth:valid_range = 0.f, 360.f ;
#      Satellite_Azimuth:_FillValue = -9999.f ;
#    float Solar_Zenith(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Solar_Zenith:long_name = "Solar zenith angles for each CrIS FOV" ;
#      Solar_Zenith:units = "degrees" ;
#      Solar_Zenith:scale_factor = 1. ;
#      Solar_Zenith:add_offset = 0. ;
#      Solar_Zenith:parameter_type = "CrIS data" ;
#      Solar_Zenith:coordinates = "Time Latitude Longitude" ;
#      Solar_Zenith:valid_range = 0.f, 90.f ;
#      Solar_Zenith:_FillValue = -9999.f ;
#    float Solar_Azimuth(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Solar_Azimuth:long_name = "Solar azimuth angles for each CrIS FOV" ;
#      Solar_Azimuth:units = "degrees" ;
#      Solar_Azimuth:scale_factor = 1. ;
#      Solar_Azimuth:add_offset = 0. ;
#      Solar_Azimuth:parameter_type = "CrIS data" ;
#      Solar_Azimuth:coordinates = "Time Latitude Longitude" ;
#      Solar_Azimuth:valid_range = 0.f, 360.f ;
#      Solar_Azimuth:_FillValue = -9999.f ;
#    short Ascending_Descending(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Ascending_Descending:long_name = "Orbital status for each CrIS FOV, 1=Descending, 0=Ascending" ;
#      Ascending_Descending:units = "none" ;
#      Ascending_Descending:scale_factor = 1 ;
#      Ascending_Descending:add_offset = 0 ;
#      Ascending_Descending:parameter_type = "CrIS data" ;
#      Ascending_Descending:coordinates = "Time Latitude Longitude" ;
#      Ascending_Descending:valid_range = 0, 1 ;
#      Ascending_Descending:_FillValue = -9999s ;
#    int Orbit_Number(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Orbit_Number:long_name = "The number of the current orbit" ;
#      Orbit_Number:units = "none" ;
#      Orbit_Number:scale_factor = 1. ;
#      Orbit_Number:add_offset = 0. ;
#      Orbit_Number:parameter_type = "CrIS data" ;
#      Orbit_Number:coordinates = "Time Latitude Longitude" ;
#      Orbit_Number:valid_range = 1.f, 100000.f ;
#      Orbit_Number:_FillValue = -9999 ;
#    float SCPosition(Number_of_CrIS_FORs, Number_of_CrIS_FOVs, Number_of_ECR_Coords) ;
#      SCPosition:long_name = "Spacecraft position in ECR Coordinates" ;
#      SCPosition:units = "kilometer" ;
#      SCPosition:scale_factor = 1. ;
#      SCPosition:add_offset = 0. ;
#      SCPosition:parameter_type = "CrIS data" ;
#      SCPosition:coordinates = "Time Latitude Longitude" ;
#      SCPosition:valid_range = 0.f, 9000000.f ;
#      SCPosition:_FillValue = -9999.f ;

###############################################################################
# NUCAPS-OLR   NUCAPS-OLR   NUCAPS-OLR   NUCAPS-OLR   NUCAPS-OLR   NUCAPS-OLR #
###############################################################################
#  variables:
#    char quality_information ;
#      quality_information:long_name = "total number of retrievals, percentage of
#        optimal retrievals, percentage of bad retrievals" ;
#      quality_information:total_number_retrievals = 1080 ;
#      quality_information:percentage_optimal_retrievals = 100.f ;
#      quality_information:percentage_bad_retrievals = 0.f ;
#    float LAT(NLINE, NFOR, NFOV) ;
#      LAT:long_name = "CrIS Latitude values for each FOV" ;
#      LAT:standard_name = "latitude" ;
#      LAT:units = "degrees_north" ;
#      LAT:parameter_type = "CrIS OLR data" ;
#      LAT:valid_range = -90.f, 90.f ;
#      LAT:_FillValue = -9999.f ;
#    float LON(NLINE, NFOR, NFOV) ;
#      LON:long_name = "CrIS Longitude values for each FOV" ;
#      LON:standard_name = "longitude" ;
#      LON:units = "degrees_east" ;
#      LON:parameter_type = "CrIS OLR data" ;
#      LON:valid_range = -180.f, 180.f ;
#      LON:_FillValue = -9999.f ;
#    double TIME(NLINE, NFOR, NFOV) ;
#      TIME:long_name = "UTC time as milliseconds from 01/01/1970" ;
#      TIME:units = "msec" ;
#      TIME:parameter_type = "CrIS OLR data" ;
#      TIME:_FillValue = -9999. ;
#    float SATZEN(NLINE, NFOR, NFOV) ;
#      SATZEN:long_name = "Satellite zenith angles for each FOV" ;
#      SATZEN:standard_name = "solar_zenith_angle" ;
#      SATZEN:units = "degrees" ;
#      SATZEN:parameter_type = "CrIS OLR data" ;
#      SATZEN:valid_range = 0.f, 180.f ;
#      SATZEN:_FillValue = -9999.f ;
#    float SATHEIGHT(NLINE, NFOR, NFOV) ;
#      SATHEIGHT:long_name = "SATELLITE HEIGHT for each FOV" ;
#      SATHEIGHT:standard_name = "altitude" ;
#      SATHEIGHT:units = "km" ;
#      SATHEIGHT:parameter_type = "CrIS OLR data" ;
#      SATHEIGHT:valid_range = 0.f, 180.f ;
#      SATHEIGHT:_FillValue = -9999.f ;
#    float VIEWANG(NLINE, NFOR, NFOV) ;
#      VIEWANG:long_name = "CrIS View Angles for each FOV" ;
#      VIEWANG:standard_name = "platform_view_angle" ;
#      VIEWANG:units = "degrees" ;
#      VIEWANG:parameter_type = "CrIS OLR data" ;
#      VIEWANG:valid_range = -49.f, 49.f ;
#      VIEWANG:_FillValue = -9999.f ;
#    float FLUX(NLINE, NFOR, NFOV) ;
#      FLUX:long_name = "CrIS OLR at top-of-atmosphere (Wm-2)" ;
#      FLUX:units = "1" ;
#      FLUX:parameter_type = "CrIS OLR data" ;
#      FLUX:valid_range = 0.f, 500.f ;
#      FLUX:_FillValue = -9999.f ;
#    short QA(NLINE, NFOR, NFOV) ;
#      QA:long_name = "CrIS level1c radiance quality flag" ;
#      QA:units = "1" ;
#      QA:scale_factor = 1. ;
#      QA:add_offset = 0. ;
#      QA:parameter_type = "CrIS OLR data" ;
#      QA:valid_range = 0, 1 ;
#      QA:_FillValue = -9999s ;
#    short AD(NLINE, NFOR, NFOV) ;
#      AD:long_name = "CrIS level1c AD flag" ;
#      AD:units = "1" ;
#      AD:scale_factor = 1. ;
#      AD:add_offset = 0. ;
#      AD:parameter_type = "CrIS OLR data" ;
#      AD:valid_range = 0, 1 ;
#      AD:_FillValue = -9999s ;

###############################################################################
#   NUCAPS-C0431   NUCAPS-C0431   NUCAPS-C0431   NUCAPS-C0431   NUCAPS-C0431  #
###############################################################################
#  variables:
#    int CrIS_Channels(Number_of_CrIS_Channels) ;
#      CrIS_Channels:long_name = "Channel number for the radiance data" ;
#      CrIS_Channels:units = "none" ;
#      CrIS_Channels:scale_factor = 1. ;
#      CrIS_Channels:add_offset = 0. ;
#      CrIS_Channels:parameter_type = "CrIS data" ;
#      CrIS_Channels:valid_range = 1.f, 431.f ;
#      CrIS_Channels:_FillValue = -9999 ;
#    float CrIS_Frequencies(Number_of_CrIS_Channels) ;
#      CrIS_Frequencies:long_name = "Frequency at which the radiances are observed" ;
#      CrIS_Frequencies:units = "wavenumber cm-1" ;
#      CrIS_Frequencies:scale_factor = 1. ;
#      CrIS_Frequencies:add_offset = 0. ;
#      CrIS_Frequencies:parameter_type = "CrIS data" ;
#      CrIS_Frequencies:valid_range = 0.f, 10000.f ;
#      CrIS_Frequencies:_FillValue = -9999.f ;
#    int ATMS_Channels(Number_of_ATMS_Channels) ;
#      ATMS_Channels:long_name = "Channel number for the radiance data" ;
#      ATMS_Channels:units = "none" ;
#      ATMS_Channels:scale_factor = 1. ;
#      ATMS_Channels:add_offset = 0. ;
#      ATMS_Channels:parameter_type = "ATMS data" ;
#      ATMS_Channels:valid_range = 1.f, 22.f ;
#      ATMS_Channels:_FillValue = -9999 ;
#    float ATMS_Frequencies(Number_of_ATMS_Channels) ;
#      ATMS_Frequencies:long_name = "Frequency at which the data is taken" ;
#      ATMS_Frequencies:units = "GHz" ;
#      ATMS_Frequencies:scale_factor = 1. ;
#      ATMS_Frequencies:add_offset = 0. ;
#      ATMS_Frequencies:parameter_type = "ATMS data" ;
#      ATMS_Frequencies:valid_range = 0.f, 16000.f ;
#      ATMS_Frequencies:_FillValue = -9999.f ;
#    int Subset_CrIS_FORs(Number_of_CrIS_FORs) ;
#      Subset_CrIS_FORs:long_name = "The index of the CrIS fields of regard to subset" ;
#      Subset_CrIS_FORs:units = "none" ;
#      Subset_CrIS_FORs:scale_factor = 1. ;
#      Subset_CrIS_FORs:add_offset = 0. ;
#      Subset_CrIS_FORs:parameter_type = "CrIS data" ;
#      Subset_CrIS_FORs:coordinates = "Time Latitude Longitude" ;
#      Subset_CrIS_FORs:valid_range = 1.f, 1350.f ;
#      Subset_CrIS_FORs:_FillValue = -9999 ;
#    int Subset_CrIS_FOVs(Number_of_CrIS_FOVs) ;
#      Subset_CrIS_FOVs:long_name = "The index of the CrIS fields of view to subset" ;
#      Subset_CrIS_FOVs:units = "none" ;
#      Subset_CrIS_FOVs:scale_factor = 1. ;
#      Subset_CrIS_FOVs:add_offset = 0. ;
#      Subset_CrIS_FOVs:parameter_type = "CrIS data" ;
#      Subset_CrIS_FOVs:valid_range = 1.f, 9.f ;
#      Subset_CrIS_FOVs:_FillValue = -9999 ;
#    int Scan_Line(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Scan_Line:long_name = "The number of the current scan line in the granule" ;
#      Scan_Line:units = "none" ;
#      Scan_Line:scale_factor = 1. ;
#      Scan_Line:add_offset = 0. ;
#      Scan_Line:parameter_type = "CrIS data" ;
#      Scan_Line:coordinates = "Time Latitude Longitude" ;
#      Scan_Line:valid_range = 1.f, 4.f ;
#      Scan_Line:_FillValue = -9999 ;
#    int CrIS_FORs(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_FORs:long_name = "The number of the current CrIS FOR for the current FOV" ;
#      CrIS_FORs:units = "none" ;
#      CrIS_FORs:scale_factor = 1. ;
#      CrIS_FORs:add_offset = 0. ;
#      CrIS_FORs:parameter_type = "CrIS data" ;
#      CrIS_FORs:coordinates = "Time Latitude Longitude" ;
#      CrIS_FORs:valid_range = 1.f, 120.f ;
#      CrIS_FORs:_FillValue = -9999 ;
#    int CrIS_FOVs(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_FOVs:long_name = "The number of the current CrIS FOV on the field of regard" ;
#      CrIS_FOVs:units = "none" ;
#      CrIS_FOVs:scale_factor = 1. ;
#      CrIS_FOVs:add_offset = 0. ;
#      CrIS_FOVs:parameter_type = "CrIS data" ;
#      CrIS_FOVs:coordinates = "Time Latitude Longitude" ;
#      CrIS_FOVs:valid_range = 1.f, 9.f ;
#      CrIS_FOVs:_FillValue = -9999 ;
#    byte CrIS_QF1(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_QF1:long_name = "CrIS quality flag number 1" ;
#      CrIS_QF1:units = "none" ;
#      CrIS_QF1:scale_factor = 1 ;
#      CrIS_QF1:add_offset = 0 ;
#      CrIS_QF1:parameter_type = "CrIS data" ;
#      CrIS_QF1:coordinates = "Time Latitude Longitude" ;
#      CrIS_QF1:valid_range = 0, 255 ;
#      CrIS_QF1:_FillValue = 95b ;
#    byte CrIS_QF2(Number_of_CrIS_FORs, Number_of_CrIS_FOVs, Number_of_CrIS_Bands) ;
#      CrIS_QF2:long_name = "CrIS quality flag number 2" ;
#      CrIS_QF2:units = "none" ;
#      CrIS_QF2:scale_factor = 1 ;
#      CrIS_QF2:add_offset = 0 ;
#      CrIS_QF2:parameter_type = "CrIS data" ;
#      CrIS_QF2:coordinates = "Time Latitude Longitude" ;
#      CrIS_QF2:valid_range = 0, 255 ;
#      CrIS_QF2:_FillValue = 95b ;
#    byte CrIS_QF3(Number_of_CrIS_FORs, Number_of_CrIS_FOVs, Number_of_CrIS_Bands) ;
#      CrIS_QF3:long_name = "CrIS quality flag number 3" ;
#      CrIS_QF3:units = "none" ;
#      CrIS_QF3:scale_factor = 1 ;
#      CrIS_QF3:add_offset = 0 ;
#      CrIS_QF3:parameter_type = "CrIS data" ;
#      CrIS_QF3:coordinates = "Time Latitude Longitude" ;
#      CrIS_QF3:valid_range = 0, 255 ;
#      CrIS_QF3:_FillValue = 95b ;
#    byte CrIS_QF4(Number_of_CrIS_FORs, Number_of_CrIS_FOVs, Number_of_CrIS_Bands) ;
#      CrIS_QF4:long_name = "CrIS quality flag number 4" ;
#      CrIS_QF4:units = "none" ;
#      CrIS_QF4:scale_factor = 1 ;
#      CrIS_QF4:add_offset = 0 ;
#      CrIS_QF4:parameter_type = "CrIS data" ;
#      CrIS_QF4:coordinates = "Time Latitude Longitude" ;
#      CrIS_QF4:valid_range = 0, 255 ;
#      CrIS_QF4:_FillValue = 95b ;
#    byte CrIS_QF5(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_QF5:long_name = "CrIS quality flag for geolocation" ;
#      CrIS_QF5:units = "none" ;
#      CrIS_QF5:scale_factor = 1 ;
#      CrIS_QF5:add_offset = 0 ;
#      CrIS_QF5:parameter_type = "CrIS data" ;
#      CrIS_QF5:coordinates = "Time Latitude Longitude" ;
#      CrIS_QF5:valid_range = 0, 255 ;
#      CrIS_QF5:_FillValue = 95b ;
#    byte CrIS_QF6(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_QF6:long_name = "CrIS quality flag for CCR" ;
#      CrIS_QF6:units = "none" ;
#      CrIS_QF6:scale_factor = 1 ;
#      CrIS_QF6:add_offset = 0 ;
#      CrIS_QF6:parameter_type = "CrIS data" ;
#      CrIS_QF6:coordinates = "Time Latitude Longitude" ;
#      CrIS_QF6:valid_range = 0, 255 ;
#      CrIS_QF6:_FillValue = 95b ;
#    byte ATMS_QF1(Number_of_CrIS_FORs, Number_of_ATMS_Channels) ;
#      ATMS_QF1:long_name = "ATMS Quality flag number 1" ;
#      ATMS_QF1:units = "none" ;
#      ATMS_QF1:scale_factor = 1 ;
#      ATMS_QF1:add_offset = 0 ;
#      ATMS_QF1:parameter_type = "ATMS data" ;
#      ATMS_QF1:coordinates = "Time Latitude Longitude" ;
#      ATMS_QF1:valid_range = 0, 255 ;
#      ATMS_QF1:_FillValue = 95b ;
#    byte ATMS_QF2(Number_of_CrIS_FORs) ;
#      ATMS_QF2:long_name = "ATMS Quality flag number 2" ;
#      ATMS_QF2:units = "none" ;
#      ATMS_QF2:scale_factor = 1 ;
#      ATMS_QF2:add_offset = 0 ;
#      ATMS_QF2:parameter_type = "Time Latitude Longitude" ;
#      ATMS_QF2:valid_range = 0, 255 ;
#      ATMS_QF2:_FillValue = 95b ;
#    float ATMS_Latitude(Number_of_CrIS_FORs) ;
#      ATMS_Latitude:long_name = "ATMS Latitude values for each FOR" ;
#      ATMS_Latitude:units = "degrees North" ;
#      ATMS_Latitude:scale_factor = 1. ;
#      ATMS_Latitude:add_offset = 0. ;
#      ATMS_Latitude:parameter_type = "ATMS data" ;
#      ATMS_Latitude:coordinates = "data" ;
#      ATMS_Latitude:valid_range = 90.f, -90.f ;
#      ATMS_Latitude:_FillValue = -9999.f ;
#    float ATMS_Longitude(Number_of_CrIS_FORs) ;
#      ATMS_Longitude:long_name = "ATMS Longitude values for each FOR" ;
#      ATMS_Longitude:units = "degrees East" ;
#      ATMS_Longitude:scale_factor = 1. ;
#      ATMS_Longitude:add_offset = 0. ;
#      ATMS_Longitude:parameter_type = "ATMS data" ;
#      ATMS_Longitude:valid_range = -180.f, 179.99f ;
#      ATMS_Longitude:_FillValue = -9999.f ;
#    float ATMS_Satellite_Zenith(Number_of_CrIS_FORs) ;
#      ATMS_Satellite_Zenith:long_name = "Satellite zenith for FOR" ;
#      ATMS_Satellite_Zenith:units = "degrees" ;
#      ATMS_Satellite_Zenith:scale_factor = 1. ;
#      ATMS_Satellite_Zenith:add_offset = 0. ;
#      ATMS_Satellite_Zenith:parameter_type = "ATMS data" ;
#      ATMS_Satellite_Zenith:coordinates = "Time Latitude Longitude" ;
#      ATMS_Satellite_Zenith:valid_range = 0.f, 90.f ;
#      ATMS_Satellite_Zenith:_FillValue = -9999.f ;
#    float ATMS_Satellite_Azimuth(Number_of_CrIS_FORs) ;
#      ATMS_Satellite_Azimuth:long_name = "Satellite azimuth for FOR" ;
#      ATMS_Satellite_Azimuth:units = "degrees" ;
#      ATMS_Satellite_Azimuth:scale_factor = 1. ;
#      ATMS_Satellite_Azimuth:add_offset = 0. ;
#      ATMS_Satellite_Azimuth:parameter_type = "ATMS data" ;
#      ATMS_Satellite_Azimuth:coordinates = "Time Latitude Longitude" ;
#      ATMS_Satellite_Azimuth:valid_range = 0.f, 360.f ;
#      ATMS_Satellite_Azimuth:_FillValue = -9999.f ;
#    float ATMS_Solar_Zenith(Number_of_CrIS_FORs) ;
#      ATMS_Solar_Zenith:long_name = "Solar zenith for each FOR" ;
#      ATMS_Solar_Zenith:units = "degrees" ;
#      ATMS_Solar_Zenith:scale_factor = 1. ;
#      ATMS_Solar_Zenith:add_offset = 0. ;
#      ATMS_Solar_Zenith:parameter_type = "ATMS data" ;
#      ATMS_Solar_Zenith:coordinates = "Time Latitude Longitude" ;
#      ATMS_Solar_Zenith:valid_range = 0.f, 90.f ;
#      ATMS_Solar_Zenith:_FillValue = -9999.f ;
#    float ATMS_Solar_Azimuth(Number_of_CrIS_FORs) ;
#      ATMS_Solar_Azimuth:long_name = "Solar azimuth for each FOR" ;
#      ATMS_Solar_Azimuth:units = "degrees" ;
#      ATMS_Solar_Azimuth:scale_factor = 1. ;
#      ATMS_Solar_Azimuth:add_offset = 0. ;
#      ATMS_Solar_Azimuth:parameter_type = "ATMS data" ;
#      ATMS_Solar_Azimuth:coordinates = "Time Latitude Longitude" ;
#      ATMS_Solar_Azimuth:valid_range = 0.f, 360.f ;
#      ATMS_Solar_Azimuth:_FillValue = -9999.f ;
#    float ATMS_Satellite_Height(Number_of_CrIS_FORs) ;
#      ATMS_Satellite_Height:long_name = "Satellite height above each ATMS FOV" ;
#      ATMS_Satellite_Height:units = "km" ;
#      ATMS_Satellite_Height:scale_factor = 1. ;
#      ATMS_Satellite_Height:add_offset = 0. ;
#      ATMS_Satellite_Height:parameter_type = "ATMS data" ;
#      ATMS_Satellite_Height:coordinates = "Time Latitude Longitude" ;
#      ATMS_Satellite_Height:valid_range = 0.f, 1000.f ;
#      ATMS_Satellite_Height:_FillValue = -9999.f ;
#    double Time(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Time:long_name = "UTC Milliseconds since Jan 1, 1970" ;
#      Time:units = "msec" ;
#      Time:scale_factor = 1. ;
#      Time:add_offset = 0. ;
#      Time:parameter_type = "CrIS data" ;
#      Time:valid_range = 0.f, 1.577837e+12f ;
#      Time:_FillValue = -9999. ;
#    float CrIS_Latitude(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_Latitude:long_name = "CrIS Latitude values for each FOV" ;
#      CrIS_Latitude:units = "degrees North" ;
#      CrIS_Latitude:scale_factor = 1. ;
#      CrIS_Latitude:add_offset = 0. ;
#      CrIS_Latitude:parameter_type = "CrIS data" ;
#      CrIS_Latitude:valid_range = 90.f, -90.f ;
#      CrIS_Latitude:_FillValue = -9999.f ;
#    float CrIS_Longitude(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_Longitude:long_name = "CrIS Longitude values for each FOV" ;
#      CrIS_Longitude:units = "degrees East" ;
#      CrIS_Longitude:scale_factor = 1. ;
#      CrIS_Longitude:add_offset = 0. ;
#      CrIS_Longitude:parameter_type = "CrIS data" ;
#      CrIS_Longitude:valid_range = -180.f, 179.99f ;
#      CrIS_Longitude:_FillValue = -9999.f ;
#    float Cloud_Fraction(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Cloud_Fraction:long_name = "Cloud fraction" ;
#      Cloud_Fraction:units = "none" ;
#      Cloud_Fraction:scale_factor = 1. ;
#      Cloud_Fraction:add_offset = 0. ;
#      Cloud_Fraction:parameter_type = "VIIRS data" ;
#      Cloud_Fraction:coordinates = "Time Latitude Longitude" ;
#      Cloud_Fraction:valid_range = 0.f, 1.f ;
#      Cloud_Fraction:_FillValue = -9999.f ;
#    float Cloud_Height(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Cloud_Height:long_name = "Cloud height" ;
#      Cloud_Height:units = "m" ;
#      Cloud_Height:scale_factor = 1. ;
#      Cloud_Height:add_offset = 0. ;
#      Cloud_Height:parameter_type = "VIIRS data" ;
#      Cloud_Height:coordinates = "Time Latitude Longitude" ;
#      Cloud_Height:valid_range = 0.f, 100000.f ;
#      Cloud_Height:_FillValue = -9999.f ;
#    float Topography(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Topography:long_name = "Surface height" ;
#      Topography:units = "meters" ;
#      Topography:scale_factor = 1. ;
#      Topography:add_offset = 0. ;
#      Topography:parameter_type = "CrIS data" ;
#      Topography:coordinates = "Time Latitude Longitude" ;
#      Topography:valid_range = 0.f, 10000.f ;
#      Topography:_FillValue = -9999.f ;
#    float Land_Fraction(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Land_Fraction:long_name = "Land fraction" ;
#      Land_Fraction:units = "none" ;
#      Land_Fraction:scale_factor = 1. ;
#      Land_Fraction:add_offset = 0. ;
#      Land_Fraction:parameter_type = "CrIS data" ;
#      Land_Fraction:coordinates = "Time Latitude Longitude" ;
#      Land_Fraction:valid_range = 0.f, 1.f ;
#      Land_Fraction:_FillValue = -9999.f ;
#    float Land_Fraction_Error(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Land_Fraction_Error:long_name = "Land fraction error" ;
#      Land_Fraction_Error:units = "none" ;
#      Land_Fraction_Error:scale_factor = 1. ;
#      Land_Fraction_Error:add_offset = 0. ;
#      Land_Fraction_Error:parameter_type = "CrIS data" ;
#      Land_Fraction_Error:coordinates = "Time Latitude Longitude" ;
#      Land_Fraction_Error:valid_range = 0.f, 10000.f ;
#      Land_Fraction_Error:_FillValue = -9999.f ;
#    float CrIS_Radiances(Number_of_CrIS_FORs, Number_of_CrIS_FOVs, Number_of_CrIS_Channels) ;
#      CrIS_Radiances:long_name = "CrIS Radiances for each FOV" ;
#      CrIS_Radiances:units = "mW/m2/cm-1/sr" ;
#      CrIS_Radiances:scale_factor = 1. ;
#      CrIS_Radiances:add_offset = 0. ;
#      CrIS_Radiances:parameter_type = "CrIS data" ;
#      CrIS_Radiances:coordinates = "Time Latitude Longitude" ;
#      CrIS_Radiances:valid_range = 0.f, 40000.f ;
#      CrIS_Radiances:_FillValue = -9999.f ;
#    float ATMS_Brightness_Temps(Number_of_CrIS_FORs, Number_of_ATMS_Channels) ;
#      ATMS_Brightness_Temps:long_name = "ATMS BTs for each CrIS FOR" ;
#      ATMS_Brightness_Temps:units = "Kelvin" ;
#      ATMS_Brightness_Temps:scale_factor = 1. ;
#      ATMS_Brightness_Temps:add_offset = 0. ;
#      ATMS_Brightness_Temps:parameter_type = "ATMS data" ;
#      ATMS_Brightness_Temps:coordinates = "Time Latitude Longitude" ;
#      ATMS_Brightness_Temps:valid_range = 0.f, 40000.f ;
#      ATMS_Brightness_Temps:_FillValue = -9999.f ;
#    float CrIS_View_Angle(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_View_Angle:long_name = "CrIS View Angles for each FOV" ;
#      CrIS_View_Angle:units = "degrees" ;
#      CrIS_View_Angle:scale_factor = 1. ;
#      CrIS_View_Angle:add_offset = 0. ;
#      CrIS_View_Angle:parameter_type = "CrIS data" ;
#      CrIS_View_Angle:coordinates = "Time Latitude Longitude" ;
#      CrIS_View_Angle:valid_range = -90.f, 90.f ;
#      CrIS_View_Angle:_FillValue = -9999.f ;
#    float Satellite_Height(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Satellite_Height:long_name = "Satellite height above each CrIS FOV" ;
#      Satellite_Height:units = "km" ;
#      Satellite_Height:scale_factor = 1. ;
#      Satellite_Height:add_offset = 0. ;
#      Satellite_Height:parameter_type = "CrIS data" ;
#      Satellite_Height:coordinates = "Time Latitude Longitude" ;
#      Satellite_Height:valid_range = 0.f, 1000.f ;
#      Satellite_Height:_FillValue = -9999.f ;
#    float Satellite_Zenith(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Satellite_Zenith:long_name = "Satellite zenith angles for each CrIS FOV" ;
#      Satellite_Zenith:units = "degrees" ;
#      Satellite_Zenith:scale_factor = 1. ;
#      Satellite_Zenith:add_offset = 0. ;
#      Satellite_Zenith:parameter_type = "CrIS data" ;
#      Satellite_Zenith:coordinates = "Time Latitude Longitude" ;
#      Satellite_Zenith:valid_range = 0.f, 90.f ;
#      Satellite_Zenith:_FillValue = -9999.f ;
#    float Satellite_Azimuth(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Satellite_Azimuth:long_name = "Satellite azimuth angles for each CrIS FOV" ;
#      Satellite_Azimuth:units = "degrees" ;
#      Satellite_Azimuth:scale_factor = 1. ;
#      Satellite_Azimuth:add_offset = 0. ;
#      Satellite_Azimuth:parameter_type = "CrIS data" ;
#      Satellite_Azimuth:coordinates = "Time Latitude Longitude" ;
#      Satellite_Azimuth:valid_range = 0.f, 360.f ;
#      Satellite_Azimuth:_FillValue = -9999.f ;
#    float Solar_Zenith(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Solar_Zenith:long_name = "Solar zenith angles for each CrIS FOV" ;
#      Solar_Zenith:units = "degrees" ;
#      Solar_Zenith:scale_factor = 1. ;
#      Solar_Zenith:add_offset = 0. ;
#      Solar_Zenith:parameter_type = "CrIS data" ;
#      Solar_Zenith:coordinates = "Time Latitude Longitude" ;
#      Solar_Zenith:valid_range = 0.f, 90.f ;
#      Solar_Zenith:_FillValue = -9999.f ;
#    float Solar_Azimuth(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Solar_Azimuth:long_name = "Solar azimuth angles for each CrIS FOV" ;
#      Solar_Azimuth:units = "degrees" ;
#      Solar_Azimuth:scale_factor = 1. ;
#      Solar_Azimuth:add_offset = 0. ;
#      Solar_Azimuth:parameter_type = "CrIS data" ;
#      Solar_Azimuth:coordinates = "Time Latitude Longitude" ;
#      Solar_Azimuth:valid_range = 0.f, 360.f ;
#      Solar_Azimuth:_FillValue = -9999.f ;
#    short Ascending_Descending(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Ascending_Descending:long_name = "Orbital status for each CrIS FOV, 1=Descending, 0=Ascending" ;
#      Ascending_Descending:units = "none" ;
#      Ascending_Descending:scale_factor = 1 ;
#      Ascending_Descending:add_offset = 0 ;
#      Ascending_Descending:parameter_type = "CrIS data" ;
#      Ascending_Descending:coordinates = "Time Latitude Longitude" ;
#      Ascending_Descending:valid_range = 0, 1 ;
#      Ascending_Descending:_FillValue = -9999s ;
#    int Orbit_Number(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Orbit_Number:long_name = "The number of the current orbit" ;
#      Orbit_Number:units = "none" ;
#      Orbit_Number:scale_factor = 1. ;
#      Orbit_Number:add_offset = 0. ;
#      Orbit_Number:parameter_type = "CrIS data" ;
#      Orbit_Number:coordinates = "Time Latitude Longitude" ;
#      Orbit_Number:valid_range = 1.f, 100000.f ;
#      Orbit_Number:_FillValue = -9999 ;
#    float SCPosition(Number_of_CrIS_FORs, Number_of_CrIS_FOVs, Number_of_ECR_Coords) ;
#      SCPosition:long_name = "Spacecraft position in ECR Coordinates" ;
#      SCPosition:units = "kilometer" ;
#      SCPosition:scale_factor = 1. ;
#      SCPosition:add_offset = 0. ;
#      SCPosition:parameter_type = "CrIS data" ;
#      SCPosition:coordinates = "Time Latitude Longitude" ;
#      SCPosition:valid_range = 0.f, 9000000.f ;
#      SCPosition:_FillValue = -9999.f ;

###############################################################################
#  ALLFOVS_PCS3B  ALLFOVS_PCS3B  ALLFOVS_PCS3B  ALLFOVS_PCS3B  ALLFOVS_PCS3B  #
###############################################################################
#  variables:
#    int CrIS_Channels(Number_of_CrIS_Channels) ;
#      CrIS_Channels:long_name = "Channel number for the radiance data" ;
#      CrIS_Channels:units = "none" ;
#      CrIS_Channels:scale_factor = 1. ;
#      CrIS_Channels:add_offset = 0. ;
#      CrIS_Channels:parameter_type = "CrIS data" ;
#      CrIS_Channels:valid_range = 1.f, 2223.f ;
#      CrIS_Channels:_FillValue = -9999 ;
#    float CrIS_Frequencies(Number_of_CrIS_Channels) ;
#      CrIS_Frequencies:long_name = "Frequency at which the radiances are observed" ;
#      CrIS_Frequencies:units = "wavenumber cm-1" ;
#      CrIS_Frequencies:scale_factor = 1. ;
#      CrIS_Frequencies:add_offset = 0. ;
#      CrIS_Frequencies:parameter_type = "CrIS data" ;
#      CrIS_Frequencies:valid_range = 0.f, 10000.f ;
#      CrIS_Frequencies:_FillValue = -9999.f ;
#    int CrIS_RR_Channels(Number_of_CrIS_RR_Channels) ;
#      CrIS_RR_Channels:long_name = "Channel number for the RR data" ;
#      CrIS_RR_Channels:units = "none" ;
#      CrIS_RR_Channels:scale_factor = 1. ;
#      CrIS_RR_Channels:add_offset = 0. ;
#      CrIS_RR_Channels:parameter_type = "CrIS data" ;
#      CrIS_RR_Channels:valid_range = 1.f, 431.f ;
#      CrIS_RR_Channels:_FillValue = -9999 ;
#    float CrIS_RR_Frequencies(Number_of_CrIS_RR_Channels) ;
#      CrIS_RR_Frequencies:long_name = "Frequency of the RR data" ;
#      CrIS_RR_Frequencies:units = "wavenumber cm-1" ;
#      CrIS_RR_Frequencies:scale_factor = 1. ;
#      CrIS_RR_Frequencies:add_offset = 0. ;
#      CrIS_RR_Frequencies:parameter_type = "CrIS data" ;
#      CrIS_RR_Frequencies:valid_range = 0.f, 10000.f ;
#      CrIS_RR_Frequencies:_FillValue = -9999.f ;
#    int ATMS_Channels(Number_of_ATMS_Channels) ;
#      ATMS_Channels:long_name = "Channel number for the radiance data" ;
#      ATMS_Channels:units = "none" ;
#      ATMS_Channels:scale_factor = 1. ;
#      ATMS_Channels:add_offset = 0. ;
#      ATMS_Channels:parameter_type = "ATMS data" ;
#      ATMS_Channels:valid_range = 1.f, 22.f ;
#      ATMS_Channels:_FillValue = -9999 ;
#    float ATMS_Frequencies(Number_of_ATMS_Channels) ;
#      ATMS_Frequencies:long_name = "Frequency at which the data is taken" ;
#      ATMS_Frequencies:units = "GHz" ;
#      ATMS_Frequencies:scale_factor = 1. ;
#      ATMS_Frequencies:add_offset = 0. ;
#      ATMS_Frequencies:parameter_type = "ATMS data" ;
#      ATMS_Frequencies:valid_range = 0.f, 16000.f ;
#      ATMS_Frequencies:_FillValue = -9999.f ;
#    int Subset_CrIS_FORs(Number_of_CrIS_FORs) ;
#      Subset_CrIS_FORs:long_name = "The index of the CrIS fields of regard to subset" ;
#      Subset_CrIS_FORs:units = "none" ;
#      Subset_CrIS_FORs:scale_factor = 1. ;
#      Subset_CrIS_FORs:add_offset = 0. ;
#      Subset_CrIS_FORs:parameter_type = "CrIS data" ;
#      Subset_CrIS_FORs:valid_range = 1.f, 1350.f ;
#      Subset_CrIS_FORs:_FillValue = -9999 ;
#    int Subset_CrIS_FOVs(Number_of_CrIS_FOVs) ;
#      Subset_CrIS_FOVs:long_name = "The index of the CrIS fields of view to subset" ;
#      Subset_CrIS_FOVs:units = "none" ;
#      Subset_CrIS_FOVs:scale_factor = 1. ;
#      Subset_CrIS_FOVs:add_offset = 0. ;
#      Subset_CrIS_FOVs:parameter_type = "CrIS data" ;
#      Subset_CrIS_FOVs:valid_range = 1.f, 9.f ;
#      Subset_CrIS_FOVs:_FillValue = -9999 ;
#    int Scan_Line(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Scan_Line:long_name = "The number of the current scan line in the granule" ;
#      Scan_Line:units = "none" ;
#      Scan_Line:scale_factor = 1. ;
#      Scan_Line:add_offset = 0. ;
#      Scan_Line:parameter_type = "CrIS data" ;
#      Scan_Line:coordinates = "Time Latitude Longitude" ;
#      Scan_Line:valid_range = 1.f, 45.f ;
#      Scan_Line:_FillValue = -9999 ;
#    int CrIS_FORs(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_FORs:long_name = "The number of the current CrIS FOR for the current FOV" ;
#      CrIS_FORs:units = "none" ;
#      CrIS_FORs:scale_factor = 1. ;
#      CrIS_FORs:add_offset = 0. ;
#      CrIS_FORs:parameter_type = "CrIS data" ;
#      CrIS_FORs:coordinates = "Time Latitude Longitude" ;
#      CrIS_FORs:valid_range = 1.f, 1350.f ;
#      CrIS_FORs:_FillValue = -9999 ;
#    int CrIS_FOVs(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_FOVs:long_name = "The number of the current CrIS FOV on the field of regard" ;
#      CrIS_FOVs:units = "none" ;
#      CrIS_FOVs:scale_factor = 1. ;
#      CrIS_FOVs:add_offset = 0. ;
#      CrIS_FOVs:parameter_type = "CrIS data" ;
#      CrIS_FOVs:coordinates = "Time Latitude Longitude" ;
#      CrIS_FOVs:valid_range = 1.f, 9.f ;
#      CrIS_FOVs:_FillValue = -9999 ;
#    byte CrIS_QF1(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_QF1:long_name = "CrIS quality flag number 1" ;
#      CrIS_QF1:units = "none" ;
#      CrIS_QF1:scale_factor = 1 ;
#      CrIS_QF1:add_offset = 0 ;
#      CrIS_QF1:parameter_type = "CrIS data" ;
#      CrIS_QF1:coordinates = "Time Latitude Longitude" ;
#      CrIS_QF1:valid_range = 0, 255 ;
#      CrIS_QF1:_FillValue = 95b ;
#    byte CrIS_QF2(Number_of_CrIS_FORs, Number_of_CrIS_FOVs, Number_of_CrIS_Bands) ;
#      CrIS_QF2:long_name = "CrIS quality flag number 2" ;
#      CrIS_QF2:units = "none" ;
#      CrIS_QF2:scale_factor = 1 ;
#      CrIS_QF2:add_offset = 0 ;
#      CrIS_QF2:parameter_type = "CrIS data" ;
#      CrIS_QF2:coordinates = "Time Latitude Longitude" ;
#      CrIS_QF2:valid_range = 0, 255 ;
#      CrIS_QF2:_FillValue = 95b ;
#    byte CrIS_QF3(Number_of_CrIS_FORs, Number_of_CrIS_FOVs, Number_of_CrIS_Bands) ;
#      CrIS_QF3:long_name = "CrIS quality flag number 3" ;
#      CrIS_QF3:units = "none" ;
#      CrIS_QF3:scale_factor = 1 ;
#      CrIS_QF3:add_offset = 0 ;
#      CrIS_QF3:parameter_type = "CrIS data" ;
#      CrIS_QF3:coordinates = "Time Latitude Longitude" ;
#      CrIS_QF3:valid_range = 0, 255 ;
#      CrIS_QF3:_FillValue = 95b ;
#    byte CrIS_QF4(Number_of_CrIS_FORs, Number_of_CrIS_FOVs, Number_of_CrIS_Bands) ;
#      CrIS_QF4:long_name = "CrIS quality flag number 4" ;
#      CrIS_QF4:units = "none" ;
#      CrIS_QF4:scale_factor = 1 ;
#      CrIS_QF4:add_offset = 0 ;
#      CrIS_QF4:parameter_type = "CrIS data" ;
#      CrIS_QF4:coordinates = "Time Latitude Longitude" ;
#      CrIS_QF4:valid_range = 0, 255 ;
#      CrIS_QF4:_FillValue = 95b ;
#    byte CrIS_QF5(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_QF5:long_name = "CrIS quality flag for geolocation" ;
#      CrIS_QF5:units = "none" ;
#      CrIS_QF5:scale_factor = 1 ;
#      CrIS_QF5:add_offset = 0 ;
#      CrIS_QF5:parameter_type = "CrIS data" ;
#      CrIS_QF5:coordinates = "Time Latitude Longitude" ;
#      CrIS_QF5:valid_range = 0, 255 ;
#      CrIS_QF5:_FillValue = 95b ;
#    byte CrIS_QF6(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_QF6:long_name = "CrIS quality flag for CCR" ;
#      CrIS_QF6:units = "none" ;
#      CrIS_QF6:scale_factor = 1 ;
#      CrIS_QF6:add_offset = 0 ;
#      CrIS_QF6:parameter_type = "CrIS data" ;
#      CrIS_QF6:coordinates = "Time Latitude Longitude" ;
#      CrIS_QF6:valid_range = 0, 255 ;
#      CrIS_QF6:_FillValue = 95b ;
#    byte ATMS_QF1(Number_of_CrIS_FORs, Number_of_ATMS_Channels) ;
#      ATMS_QF1:long_name = "ATMS Quality flag number 1" ;
#      ATMS_QF1:units = "none" ;
#      ATMS_QF1:scale_factor = 1 ;
#      ATMS_QF1:add_offset = 0 ;
#      ATMS_QF1:parameter_type = "ATMS data" ;
#      ATMS_QF1:coordinates = "Time Latitude Longitude" ;
#      ATMS_QF1:valid_range = 0, 255 ;
#      ATMS_QF1:_FillValue = 95b ;
#    byte ATMS_QF2(Number_of_CrIS_FORs) ;
#      ATMS_QF2:long_name = "ATMS Quality flag number 2" ;
#      ATMS_QF2:units = "none" ;
#      ATMS_QF2:scale_factor = 1 ;
#      ATMS_QF2:add_offset = 0 ;
#      ATMS_QF2:parameter_type = "ATMS data" ;
#      ATMS_QF2:coordinates = "Time Latitude Longitude" ;
#      ATMS_QF2:valid_range = 0, 255 ;
#      ATMS_QF2:_FillValue = 95b ;
#    float ATMS_Latitude(Number_of_CrIS_FORs) ;
#      ATMS_Latitude:long_name = "ATMS Latitude values for each FOR" ;
#      ATMS_Latitude:units = "degrees North" ;
#      ATMS_Latitude:scale_factor = 1. ;
#      ATMS_Latitude:add_offset = 0. ;
#      ATMS_Latitude:parameter_type = "ATMS data" ;
#      ATMS_Latitude:valid_range = 90.f, -90.f ;
#      ATMS_Latitude:_FillValue = -9999.f ;
#    float ATMS_Longitude(Number_of_CrIS_FORs) ;
#      ATMS_Longitude:long_name = "ATMS Longitude values for each FOR" ;
#      ATMS_Longitude:units = "degrees East" ;
#      ATMS_Longitude:scale_factor = 1. ;
#      ATMS_Longitude:add_offset = 0. ;
#      ATMS_Longitude:parameter_type = "ATMS data" ;
#      ATMS_Longitude:valid_range = -180.f, 179.99f ;
#      ATMS_Longitude:_FillValue = -9999.f ;
#    float ATMS_Satellite_Zenith(Number_of_CrIS_FORs) ;
#      ATMS_Satellite_Zenith:long_name = "Satellite zenith for FOR" ;
#      ATMS_Satellite_Zenith:units = "degrees" ;
#      ATMS_Satellite_Zenith:scale_factor = 1. ;
#      ATMS_Satellite_Zenith:add_offset = 0. ;
#      ATMS_Satellite_Zenith:parameter_type = "ATMS data" ;
#      ATMS_Satellite_Zenith:coordinates = "Time Latitude Longitude" ;
#      ATMS_Satellite_Zenith:valid_range = 0.f, 90.f ;
#      ATMS_Satellite_Zenith:_FillValue = -9999.f ;
#    float ATMS_Satellite_Azimuth(Number_of_CrIS_FORs) ;
#      ATMS_Satellite_Azimuth:long_name = "Satellite azimuth for FOR" ;
#      ATMS_Satellite_Azimuth:units = "degrees" ;
#      ATMS_Satellite_Azimuth:scale_factor = 1. ;
#      ATMS_Satellite_Azimuth:add_offset = 0. ;
#      ATMS_Satellite_Azimuth:parameter_type = "ATMS data" ;
#      ATMS_Satellite_Azimuth:coordinates = "Time Latitude Longitude" ;
#      ATMS_Satellite_Azimuth:valid_range = 0.f, 360.f ;
#      ATMS_Satellite_Azimuth:_FillValue = -9999.f ;
#    float ATMS_Solar_Zenith(Number_of_CrIS_FORs) ;
#      ATMS_Solar_Zenith:long_name = "Solar zenith for each FOR" ;
#      ATMS_Solar_Zenith:units = "degrees" ;
#      ATMS_Solar_Zenith:scale_factor = 1. ;
#      ATMS_Solar_Zenith:add_offset = 0. ;
#      ATMS_Solar_Zenith:parameter_type = "ATMS data" ;
#      ATMS_Solar_Zenith:coordinates = "Time Latitude Longitude" ;
#      ATMS_Solar_Zenith:valid_range = 0.f, 90.f ;
#      ATMS_Solar_Zenith:_FillValue = -9999.f ;
#    float ATMS_Solar_Azimuth(Number_of_CrIS_FORs) ;
#      ATMS_Solar_Azimuth:long_name = "Solar azimuth for each FOR" ;
#      ATMS_Solar_Azimuth:units = "degrees" ;
#      ATMS_Solar_Azimuth:scale_factor = 1. ;
#      ATMS_Solar_Azimuth:add_offset = 0. ;
#      ATMS_Solar_Azimuth:parameter_type = "ATMS data" ;
#      ATMS_Solar_Azimuth:coordinates = "Time Latitude Longitude" ;
#      ATMS_Solar_Azimuth:valid_range = 0.f, 360.f ;
#      ATMS_Solar_Azimuth:_FillValue = -9999.f ;
#    float ATMS_Satellite_Height(Number_of_CrIS_FORs) ;
#      ATMS_Satellite_Height:long_name = "Satellite height above each ATMS FOV" ;
#      ATMS_Satellite_Height:units = "km" ;
#      ATMS_Satellite_Height:scale_factor = 1. ;
#      ATMS_Satellite_Height:add_offset = 0. ;
#      ATMS_Satellite_Height:parameter_type = "ATMS data" ;
#      ATMS_Satellite_Height:coordinates = "Time Latitude Longitude" ;
#      ATMS_Satellite_Height:valid_range = 0.f, 1000.f ;
#      ATMS_Satellite_Height:_FillValue = -9999.f ;
#    double Time(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Time:long_name = "UTC Milliseconds since Jan 1, 1970" ;
#      Time:units = "msec" ;
#      Time:scale_factor = 1. ;
#      Time:add_offset = 0. ;
#      Time:parameter_type = "CrIS data" ;
#      Time:valid_range = 0.f, 1.577837e+12f ;
#      Time:_FillValue = -9999. ;
#    float CrIS_Latitude(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_Latitude:long_name = "CrIS Latitude values for each FOV" ;
#      CrIS_Latitude:units = "degrees North" ;
#      CrIS_Latitude:scale_factor = 1. ;
#      CrIS_Latitude:add_offset = 0. ;
#      CrIS_Latitude:parameter_type = "CrIS data" ;
#      CrIS_Latitude:valid_range = 90.f, -90.f ;
#      CrIS_Latitude:_FillValue = -9999.f ;
#    float CrIS_Longitude(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_Longitude:long_name = "CrIS Longitude values for each FOV" ;
#      CrIS_Longitude:units = "degrees East" ;
#      CrIS_Longitude:scale_factor = 1. ;
#      CrIS_Longitude:add_offset = 0. ;
#      CrIS_Longitude:parameter_type = "CrIS data" ;
#      CrIS_Longitude:valid_range = -180.f, 179.99f ;
#      CrIS_Longitude:_FillValue = -9999.f ;
#    float Cloud_Fraction(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Cloud_Fraction:long_name = "Cloud fraction" ;
#      Cloud_Fraction:units = "none" ;
#      Cloud_Fraction:scale_factor = 1. ;
#      Cloud_Fraction:add_offset = 0. ;
#      Cloud_Fraction:parameter_type = "VIIRS data" ;
#      Cloud_Fraction:coordinates = "Time Latitude Longitude" ;
#      Cloud_Fraction:valid_range = 0.f, 1.f ;
#      Cloud_Fraction:_FillValue = -9999.f ;
#    float Cloud_Height(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Cloud_Height:long_name = "Cloud height" ;
#      Cloud_Height:units = "m" ;
#      Cloud_Height:scale_factor = 1. ;
#      Cloud_Height:add_offset = 0. ;
#      Cloud_Height:parameter_type = "VIIRS data" ;
#      Cloud_Height:coordinates = "Time Latitude Longitude" ;
#      Cloud_Height:valid_range = 0.f, 100000.f ;
#      Cloud_Height:_FillValue = -9999.f ;
#    float Topography(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Topography:long_name = "Surface height" ;
#      Topography:units = "meters" ;
#      Topography:scale_factor = 1. ;
#      Topography:add_offset = 0. ;
#      Topography:parameter_type = "CrIS data" ;
#      Topography:coordinates = "Time Latitude Longitude" ;
#      Topography:valid_range = 0.f, 10000.f ;
#      Topography:_FillValue = -9999.f ;
#    float Land_Fraction(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Land_Fraction:long_name = "Land fraction" ;
#      Land_Fraction:units = "none" ;
#      Land_Fraction:scale_factor = 1. ;
#      Land_Fraction:add_offset = 0. ;
#      Land_Fraction:parameter_type = "CrIS data" ;
#      Land_Fraction:coordinates = "Time Latitude Longitude" ;
#      Land_Fraction:valid_range = 0.f, 1.f ;
#      Land_Fraction:_FillValue = -9999.f ;
#    float Land_Fraction_Error(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Land_Fraction_Error:long_name = "Land fraction error" ;
#      Land_Fraction_Error:units = "none" ;
#      Land_Fraction_Error:scale_factor = 1. ;
#      Land_Fraction_Error:add_offset = 0. ;
#      Land_Fraction_Error:parameter_type = "CrIS data" ;
#      Land_Fraction_Error:coordinates = "Time Latitude Longitude" ;
#      Land_Fraction_Error:valid_range = 0.f, 10000.f ;
#      Land_Fraction_Error:_FillValue = -9999.f ;
#    float CrIS_Radiances(Number_of_CrIS_FORs, Number_of_CrIS_FOVs, Number_of_CrIS_Channels) ;
#      CrIS_Radiances:long_name = "CrIS Radiances for each FOV" ;
#      CrIS_Radiances:units = "mW/m2/cm-1/sr" ;
#      CrIS_Radiances:scale_factor = 1. ;
#      CrIS_Radiances:add_offset = 0. ;
#      CrIS_Radiances:parameter_type = "CrIS data" ;
#      CrIS_Radiances:coordinates = "Time Latitude Longitude" ;
#      CrIS_Radiances:valid_range = 0.f, 40000.f ;
#      CrIS_Radiances:_FillValue = -9999.f ;
#    float Principal_Components(Number_of_CrIS_FORs, Number_of_CrIS_FOVs, Number_of_Principal_Components) ;
#      Principal_Components:long_name = "CrIS principal components" ;
#      Principal_Components:units = "mW/m2/cm-1/sr" ;
#      Principal_Components:scale_factor = 1. ;
#      Principal_Components:add_offset = 0. ;
#      Principal_Components:parameter_type = "CrIS data" ;
#      Principal_Components:coordinates = "Time Latitude Longitude" ;
#      Principal_Components:valid_range = 0.f, 40000.f ;
#      Principal_Components:_FillValue = -9999.f ;
#    float PC_Scores(Number_of_CrIS_FORs, Number_of_CrIS_FOVs, Number_of_PC_Scores) ;
#      PC_Scores:long_name = "CrIS PC scores for each FOV" ;
#      PC_Scores:units = "mW/m2/cm-1/sr" ;
#      PC_Scores:scale_factor = 1. ;
#      PC_Scores:add_offset = 0. ;
#      PC_Scores:parameter_type = "CrIS data" ;
#      PC_Scores:coordinates = "Time Latitude Longitude" ;
#      PC_Scores:valid_range = 0.f, 40000.f ;
#      PC_Scores:_FillValue = -9999.f ;
#    short CrIS_RR_Quality_Flag(Number_of_CrIS_FORs, Number_of_CrIS_FOVs, Number_of_PC_Scores) ;
#      CrIS_RR_Quality_Flag:long_name = "RR QC flag 0=accept 1=reject" ;
#      CrIS_RR_Quality_Flag:units = "none" ;
#      CrIS_RR_Quality_Flag:scale_factor = 1. ;
#      CrIS_RR_Quality_Flag:add_offset = 0. ;
#      CrIS_RR_Quality_Flag:parameter_type = "CrIS data" ;
#      CrIS_RR_Quality_Flag:coordinates = "Time Latitude Longitude" ;
#      CrIS_RR_Quality_Flag:valid_range = 0, 1 ;
#      CrIS_RR_Quality_Flag:_FillValue = -9999s ;
#    float CrIS_RR(Number_of_CrIS_FORs, Number_of_CrIS_FOVs, Number_of_CrIS_RR_Channels) ;
#      CrIS_RR:long_name = "CrIS Reconstructed Radiances for each FOV" ;
#      CrIS_RR:units = "mW/m2/cm-1/sr" ;
#      CrIS_RR:scale_factor = 1. ;
#      CrIS_RR:add_offset = 0. ;
#      CrIS_RR:parameter_type = "CrIS data" ;
#      CrIS_RR:coordinates = "Time Latitude Longitude" ;
#      CrIS_RR:valid_range = 0.f, 40000.f ;
#      CrIS_RR:_FillValue = -9999.f ;
#    float ATMS_Brightness_Temps(Number_of_CrIS_FORs, Number_of_ATMS_Channels) ;
#      ATMS_Brightness_Temps:long_name = "ATMS BTs for each CrIS FOR" ;
#      ATMS_Brightness_Temps:units = "Kelvin" ;
#      ATMS_Brightness_Temps:scale_factor = 1. ;
#      ATMS_Brightness_Temps:add_offset = 0. ;
#      ATMS_Brightness_Temps:parameter_type = "ATMS data" ;
#      ATMS_Brightness_Temps:coordinates = "Time Latitude Longitude" ;
#      ATMS_Brightness_Temps:valid_range = 0.f, 40000.f ;
#      ATMS_Brightness_Temps:_FillValue = -9999.f ;
#    float CrIS_View_Angle(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      CrIS_View_Angle:long_name = "CrIS View Angles for each FOV" ;
#      CrIS_View_Angle:units = "degrees" ;
#      CrIS_View_Angle:scale_factor = 1. ;
#      CrIS_View_Angle:add_offset = 0. ;
#      CrIS_View_Angle:parameter_type = "CrIS data" ;
#      CrIS_View_Angle:coordinates = "Time Latitude Longitude" ;
#      CrIS_View_Angle:valid_range = -90.f, 90.f ;
#      CrIS_View_Angle:_FillValue = -9999.f ;
#    float Satellite_Height(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Satellite_Height:long_name = "Satellite height above each CrIS FOV" ;
#      Satellite_Height:units = "km" ;
#      Satellite_Height:scale_factor = 1. ;
#      Satellite_Height:add_offset = 0. ;
#      Satellite_Height:parameter_type = "CrIS data" ;
#      Satellite_Height:coordinates = "Time Latitude Longitude" ;
#      Satellite_Height:valid_range = 0.f, 1000.f ;
#      Satellite_Height:_FillValue = -9999.f ;
#    float Satellite_Zenith(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Satellite_Zenith:long_name = "Satellite zenith angles for each CrIS FOV" ;
#      Satellite_Zenith:units = "degrees" ;
#      Satellite_Zenith:scale_factor = 1. ;
#      Satellite_Zenith:add_offset = 0. ;
#      Satellite_Zenith:parameter_type = "CrIS data" ;
#      Satellite_Zenith:coordinates = "Time Latitude Longitude" ;
#      Satellite_Zenith:valid_range = 0.f, 90.f ;
#      Satellite_Zenith:_FillValue = -9999.f ;
#    float Satellite_Azimuth(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Satellite_Azimuth:long_name = "Satellite azimuth angles for each CrIS FOV" ;
#      Satellite_Azimuth:units = "degrees" ;
#      Satellite_Azimuth:scale_factor = 1. ;
#      Satellite_Azimuth:add_offset = 0. ;
#      Satellite_Azimuth:parameter_type = "CrIS data" ;
#      Satellite_Azimuth:coordinates = "Time Latitude Longitude" ;
#      Satellite_Azimuth:valid_range = 0.f, 360.f ;
#      Satellite_Azimuth:_FillValue = -9999.f ;
#    float Solar_Zenith(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Solar_Zenith:long_name = "Solar zenith angles for each CrIS FOV" ;
#      Solar_Zenith:units = "degrees" ;
#      Solar_Zenith:scale_factor = 1. ;
#      Solar_Zenith:add_offset = 0. ;
#      Solar_Zenith:parameter_type = "CrIS data" ;
#      Solar_Zenith:coordinates = "Time Latitude Longitude" ;
#      Solar_Zenith:valid_range = 0.f, 90.f ;
#      Solar_Zenith:_FillValue = -9999.f ;
#    float Solar_Azimuth(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Solar_Azimuth:long_name = "Solar azimuth angles for each CrIS FOV" ;
#      Solar_Azimuth:units = "degrees" ;
#      Solar_Azimuth:scale_factor = 1. ;
#      Solar_Azimuth:add_offset = 0. ;
#      Solar_Azimuth:parameter_type = "CrIS data" ;
#      Solar_Azimuth:coordinates = "Time Latitude Longitude" ;
#      Solar_Azimuth:valid_range = 0.f, 360.f ;
#      Solar_Azimuth:_FillValue = -9999.f ;
#    short Ascending_Descending(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Ascending_Descending:long_name = "Orbital status for each CrIS FOV, 1=Descending, 0=Ascending" ;
#      Ascending_Descending:units = "none" ;
#      Ascending_Descending:scale_factor = 1 ;
#      Ascending_Descending:add_offset = 0 ;
#      Ascending_Descending:parameter_type = "CrIS data" ;
#      Ascending_Descending:coordinates = "Time Latitude Longitude" ;
#      Ascending_Descending:valid_range = 0, 1 ;
#      Ascending_Descending:_FillValue = -9999s ;
#    int Orbit_Number(Number_of_CrIS_FORs, Number_of_CrIS_FOVs) ;
#      Orbit_Number:long_name = "The number of the current orbit" ;
#      Orbit_Number:units = "none" ;
#      Orbit_Number:scale_factor = 1. ;
#      Orbit_Number:add_offset = 0. ;
#      Orbit_Number:parameter_type = "CrIS data" ;
#      Orbit_Number:coordinates = "Time Latitude Longitude" ;
#      Orbit_Number:valid_range = 1.f, 100000.f ;
#      Orbit_Number:_FillValue = -9999 ;
#    float SCPosition(Number_of_CrIS_FORs, Number_of_CrIS_FOVs, Number_of_ECR_Coords) ;
#      SCPosition:long_name = "Spacecraft position in ECR Coordinates" ;
#      SCPosition:units = "kilometer" ;
#      SCPosition:scale_factor = 1. ;
#      SCPosition:add_offset = 0. ;
#      SCPosition:parameter_type = "CrIS data" ;
#      SCPosition:coordinates = "Time Latitude Longitude" ;
#      SCPosition:valid_range = 0.f, 9000000.f ;
#      SCPosition:_FillValue = -9999.f ;


