#!/usr/bin/perl
#
# Top-level perl script for CSPP_NUCAPS package.
# Uses only Perl modules that are part of the v5.006 standard install.
#
# A set of user-defined functions are appended to this main script;
# they are listed in alphabetical order.
#
# Pre-requisites:
# NUCAPS distribution in directory $CSPP_NUCAPS_HOME/$CSPP_NUCAPS_REVNAME
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#                                                                         #
#   For the present (13 September 2014), as an aide de memoir, I am       #
#   going to put in place-holders for non-npp CrIS/ATMS data sources.     #
#                                                                         #
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# START-HELP (N.B. This tag is used in printing help, do not touch!)
#
#  Options:
#  --version             show program's version number and exit
#  ?, -h, --help         show this help message and exit
#  -n, --nothing         make the PCF files, but do not execute.
#
#  Mandatory Arguments:
#    At a minimum these arguments must be specified [i.e. no default].
#
#    -i INPUT_FILES, --input_files=INPUT_FILES
#                        Path to input files. May be directory name, or
#                        directory/filemask (use * and ?, not regexp).
#                        Wildcarded filemasks MUST be "quoted" (or OS expands).
#                        e.g. -i "/dataDir/{GATMO,GCRSO,SCRIS,TATMS}*t18*.h5"
#                        (BUT -i /dataDir  really is the simplest way).
#    -s MISSION, --sat=MISSION
#                        The mission satellite data for NUCAPS algorithm.
#                        Possible values are: 'npp' and 'j01'.
#
#  Extra Options:
#    These options may be used to customize behaviour of this program.
#
#    -w WORK_DIR, --work=WORK_DIR
#                        The directory in which all activity will occur,
#                        this is where output files will be put.
#                        [default: current directory].
#    -m MODE, --mode=MODE
#                        Mode of processing FSR is called HR, NSR called LR,
#                        microwave and infrared is IRMW, infrared only IRO.
#                        Valid are: 'HR_IRMW', 'LR_IRMW', 'HR_IRO', 'LR_IRO'.
#                        [default: LR_IRMW].
#    -d DYNAMIC_ANC, --dynanc=DYNAMIC_ANC
#                        Path to root of dynamic ancillary data tree;
#                        beneath here ancillary data are organized as
#                        per the SSEC remote ancillary server for CSPP,
#                        https://jpssdb.ssec.wisc.edu/cspp_v_2_0/ancillary/,
#                        i.e. %Y_%m_%d_%j where %Y is four-digit year, %m is
#                        two-digit month, %d is two-digit day of month and
#                        %j is three-digit day of year.
#                        [CSPP_NUCAPS_HOME/data/dynamic_ancillary]
#    -r GRIB, --reproc=GRIB
#                        Reprocessing mode using GFS data in grib version GRIB,
#                        where GRIB is 1 or 2. Activating this option also sets
#                        the GFS local repo to DYNAMIC_ANC/%Y/%m (see above)
#                        AND forces --quarantine (see below).  ALSO NOTE:
#                        --reproc=0 means use 1.0-degree GFS in grib2 format,
#                        --reproc=3 means use 0.5-degree GFS in grib2 format
#                        In both these cases (0 & 3) reprocessing mode is OFF.
#                        [default: --reproc=0].
#    -f N_FOVS, --fovs=N_FOVS
#                        The number of fields of view per CrIS field of regard.
#                        Added to accommodate loss of FOV.4 from DB data stream
#                        starting 1-Dec-2014. For IDPS data, and DB data before
#                        01-Dec-2014, you can use --fovs=9 (or -f 9) and for DB
#                        data before 18-Aug-2015 use --fovs=8 (or -f 8).
#                        [default: --fovs=7 for npp, --fovs=9 for j01].
#    -p PROCESSORS, --processors=PROCESSORS
#                        Number of cpus to use for NUCAPS processing.
#                        [default: --processors=1].
#    -z, --subset        Create channel subsets as a post-processing step
#                        [default: --nosub or -noz; --subset or -z contrary].
#    -k, --skipretrieval Do not run retrieval, but do create channel subsets.
#                        N.B. Choosing this option also forces --subset
#                        [default: --noskip; --skip or -k is the contrary].
#    -v, --verbose       Each occurrence increases verbosity 1 level from
#                        [default: ERROR only] -v=WARNING -vv=INFO -vvv=DEBUG
#    -c, --clean         Deletes temporary working directory under WORK_DIR
#                        at process completion.
#                        [default: --clean; --noclean is the contrary option].
#    -q, --quarantine    Processing machine is not connected to the web for the
#                        purpose of fetching required dynamic ancillary data.
#                        [default: --noquarantine].
#    -a, --aggregate     Aggregates NUCAPS-EDR and NUCAPS-CCR-AR output files
#                        into swaths of up to 30 minutes as a POST-PROCESSING
#                        step.  Variables are reshaped to a non-standard form
#                        and files named NUCAPS-AGG-EDR & NUCAPS-AGG-CCR-AR to
#                        clearly distinguish from standard 32-second granules.
#                        [default: --noagg].
#    -b, --break         Breaks (disaggregates) 'npp' and 'j01' SDRs into 32s
#                        granules as a PRE-PROCESSING step. If input SDRs are
#                        provided ALREADY as 32-second granules, choosing the
#                        contrary option, --nobreak, saves both time and
#                        temporary disk space.
#                        [default: --break, --nobreak or -nob is the contrary].
#
# END-HELP (N.B. This tag is used in printing help, do not touch!)
#
# The following support has been dropped until I know what data sources are
# allowed and how they can be mixed & matched.
#
#    -x, --xfiles        Use non DB filenames for non Suomi-NPP data sources.
#                        [default: Direct Broadcast filenames].
#    -j JRR_VIIRS, --jrr=JRR_VIIRS
#                        JPSS Risk Reduction or VIIRS Cloud Mask & Height
#                        files available? Valid are: 'JRR', 'VIIRS', or 'OFF'.
#                        [default: --jrr=OFF]
#    -g GEOLOC_VIIRS, --geo_viirs=GEOLOC_VIIRS
#                        Format of VIIRS geolocation files, if JRR_VIIRS is
#                        set to VIIRS.  Valid values are 'GMODO' or 'GMTCO'
#                        [default: --geo_viirs=GMTCO]
#
# NOTES-TO-SELF:
# -------------
#   $!=N to change exit code on die() to N; default is 255
#
#   --clean removes the temporary directory tree. If the user selects --noclean
#   the temporary directory is NOT removed and all NUCAPS product files are 
#   available in the temporary directory structure. This option might be useful
#   for debugging and is necessary if the user wants access to the product files
#   generated prior to the output netCDF files (that are copied up to WORK_DIR).
#   It is then user responsibility to dispense with unwanted directories/files.
#
#
# This is a valid way to run a single S-NPP granule ...
#
#   ./run_nucaps.pl -s npp -m LR_IRMW \
#     -i "/dataDir/{GATMO,GCRSO,SCRIS,TATMS}_npp_d20131120_t0656260_*_cspp_dev.h5"
#
# The quotes are to prevent OS expanding wildcards/tokens on the commandline.
#
# A design decision (that might get revoked) is to require input data to be in
# a single directory (even if user is using a filemask, you presently cannot
# escape this - see use of $options{input_dir} in function validate_options().
#
#
# DESIGN PHILOSOPHY (is there one?)
# ---------------------------------
# Build up what you need to run the process in a temporary working directory,
# using symlinks where you can to minimise i/o and duplication. Because you
# could have multiple swaths of data in INPUTFILES, create a swath_nnn below
# temporary working directory for each swath. The typical DB use case is going
# to be a single swath per processing run (I think). I am going to build the
# case for, and process, swath_001 before I even start building the case for
# swath_002 (if it exists). This, and the fact that processing is done granule
# by granule within a swath, has implications for abort conditions - I do not
# want to abort the top-level script if there is any chance that subsequent
# granules and swaths COULD complete successfully.  So, where in CSPP_MIRS I
# have aborted, I need to WARN - which means carry on with next and print
# message on $options{verbose} >= 1.
# 
# About processing order.  You have some options here.  I do not plan to offer
# recovery from retrieval phase failure by keeping successful preprocess phase
# data.  So this means I would like to check I have all I need for both preproc
# and retrieval before I kick off even preproc.  The PCF filenames for scripts
# from NOAA/STAR: NUCAPS_Preproc.pl and NUCAPS_Retrieval.pl are hard-coded to
# NUCAPS_Preproc.pl.PCF and NUCAPS_Retrieval.pl.PCF, respectively. The logs,
# NUCAPS_Preproc.pl.log and NUCAPS_Preproc.pl.log are appended with each run.
# The PSF files are over-written with each run.  Output files are encoded with
# granule date/time.  The upshot is that I can run like this ...
#
# Make $tmpdir; cd $tmpdir
#   cd swath_001
#     Gather precessing information for granules in @swath
#     Loop over granules in @swath
#       Make NUCAPS_Preproc.pl.PCF & NUCAPS_Retrieval.pl.PCF
#       Run NUCAPS_Preproc.pl
#       Run NUCAPS_Retrieval.pl
#       Move results up to $tmpdir, rename to avoid possible ambiguities
#     End Loop
#     Move log file up to $tmpdir, rename to avoid possible ambiguity
#     cd $tmpdir
#   cd swath_002
#     etc
#
# The over-arching idea is that, as you develop, wherever you get to, or 
# wherever it fails, there can be kept a temporary directory where you can go
# and run interactively with all required inputs and control files present.
# This helps development, debugging, and in investigating user error post
# distribution.
#
# A note on aggregation:  You cannot aggregate pre-processing and have NUCAPS
# work; you may even need to dis-aggregate (with nagg), in fact that is what I
# now do (i.e. disaggregate with nagg) whether it is needed or not; the typical
# use case has CrIS/ATMS SDR files as swath (overpass) length aggregates. This
# means I must use the CSPP patched nagg with knows how to disaggregate SCRIF
# files! 
#
# The final piece of the puzzle is to post-aggregate the NUCAPS-CCR-AR_* and
# NUCAPS-EDR_* 4-scan granules into overpass length files before posting
# up to WORK_DIR. This has been a bit troublesome as the 4-scan granules have
# been flattened to one horizontal dimension; but it makes little sense to just
# grow this one dimension when aggregating the retrievals into swaths. So I
# have reshaped as 30 FORs by as many scanlines as are present (4 per NUCAPS
# output netCDF4 file before post-aggregation).  So dimensions had to change.
# Also, in order to display the 3D (profile) variables in McIDAS-V, I needed to
# have a single vertical-levels profile, so 3D Pressure and Effective_Pressure
# were recast as 1D and all the duplicates were discarded (at every 2D data
# point, we did have the same set of pressure levels).  I have not changed the
# following 3D fields to be McIDAS-V imaging compatible: Cloud_Top_Pressure,
# Cloud_Top_Fraction.  There are 8 cloud levels supported and I would have to
# split both of these into 8 separately named 2D fields (Cloud_Top_Pressure_1,
# Cloud_Top_Pressure_2, etc), and that is messy. And because of how these data
# are stored, I would also have to do much more coding to achieve (all the
# other fields are processed analagous to running through sed and changing a
# couple of things; Cloud_Top_Pressure & Cloud_Top_Fraction need reshaping
# and managing in memory before writing out - a very different pattern).
# NOTE: The above might be a dangerous strategy if pressure levels change
# to be specific to the pixel. Better to save presures as a 3D cube and then
# figure out how to display later - fidelity more important than convenience
#
# Aronne suggested a sensible rule for aggregation.  If the user does not
# choose aggregate, the output granules are the same size as the input
# granules, otherwise they are aggregated to swath (i.e. overpass) length,
# or to 30 minutes, whichever is shorter.  At first blush, this makes a lot
# of sense. But if the input granules are aggregated ones, they have to be
# disaggregated in pre-processing (this is a NUCAPS requirement), so to get
# back to their input size, the outputs still need to be aggregated - and
# that means they have the data format of an aggregated file!  Currently,
# input files are disaggregated into one big bucket and their data times
# determine how to order into overpasses and, thus, into output files - so
# there are problems to solve here still to achieve fileOutSize==fileInSize.
# My view is that if you don't like how I aggregate in post-processing:
# Don't choose it, consult the NOAA documentation for file formats, and
# sort it out yourself.
#
# JED  Nov-2014, updated  Jul-2015, and again in Oct-2015.
#
# JED  Oct 2017 for version 2.0, now version-controlled by git.
#
# JED  Aug-2018 for version 2.1.
#


# Pragmata
# ````````
use 5.006;
use strict;
use warnings;


# Load Perl modules - standard modules only
# `````````````````````````````````````````
use POSIX;
use Storable;
use File::Spec;
use File::Copy;
use Data::Dumper;
use Getopt::Long;
use File::Basename;
use Cwd qw(abs_path);
use Encode qw(decode);
use Time::gmtime qw(:FIELDS);
use List::Util qw(sum min max);
use File::Path qw(mkpath rmtree);
use Time::Local qw(timegm_nocheck);
use File::Temp qw(tempfile tempdir);
use Scalar::Util qw(looks_like_number);
use I18N::Langinfo qw(langinfo CODESET);


# Global variables are all UPPERCASE throughout
# `````````````````````````````````````````````
our $AGG_WINDOW_MINUTES = 30; # 30-minute aggregation window for npp
our $SLEEP_GFS_MINUTES = 0;   # Disable wait for preferred GFS data
our $MAIN = basename($0);
defined $ENV{CSPP_NUCAPS_HOME} or die "$MAIN\:: CSPP_NUCAPS_HOME not set.\nAbort";
defined $ENV{CSPP_NUCAPS_REVNAME} or die "$MAIN\:: CSPP_NUCAPS_REVNAME not set.\nAbort";
defined $ENV{CSPP_NUCAPS_REVDATE} or die "$MAIN\:: CSPP_NUCAPS_REVDATE not set.\nAbort";
#our $HOME = File::Spec -> catfile($ENV{CSPP_NUCAPS_HOME}, $ENV{CSPP_NUCAPS_REVNAME}, 'CrISOPS');
our $HOME = File::Spec -> catfile($ENV{CSPP_NUCAPS_HOME}, $ENV{CSPP_NUCAPS_REVNAME}, 'OPS');
(-d $HOME) or die "$MAIN\:: CSPP_NUCAPS subdirectory $HOME not found.\nAbort";


# $H5DUMP points to supplied static-compiled h5dump (version 1.8.11)
# ``````````````````````````````````````````````````````````````````
our $H5DUMP = File::Spec -> catfile($ENV{'CSPP_NUCAPS_HOME'}, 'bin', 'h5dump');
(-e $H5DUMP) or die "$MAIN\:: h5dump $H5DUMP not found.\nAbort";
(-x $H5DUMP) or die "$MAIN\:: h5dump $H5DUMP not executable.\nAbort";


# $H5REPACK points to supplied static-compiled h5dump (version 1.8.11)
# ````````````````````````````````````````````````````````````````````
our $H5REPACK = File::Spec -> catfile($ENV{'CSPP_NUCAPS_HOME'}, 'bin', 'h5repack');
(-e $H5REPACK) or die "$MAIN\:: h5repack $H5REPACK not found.\nAbort";
(-x $H5REPACK) or die "$MAIN\:: h5repack $H5REPACK not executable.\nAbort";


# $WGRIB points to supplied static-compiled wgrib (v1.8.1.2a)
# ```````````````````````````````````````````````````````````
our $WGRIB = File::Spec -> catfile($ENV{'CSPP_NUCAPS_HOME'}, 'bin', 'wgrib');
(-e $WGRIB) or die "$MAIN\:: h5repack $WGRIB not found.\nAbort";
(-x $WGRIB) or die "$MAIN\:: h5repack $WGRIB not executable.\nAbort";


# $WGRIB2 points to supplied static-compiled wgrib2 (v0.1.9.5.1)
# ``````````````````````````````````````````````````````````````
our $WGRIB2 = File::Spec -> catfile($ENV{'CSPP_NUCAPS_HOME'}, 'bin', 'wgrib2');
(-e $WGRIB2) or die "$MAIN\:: h5repack $WGRIB2 not found.\nAbort";
(-x $WGRIB2) or die "$MAIN\:: h5repack $WGRIB2 not executable.\nAbort";


# $H5NAGG points to supplied static-compiled nagg
# ```````````````````````````````````````````````
# But you can point to your own nagg if you wish. Ours is Version 1.5.1
our $H5NAGG = File::Spec -> catfile($ENV{'CSPP_NUCAPS_HOME'}, 'bin', 'nagg');
(-e $H5NAGG) or die "$MAIN\:: nagg $H5NAGG not found.\nAbort";
(-x $H5NAGG) or die "$MAIN\:: nagg $H5NAGG not executable.\nAbort";


# Get the command line options, display help or revision if asked
# ```````````````````````````````````````````````````````````````
my $revision = "CSPP_NUCAPS v2.1.1 $ENV{CSPP_NUCAPS_REVDATE}";
my $pwd = getcwd();
my %options = get_commandline($revision, $pwd);
($options{verbose} >= 2) and pretty_dump('%options', %options);


# uuidgen is a system requirement
# ```````````````````````````````
our $UUIDGEN = `which uuidgen`; chomp($UUIDGEN);
(index($UUIDGEN, 'uuidgen') < 0) and
  die "$MAIN\:: Required utility uuidgen not found\nAbort"; 


# At present we are saying that lftp is a system requirement
# ``````````````````````````````````````````````````````````
our $LFTP = 'lftp';
($options{quarantine}) or do {
  system("$LFTP --version 1>/dev/null 2>/dev/null") == 0
    or die "$MAIN\:: Required utility lftp not found\nAbort"; };


# $NCDUMP points to supplied static-compiled ncdump (netcdf library version 4.3.0)
# ````````````````````````````````````````````````````````````````````````````````
our $NCDUMP = File::Spec -> catfile($ENV{'CSPP_NUCAPS_HOME'}, 'bin', 'ncdump');
($options{aggregate}) and do {
  (-e $NCDUMP) or die "$MAIN\:: $NCDUMP not found.\nAbort";
  (-x $NCDUMP) or die "$MAIN\:: $NCDUMP not executable.\nAbort"; };


# $NCGEN points to supplied static-compiled ncgen (netcdf library version 4.3.0)
# ``````````````````````````````````````````````````````````````````````````````
our $NCGEN = File::Spec -> catfile($ENV{'CSPP_NUCAPS_HOME'}, 'bin', 'ncgen');
($options{aggregate}) and do {
  (-e $NCGEN) or die "$MAIN\:: $NCGEN not found.\nAbort";
  (-x $NCGEN) or die "$MAIN\:: $NCGEN not executable.\nAbort"; };


# These are used for fileglobbing when INPUTFILES is a directory name
# ```````````````````````````````````````````````````````````````````
# Change to suit your site; for more control edit function validate_options(),
# or provide "<dirname>/<my_filemask>" on the command line as INPUTFILES; the
# "quotes" are important to prevent the OS expanding wildcard tokens. This is
# also where you can encode for different DB and DAAC naming conventions.
# You MAY be able to adapt this script to use for ISRP, i.e. have just ONE
# top-level Perl script for both, so leave noaa/metop support in %filemask
# for the present, just don't forget that this script does not yet support.
# WARNING: At this juncture I am not distingushing npp from j01 !!!
my %filemask = (j01 => '{GATMO,GCRSO,SCRIS,TATMS}_j01_*.h5',
                npp => '{GATMO,GCRSO,SCRIS,TATMS}_npp_*.h5',
                n18 => '{dummy1,dummy2,dummy3}l1b_noaa18*.l1b',
                n19 => '{dummy1,dummy2,dummy3}l1b_noaa19*.l1b',
                metopA => '{dummy1,dummy2,dummy3}l1b_M02*.l1b',
                metopB => '{dummy1,dummy2,dummy3}l1b_M01*.l1b');
($options{xfiles}) and do { # i.e. non-DB filenaming conventions
  %filemask = (j01 => '{GATMO,GCRSO,SCRIS,TATMS}_j01_*.h5',
               npp => '{GATMO,GCRSO,SCRIS,TATMS}_npp_*.h5',
               n18 => '*.{DUMMY1,DUMMY2,DUMMY3}.NN*',
               n19 => '*.{DUMMY1,DUMMY2,DUMMY3}.NP*',
               metopA => '*.{DUMMY1,DUMMY2,DUMMY3}.M2*',
               metopB => '*.{DUMMY1,DUMMY2,DUMMY3}.M1*'); };
# Distinguish SCRIS from SCRIF because they might be in same directory!
(index($options{mode}, 'HR') < 0) or do {
  $filemask{j01} =~ s/SCRIS/SCRIF/g;
  $filemask{npp} =~ s/SCRIS/SCRIF/g; };
# Add in cloud mask & height files.
($options{jrr} eq 'JRR') and do {
  $filemask{j01} =~ s/TATMS/TATMS,$options{geo_viirs},JRR-CloudHeight,JRR-CloudMask/g;
  $filemask{npp} =~ s/TATMS/TATMS,$options{geo_viirs},JRR-CloudHeight,JRR-CloudMask/g;
  $filemask{j01} =~ s/\.h5/\.{h5,nc}/g; # because the JRR are .nc
  $filemask{npp} =~ s/\.h5/\.{h5,nc}/g; };
($options{jrr} eq 'VIIRS') and do {
  $filemask{j01} =~ s/TATMS/TATMS,$options{geo_viirs},VICMO,VCTHO/g;
  $filemask{npp} =~ s/TATMS/TATMS,$options{geo_viirs},VICMO,VCTHO/g; };
# If ATMS processing is not selected, drop them from filemask!
(index($options{mode}, 'IRO') < 0) or do {
  $filemask{j01} =~ s/GATMO,//g;
  $filemask{j01} =~ s/,TATMS//g;
  $filemask{npp} =~ s/GATMO,//g;
  $filemask{npp} =~ s/,TATMS//g; };
# Debug information
($options{verbose} == 3) and pretty_dump('%filemask', %filemask);


# Parse and validate command line %options & make temporary working directory
# ```````````````````````````````````````````````````````````````````````````
my $sat = $options{sat};
%options = validate_options($filemask{$sat}, %options);
($options{verbose} >= 1) and pretty_dump('%options', %options);
my $tmpdir = tempdir('TMP_XXXXXX', DIR => $options{work},
  CLEANUP => $options{clean});
chdir($tmpdir) or die "$MAIN\:: Cannot chdir to temp dir $tmpdir\nAbort";
($options{verbose} >= 2) and print "chdir to working directory $tmpdir\n";


# Collect required files into /support/
# `````````````````````````````````````
# There is a problem with some processing options (LR_IRMW is what I have
# definitely discovered) whereby links made by the DAP system get messed
# up.  I plan to make the links myself in $PWD as it is at runtime, that
# being swath_NNN, but I am going to link to the files collected in the
# directory /support/ being made here.  When I create the PCF templates,
# in the routines that shortly follow, I will know what these files are,
# but I won't be able to put them in the swath_NNN directory because it
# (or they) do not yet exist.  So instead I will copy the files into
# /support/, make the file references in the PCF templates to files in
# $PWD, and then in function cspp_preprocess(), as soon at the swath_NNN
# directory is made, and chdir'ed into, make links out to all the files
# in ../support.
mkpath('support') or die "$MAIN\:: WORK_DIR/support cannot be made.\nAbort";
#my @steps = ('preproc');
#($options{subset}) and push(@steps, 'subset');
#($options{skipretrieval}) or push(@steps, 'retrieval');
foreach ( qw/preproc retrieval subset/ ) {
  mkpath("support/$_") or
    die "$MAIN\:: WORK_DIR/support/$_ cannot be made.\nAbort";
}


# Make the template PCF files at the $tmpdir level (this is $PWD)
# ```````````````````````````````````````````````````````````````
make_nucaps_preproc_pcf_template('NUCAPS_Preproc.pl.PCF.template',
  $options{sat}, $options{mode}, $options{jrr});
make_nucaps_retrieval_pcf_template('NUCAPS_Retrieval.pl.PCF.template',
  $options{sat}, $options{mode}, $options{fovs}, $options{reproc});
($options{subset}) and do {
  make_nucaps_subset_pcf_template('NUCAPS_Subset.pl.PCF.template',
    $options{sat}, $options{mode}); };


# Preprocess and stage data for NUCAPS system compatibility
# `````````````````````````````````````````````````````````
# I have added logic to disaggregate GMTCO or GMODO files if -jrr='JRR',
# and to copy the JRR-CloudMask/Height files across into input/, but I have
# not finished (or even started) the logic that makes sure that the NUCAPS
# preprocessor PCF is correctly configured. It gets ugly - the ATMS files
# have one set of file timecodes, the CRIS files have another, and now the
# VIIRS/JRR have yet another.  There is logic to deal with lining up the
# right ATMS and CRIS files [in function populate_npp_swath_info()], with
# results put in the %granule hashes that make up @swath.  This is where
# I would need to add the logic I need to get ready, and then I'd need to
# add some extra logic in the main loop below to get the files into the
# NUCAPS_Preproc.pl.PCF, made trickier because VIIRS_COUNT is variable
# and so the number of files to be swapped in is also variable. (For ATMS
# and CRIS, I have to figure which files I need, but it is always 3 ATMS
# and 1 CRIS file per PCF).
%options = cspp_preprocess(%options);
($options{verbose} == 3) and pretty_dump('%options', %options);


# Define dynamic data remote & local locations and file naming conventions
# ````````````````````````````````````````````````````````````````````````
# The root of the local directory for dynamic ancillary data is either defined
# on the command line or is assigned a default value. I am appending the agreed
# subdirectory naming convention to this root and copying it across to a hash
# that contains much of what I need to know to fetch and store ancillary data.
# See https://jpssdb.ssec.wisc.edu/cspp_v_2_0/ancillary/ for directory structure.
# I have added support for GRIB1 files and an alternative repo directory for
# reprocessing.  Also added support for 1/2 GFS 
my $usual_repo = File::Spec -> catfile($options{dynanc}, '%Y_%m_%d_%j');
my $repro_repo = File::Spec -> catfile($options{dynanc}, '%Y/%m');
my $local_repo = $usual_repo;
($options{reproc} == 1) and $local_repo = $repro_repo;
($options{reproc} == 2) and $local_repo = $repro_repo;
my $grib1_template = 'gblav.PGrbF%f.%y%m%d.%Hz';
my $grib2_template = 'gfs.t%H.%y%m%d.pgrbf%f';
($options{reproc} == 3) and $grib2_template = 'gfs.t%Hz.pgrb2.0p50.f0%f.%Y%m%d';
my $gfs_template = $grib2_template;
($options{reproc} == 1) and $gfs_template = $grib1_template;
my %dynanc_loc = (GFSPROTO => 'https://',
                  GFSSITE => 'jpssdb.ssec.wisc.edu',
                  GFSREMOTE => 'cspp_v_2_0/ancillary/%Y_%m_%d_%j',
                  GFSTEMPLATE => $gfs_template,
                  GFSLOCAL => $local_repo);


# Main processing loop in which $PWD is cycled through swath_nnn
# ``````````````````````````````````````````````````````````````
# Actually swath_ijk may exist and be loaded with links to satellite data,
# but be skipped in processing because populate_npp_swath_info() does not
# find a grouping of valid data to process.  If the --noclean has been set
# on command line, the temporary working directory will remain when this
# script terminates and in there will be bash scripts NUCAPS_Preproc.pl.bash
# and NUCAPS_Retrieve.pl.bash that you could use for debugging, say.
# They will just run the last made PCFs, i.e. the last granule.
# To add parallel processing of granules, I need to have a separate
# directory not just for each swath, but for each granule.  I want to
# add as little code as possible to do this, and to copy as little
# extra into the temporary working directory
sub count_done { 
  my @k = glob('granule_???/*.out');
  my $i = 0;
  foreach (glob('granule_???/*.out')) {
    $i++;
    my $s = substr($_, 8,3);
    my $j = int($s);
    print "$i  $s  $j"."\n";
    sleep 1;
    fileno($_[$j]) and close($_[$j]);
  }
  return($#k+1);
}
sub countNclose { 
  my $k = 0;
  foreach (glob('granule_???/*.out')) {
    $k++; my $j = int(substr($_, 8,3));
    fileno($_[$j]) and close($_[$j]);
  }
  return($k);
}
foreach my $wrkdir (@{$options{process_dir}}) { # $wrkdir is the swath
  chdir("$tmpdir/$wrkdir") or die "$MAIN\:: Cannot chdir to $wrkdir\nAbort";
  store(\%options, 'options.store') or die "$MAIN\:: Can't write options.store\n";
  #
  # npp and j01 CrIS/ATMS processing now supported
  # ``````````````````````````````````````````````
  (grep {$_ eq $sat} qw/npp j01/) and do {
    my @swath = populate_npp_swath_info(\%dynanc_loc, %options);
    ($#swath < 0) and do { # @swath holds the granules in this swath
      ($options{verbose} >= 1) and 
        warn "WARNING: No eligible granules in $wrkdir";
      next; };
    #
    # Process each granule in swath
    # `````````````````````````````
    my $start = time();
    my $secs_per_granule = 600;
    my $timeout_secs = int(($#swath+1)/$options{processors} + 0.999) * $secs_per_granule;
    my @fh = ();
    my ($i, $j, $k) = (-1, 0, 0);
    while ($k <= $#swath) {
      $k = count_done_and_close_filehandles(@fh);
      my $n = $j - $k;
      (($n < $options{processors}) and ($i < $#swath)) and do {
        $i = $j;  $j++;  $n = $j - $k;
        my $instance = sprintf("granule_%03d", $j);
        mkpath($instance) or
          die "$MAIN\:: INSTANCE_DIR $wrkdir/$instance cannot be made.\nAbort";
        chdir($instance) or
          die "$MAIN\:: Cannot chdir to $wrkdir/$instance\nAbort";
        my %granule = %{$swath[$i]};
        store(\%granule, 'granule.store') or die "$MAIN\:: Can't write granule.store\n";
        open ($fh[$j], '|nucaps_granule.pl');
        chdir('..');  # chdir up from granule level to swath level
      };
      my $diff = time() - $start;
      ($diff > $timeout_secs) and do {
        warn "WARNING: Some granules may not have completed before timeout\n";
        last; };
      sleep 1;  # I don't need this loop racing
    }
    my @movefiles = ();
    push(@movefiles, sort(glob("*.log")));
    push(@movefiles, sort(glob("*.PCF")));
    foreach (@movefiles) {
      move($_, '..') or do {
        ($options{verbose} >= 1) and warn "WARNING: Move failed: $!"; };
    }
  };
  #
  # No other missions presently supported !!!  ABORT
  # ````````````````````````````````````````````````
  (grep {$_ eq $sat} qw/npp j01/) or do {
    die "ERROR: MISSION $options{sat} not yet supported \nAbort";
  };
  #
  # Swath complete for any MISSION, copy files up to $tmpdir
  # ````````````````````````````````````````````````````````
  ##### By this artifice, I am leaving NUCAPS_ALL_*.nc * NUCAPS_CCR_*.nc in the
  ##### swath directory - and NUCAPS_ALL_*.nc is needed for the debug-useful
  ##### execNUCAPS.bash 
  # Why not move the netCDF files sooner?  Because I might be going to
  # post-aggregate (i.e. concatenate) the product files into swaths.
  # I now copy files out instead of move files out so that I have
  # the files I need in place for debugging purposes.
  ($options{nothing}) or do {
    my @copyfiles = ();
    push(@copyfiles, sort(glob("NUCAPS*.nc")));
    push(@copyfiles, sort(glob("NUCAPS*.txt")));
    push(@copyfiles, sort(glob("*.out")));
    if ($options{aggregate}) {
      # Aggregate NUCAPS-EDR and NUCAPS-CCR files and push to parent directory
      aggregate_nucaps_retrievals('.', '..', $revision);
      my @out = sort(glob("*.out"));
      my @first = split(/\_/, $out[0]);
      my @last = split(/\_/, $out[$#out]);
      my $outname = join ('_', @first[0..1], @last[2..$#last]);
      open(my $fhO, ">../$outname") or die "$MAIN\:: Cannot open $outname for write.\nAbort";
      foreach (@out) {
        open(my $fhI, $_) or die "$MAIN\:: Cannot open $_ for read.\nAbort";
        while (<$fhI>) { chomp; print $fhO "$_\n"; }
        close($fhI) or die "$MAIN\:: Cannot close $_.\nAbort";
      }
      close($fhO) or die "$MAIN\:: Cannot close $outname.\nAbort";
    }
    foreach (@copyfiles) {
      copy($_, '..') or do {
        ($options{verbose} >= 1) and warn "WARNING: Copy failed: $!"; };
    }
  };
  chdir($tmpdir) or die "$MAIN\:: Cannot chdir to temp dir $tmpdir\nAbort";
  #
  # Swath complete, move NUCAPS-*.nc files up to $options{work}, concat log files
  # `````````````````````````````````````````````````````````````````````````````
  # This part used to happen after all swaths complete, now I push data up to
  # $options{work} as each swath completes, which will be one NUCAPS-EDR and
  # one NUCAPS-CCR-AR file IF aggregation is on, otherwise many small files.
  ($options{nothing}) or do {
    $options{clean} and system("rm -rf $wrkdir"); # save disk space on big jobs
    foreach (sort(glob("{NUCAPS*.nc,*.out}"))) {
      move($_, '..') or do {
        ($options{verbose} >= 1) and warn "WARNING: Move failed: $!"; };
    }
  };
}


# All done, chdir to starting directory so that $tmpdir can be removed
# ````````````````````````````````````````````````````````````````````
if (not $options{clean}) {
  print "--noclean set, reminder: tmpdir is ...\n$tmpdir\n";
}
chdir($pwd) or die "$MAIN\:: Cannot chdir back to $pwd\nAbort";
exit(0);


###############################################################################
#                                                                             #
#         User-defined functions - they are listed alphabetically             #
#                                                                             #
#   NOTE: My editor can collapse code between matching {} braces, this        #
#   explains the strange formatting - I can collapse each function to see     #
#   just the function name and an example of how to call it; and expand       #
#   to see the more detailed documentation and to edit.      JED Nov 2013     #
#                                                                             #
###############################################################################

# ------------------
  sub aggregate_data
# ==================
# @data = aggregate_data($varname, @granule_filenames);
  {
    # @data is a vector of strings that comprise the data from
    # all @granule_filenames for specific $varname and in a form
    # that can be appended to the .cdl under creation.
    # NOTE: I think it is cheating, but by changing the Pressure and
    # Effective_Pressure dimensions to 1D [in aggregate_varatts()] it appears
    # to be unnecessary to limit the pressure data to one profile as
    # ncgen discards all but the first profile - which is basically what I
    # want without all the work of hacking the data stream.
    my $varname = shift;
    my @files = sort(@_);
    my $name = (caller(0))[3];
    $name =~ s/main/$MAIN/g;
    my @data = ();
    foreach my $fullname (@files) {
      my $key = (split('\.', basename($fullname)))[0];
      my $cmd = "$NCDUMP -v $varname $fullname |";
      open(my $fh, $cmd) or die "$name\: Could not run command $cmd\nAbort";
      while (<$fh>) { chomp; ( /data:/ ) and last; }
      my @buffer = ();
      while (<$fh>) { chomp; ( /}/ ) and last; push(@buffer, $_); }
      close($fh);
      foreach (@buffer) {
        my $value = '';
        my @parts = split(/=/, $_);
        ($#parts == -1) and next;
        ($#parts ==  0 and /=/) and next;
        ($#parts ==  0) and push(@data, (split(/;/, $parts[0]))[0]);
        ($#parts ==  1) and push(@data, (split(/;/, $parts[1]))[0]);
      }
      ($fullname eq $files[-1]) and $data[-1] = $data[-1]." ;";
      ($fullname ne $files[-1]) and $data[-1] = $data[-1].",";
    }
    return @data;
  }

# ----------------------
  sub aggregate_fileroot
# ======================
# $fileroot = aggregate_fileroot(@granule_filenames);
  {
    # The returned fileroot includes neither the file path nor the extension
    # It is up to the user to check that @granule_filenames has at least
    # one element. If the filenames do not break on '_' as expected, this
    # routine will bail out with an error message.
    my @files = sort(@_);
    my $name = (caller(0))[3];
    $name =~ s/main/$MAIN/g;
    my $first = basename($files[0]);
    my $last = basename($files[$#files]);
    my @first = split(/\_/, $first);
    my @last = split(/\_/, $last);
    ($#first == 5) or die "$name\: first file $first invalid\nAbort";
    ($#last == 5) or die "$name\: last file $last invalid\nAbort";
    gmtime();
    my $creation = strftime("c%Y%m%d%H%M%S0", $tm_sec, $tm_min, $tm_hour,
      $tm_mday, $tm_mon, $tm_year);
    my $fileroot = join ( '_', @first[0..3], $last[4], $creation);
    $fileroot =~ s/NUCAPS-/NUCAPS-AGG-/g;
    return $fileroot;
  }

# ----------------------
  sub aggregate_globatts
# ======================
# %globatt = aggregate_globatts($attname, $opcode, \%headers, %globatt);
  {
    # %globatt is updated with a key=$attname and a value=full attrbute line
    # Supported opcodes are 'start', 'finish', 'list', 'polygon'
    # NOTE: There is nothing to be gained in call this routine if the number
    # of %input_cls is just ONE. In fact there would have been no purpose in
    # even populating %input_cls  SO  this routine ASSUMES there are at least
    # TWO files represented in %input_cdls.
    my ($attname, $opcode) = (shift, shift);
    my %headers = %{(shift)};
    my (%globatt) = @_;
    my @values = ();
    foreach my $key (sort keys %headers) {
      my @buffer = @{$headers{$key}};
      my $i = 0;
      foreach (@buffer) { ( /global attributes:/ ) and last; $i++; }
      my $j = $i;
      foreach (@buffer[$i..$#buffer]) { ( /$attname/ ) and last; $j++; }
      push(@values, $buffer[$j]);
    }
    ($opcode eq 'start') and $globatt{$attname} = $values[0];
    ($opcode eq 'finish') and $globatt{$attname} = $values[-1];
    ($opcode eq 'list') and do {
      my @part = split(/=/, $values[0]);
      my $item = trim_string((split(';', $part[1]))[0]);
      looks_like_number($item) or $item = (split(/\"/, $item))[1];
      $globatt{$attname} = sprintf("%s  = \"LIST((%s", $part[0], $item);
      foreach (@values[1..$#values]) {
        $item = trim_string((split(';', (split(/=/, $_))[1]))[0]);
        looks_like_number($item) or $item = (split(/\"/, $item))[1];
        $globatt{$attname} = sprintf("%s, %s", $globatt{$attname}, $item);
      }
      $globatt{$attname} = sprintf("%s))\" ;", $globatt{$attname});
    };
    ($opcode eq 'polygon') and do {
      my %polygon = ();
      for (my $i=0; $i<=$#values; $i++) {
        my $q1 = (split('\(\(', $values[$i]))[1];
        my $q2 = (split('\)\)', $q1))[0];
        my @q3 = split(',', $q2);
        @{$polygon{$i}} = @q3;
      }
      my $n = scalar(keys %polygon) - 1; # largest key in %polygon
      my $newvalue = "\t\t".':geospatial_bounds = "POLYGON((';
      $newvalue = $newvalue.join(',', @{$polygon{0}}[0..2]);
      foreach my $key (sort keys %polygon) {
        $newvalue = join(',', $newvalue, @{$polygon{$key}}[3..4]);
      }
      $newvalue = join(',', $newvalue, @{$polygon{$n}}[5..6]);
      foreach my $key (reverse sort keys %polygon) {
        $newvalue = join(',', $newvalue, @{$polygon{$key}}[7..8]);
      }
      $newvalue = $newvalue.'))" ;';
      $globatt{$attname} = $newvalue;
    };
    return %globatt;
  }

# --------------------
  sub aggregate_header
# ====================
# @header = aggregate_header(\%headers, \%newDim, \%varatt, %globatt);
  {
    # @header is the aggregated header based on the individual granule
    # headers in %headers, the new dimensions information in %newDim, the
    # updates to variable attributes in %varatt and the global attributes
    # modified where appropriate to represent the entire aggregated file
    # in %globatt.
    my %headers = %{(shift)};
    my %newDim = %{(shift)};
    my %varatt = %{(shift)};
    my (%globatt) = @_;
    my @header = ();
    my $filekey =  (sort(keys %headers))[0];
    my @buffer = @{$headers{$filekey}};
    my $section = 0;
    foreach (@buffer) {
      /$filekey/ and next; # because name has been replaced with $ccr_aggregate
      /Number_of_CrIS_FORs = 120 ;/ and next;
      /Scan_Line/ and next;
      /\tint CrIS_FORs/ and next;
      /\t\tCrIS_FORs:/ and next;
      /dimensions:/ and $section = 1;
      /variables:/ and $section = 2;
      /global attributes:/ and $section = 3;
      my $value = $_;
      ($section == 1) and do {
        /dimensions:/ and do { foreach (@{$newDim{dim}}) { $value = $value."\n".$_; } }; };
      ($section == 2) and do {
        /variables:/ and do { foreach (@{$newDim{var}}) { $value = $value."\n".$_; } };
        my $verify = 'N/A';
        if ( /=/ ) {
          $verify = trim_string((split(/=/, $_))[0]);
        } else {
          ( /\t/ ) and do {
            $verify = trim_string((split(/ /, $_))[1]);
            $verify =~ s/\(/\\\(/g;  # Need to escape the brackets otherwise the
            $verify =~ s/\)/\\\)/g;  #   search/replace fails in aggregate_header()
          };
        }
        foreach my $key (sort keys %varatt) {
          ($key eq $verify) and do { $value = $varatt{$key}; last; };
        }
      };
      ($section == 3) and do {
        foreach my $key (sort keys %globatt) {
          /$key/ and do { $value = $globatt{$key}; last; };
        }
      };
      push (@header, $value);
    }
    my $value = sprintf("\t\t:aggregated_by = \"%s\" ;", $globatt{aggregate});
    push (@header, $value);
    gmtime();
    my $creation_date = sprintf("%04d-%02d-%02d %02d:%02d:%02d GMT",
      $tm_year+1900, $tm_mon+1, $tm_mday, $tm_hour, $tm_min, $tm_sec);
    $value = sprintf("\t\t:creation_date = \"%s\" ;", $creation_date);
    push (@header, $value);
    return @header;
  }

# --------------------
  sub aggregate_newDim
# ====================
#  %newDim = aggregate_newDim(@granule_filenames);
  {
    # %newDim is a hash containg the extra information needed to build
    # an extra dimension in the aggregated file, that being simply the
    # number of granules.  It has three keys, {dim} goes in the dimensions:
    # section of the .cdl file, {var} goes in the variables: section and
    # {dat} in the data: section.
    my @files = @_;
    my $n = $#files + 1;
    my $p = 30;
    my $l = $n * 120 / $p;
    my $m = $l * $p;
    my %info = ();
    my $dimy = 'Number_of_CrIS_scans';
    my $dimx = 'Number_of_CrIS_FOR';
    @{$info{dim}} = ();
    push(@{$info{dim}}, sprintf("\t%s = %d ;", $dimy, $l));
    push(@{$info{dim}}, sprintf("\t%s = %d ;", $dimx, $p));
    @{$info{var}} = ();
    my $var = 'Scan_Number';
    push(@{$info{var}}, "\tint $var($dimy, $dimx) ;");
    push(@{$info{var}}, "\t\t$var:long_name = \"Current scan in aggregate\" ;");
    push(@{$info{var}}, sprintf("\t\t%s:valid_range = %d, %d ;", $var, 1, $l));
    push(@{$info{var}}, sprintf("\t\t%s:_FillValue = -9999 ;", $var));
    my @lin = sort {$a <=> $b} ((1..$l) x $p);
    $info{dat} = sprintf(" %s = 1", $var);
    for (my $j=1; $j<$m; $j++) {
      $info{dat} = sprintf("%s, %d", $info{dat}, $lin[$j]);
    }
    $info{dat} = sprintf("%s ;\n", $info{dat});
    $var = 'FOR_Number';
    push(@{$info{var}}, "\tint $var($dimy, $dimx) ;");
    push(@{$info{var}}, "\t\t$var:long_name = \"Current F.O.R. in aggregate\" ;");
    push(@{$info{var}}, sprintf("\t\t%s:valid_range = %d, %d ;", $var, 1, $p));
    push(@{$info{var}}, sprintf("\t\t%s:_FillValue = -9999 ;", $var));
    my @pix = (1..$p) x $l;
    $info{dat} = sprintf("$info{dat}\n %s = 1", $var);
    for (my $j=1; $j<$m; $j++) {
      $info{dat} = sprintf("%s, %d", $info{dat}, $pix[$j]);
    }
    $info{dat} = sprintf("%s ;", $info{dat});
    return %info;
  }

# ------------------------------
  sub aggregate_nucaps_filegroup
# ==============================
#  aggregate_nucaps_filegroup($destDir, $revision, @files);
  {
    # $destDir is the destination directory for aggregated output files
    # $revision is the revision number of this script for global attributes
    # @files is the full pathname of the CCR or EDR files, the
    # code in this routine works for both sets of files!
    my ($destDir, $revision) = (shift, shift);
    my @files = @_;
    my $name = (caller(0))[3];
    $name =~ s/main/$MAIN/g;
    my $aggregated_filename = aggregate_fileroot(@files);
    # read headers of all files in filegroup
    my %headers = ();
    foreach my $fullname (@files) {
      my $key = (split('\.', basename($fullname)))[0];
      my $cmd = "$NCDUMP -h $fullname |";
      open(my $fh, $cmd) or die "$name\: Could not run command $cmd\nAbort";
      my @buffer = ();
      while (<$fh>) { chomp; push(@buffer, $_); }
      close($fh);
      @{$headers{$key}} = @buffer[0..($#buffer-1)]; # clip the trailing '}'
    }
    # define the new dimension that need to be inserted in aggregate
    my %newDim = aggregate_newDim(@files); # UNLIMITED dimension
    # variable attributes that need changing are keyed by their names
    my %varatt = ();
    %varatt = aggregate_varatts('coordinates', \%headers, %varatt);
    %varatt = aggregate_varatts('dimensions', \%headers, %varatt);
    %varatt = aggregate_varatts('quality', \%headers, %varatt);
    ### pretty_dump('%varatt', %varatt);
    # variable names for later writing into data section
    my @varnames = aggregate_varnames(%headers);
    ### print "@varnames\n";
    # global attributes that need changing are also keyed by their names
    my %globatt = (); # values are opcodes that get replaced by attributes
    $globatt{starting_and_ending_orbit_node} = 'list' ;
    $globatt{date_created} = 'start' ;
    $globatt{Metadata_Link} = 'list' ;
    $globatt{source} = 'list' ;
    $globatt{start_orbit_number} = 'start' ;
    $globatt{end_orbit_number}   =  'finish' ;
    $globatt{day_night_data_flag} = 'list' ;
    $globatt{ascend_descend_data_flag} = 'list' ;
    $globatt{time_coverage_start} = 'start' ;
    $globatt{time_coverage_end}   =  'finish' ;
    $globatt{geospatial_first_scanline_first_fov_lat} = 'start' ;
    $globatt{geospatial_first_scanline_last_fov_lat}  =  'finish' ;
    $globatt{geospatial_first_scanline_first_fov_lon} = 'start' ;
    $globatt{geospatial_first_scanline_last_fov_lon}  =  'finish' ;
    $globatt{geospatial_last_scanline_first_fov_lat}  = 'start' ;
    $globatt{geospatial_last_scanline_last_fov_lat}   =  'finish' ;
    $globatt{geospatial_last_scanline_first_fov_lon}  = 'start' ;
    $globatt{geospatial_last_scanline_last_fov_lon}   =  'finish' ;
    $globatt{geospatial_bounds} = 'polygon' ;
    $globatt{aggregate} = "$revision ($name)";
    foreach my $key (keys %globatt) {
      my $opcode = $globatt{$key};
      %globatt = aggregate_globatts($key, $opcode, \%headers, %globatt);
    }
    ### pretty_dump('%globatt', %globatt);
    # we can now write the header (but not the data, thus far uninterrogated)
    my $agg_cdl = join('.', $aggregated_filename, 'cdl');
    my $agg_nc4 = join('.', $aggregated_filename, 'nc');
    my $out_cdl = File::Spec -> catfile($destDir, $agg_cdl); 
    my $out_nc4 = File::Spec -> catfile($destDir, $agg_nc4); 
    # open the output file and write the "salutation"
    open (my $FH, ">$out_cdl") or die "$name\: Cannot open $out_cdl for writing\nAbort";
    print $FH "netcdf $aggregated_filename {\n";
    # construct and write the aggregated header
    my @header = aggregate_header(\%headers, \%newDim, \%varatt, %globatt);
    # REMOVE ME is a tag to replace unwanted coordinates on Pressure & Effective_Pressure
    foreach (@header) { ( /REMOVE ME/ ) and next; print $FH "$_\n"; }
    # write the new dimension data
    print $FH "data:\n\n";
    print $FH $newDim{dat}."\n";
    # outer loop over all remaining variables (in @varnames)
    foreach my $varname (@varnames) {
      my @data = ();
      if (grep {$_ eq $varname} qw/quality_information CrIS_Frequencies/) {
        @data = aggregate_data($varname, $files[0]);
      } else {
        @data = aggregate_data($varname, @files);
      }
      $data[0] = "\n$varname = ".$data[0];
      foreach (@data) { print $FH "$_\n"; }
    }
    # write the closing brace and close file
    print $FH "}\n";
    close($FH);
    # convert the .cdl file to netCDF4
    my $cmd = "$NCGEN -k4 -o $out_nc4 $out_cdl";
    (system($cmd) == 0) or warn "$name\: Could not run command $cmd\n";
    return;
  }

# -------------------------------
  sub aggregate_nucaps_retrievals
# ===============================
#  aggregate_nucaps_retrievals($sourceDir, $destDir, $revision);
  {
    # This is the top level routine for nucaps output netCDF4 file
    # aggregation that needs to be slotted into run_nucaps.pl
    my ($sourceDir, $destDir, $revision) = (shift, shift, shift);
    my $name = (caller(0))[3];
    $name =~ s/main/$MAIN/g;
    my @ccr_files = (sort(glob("$sourceDir/NUCAPS-CCR-AR*.nc")));
    my @edr_files = (sort(glob("$sourceDir/NUCAPS-EDR*.nc")));
    aggregate_nucaps_filegroup($destDir, $revision, @ccr_files);
    aggregate_nucaps_filegroup($destDir, $revision, @edr_files);
    return;
  }

# ---------------------
  sub aggregate_varatts
# =====================
# %varatt = aggregate_varatts($opcode, \%headers, %varatt);
  {
    # %varatt is a hash of replacements to make to variable attributes
    # $opcode is identifies the search strategy, basically.
    # %varatt is updated with a key=$attname and a value=full attribute line
    # Supported opcodes are 'quality', 'dimensions' and 'coordinates'.
    #
    # $opcode 'quality' triggers a very specific response, namely it searches
    # for and updates quality_information and its attributes that are counts
    # or statistics.
    #
    # $opcode 'coordinates' triggers a search for every occurance of 
    # 'coordinates = "Time Latitude Longitude"' and makes into 
    # 'coordinates = "Latitude Longitude" '
    # It also drops the attributes Pressure:coordinates and
    # Effective_Pressure:coordinates
    #
    # $opcode 'dimensions' changes 1D horizontal space dimension into
    # 2D AND makes vertical dimension 1D.
    my $opcode = shift;
    my %headers = %{(shift)};
    my (%varatt) = @_;
    my $name = (caller(0))[3];
    $name =~ s/main/$MAIN/g;
    ($opcode eq 'coordinates') and do {
      foreach my $key (sort keys %headers) {
        my @buffer = @{$headers{$key}};
        foreach (@buffer) {
          ( /\t\t/ ) or next;
          ( /coordinates = \"Time / ) and do {
            my $key = trim_string((split(/=/, $_))[0]);
            my $value = $_;
            $value =~ s/Time //g;
            $varatt{$key} = $value;
          };
        }
        $varatt{'Pressure:coordinates'} = 'REMOVE ME';
        $varatt{'Effective_Pressure:coordinates'} = 'REMOVE ME';
        last; # Only need to pass through one header for this task
      }
    };
    ($opcode eq 'dimensions') and do {
      foreach my $key (sort keys %headers) {
        my @buffer = @{$headers{$key}};
        foreach (@buffer) {
          ( /\t\t/ ) and next;
          ( /CrIS_Frequencies\(Number_of_CrIS_Frequencies\)/ ) and next;
          ( /\(/) or next;
          ( /\t/ ) and do {
            my $key = trim_string((split(/ /, $_))[1]);
            $key =~ s/\(/\\\(/g;  # Need to escape the brackets otherwise the
            $key =~ s/\)/\\\)/g;  #   search/replace fails in aggregate_header()
            my $value = $_;
            # exception processing to make vertical coordinates one dimensional
            $value =~ s/float Pressure\(Number_of_CrIS_FORs, /float Pressure\(/g;
            $value =~ s/float Effective_Pressure\(Number_of_CrIS_FORs, /float Effective_Pressure\(/g;
            # general processing to make 1D horizontal space into 2D space
            $value =~ s/Number_of_CrIS_FORs/Number_of_CrIS_scans, Number_of_CrIS_FOR/g;
            $varatt{$key} = $value;
          };
        }
        last; # Only need to pass through one header for this task
      }
    };
    ($opcode eq 'quality') and do {
      my ($Q1, $Q2, $Q3, $Q4) = ('N/A', 'N/A', 'N/A', 'N/A');
      my ($k1, $k2, $k3, $k4) = ('N/A', 'N/A', 'N/A', 'N/A');
      my (@q2, @q3, @q4) = ((), (), ());
      foreach my $key (sort keys %headers) {
        my @buffer = @{$headers{$key}};
        my $i = 0;
        foreach (@buffer) {
          $i++; ( /char quality_information/ ) and last;
        }
        $Q1 = $buffer[$i];
        $k1 = trim_string((split(/=/, $Q1))[0]);
        $Q2 = $buffer[$i+1];
        $k2 = trim_string((split(/=/, $Q2))[0]);
        my $num = trim_string((split(';', (split(/=/, $Q2))[1]))[0]);
        is_integer($num) or 
          die "$name\: total_number_retrievals non-integer\nAbort";
        push(@q2, $num);
        $Q3 = $buffer[$i+2];
        $k3 = trim_string((split(/=/, $Q3))[0]);
        my $opt = trim_string((split('f', (split(/=/, $Q3))[1]))[0]);
        looks_like_number($opt) or 
          die "$name\: percentage_optimal_retrievals not a number\nAbort";
        push(@q3, $opt);
        $Q4 = $buffer[$i+3];
        $k4 = trim_string((split(/=/, $Q4))[0]);
        my $bad = trim_string((split('f', (split(/=/, $Q4))[1]))[0]);
        looks_like_number($bad) or 
          die "$name\: percentage_optimal_retrievals not a number\nAbort";
        push(@q4, $bad);
      }
      my $NUM = sum(@q2);
      my $OPT = sprintf(" %.2ff ;", 0);
      my $BAD = sprintf(" %.2ff ;", 0);
      ($NUM > 0) and do {
        my ($WOPT, $WBAD) = (0, 0);
        for (my $j=0; $j<=$#q2; $j++) {
          $WOPT += $q2[$j] * $q3[$j];
          $WBAD += $q2[$j] * $q4[$j];
        }
        $OPT = sprintf(" %.2ff ;", $WOPT/$NUM);
        $BAD = sprintf(" %.2ff ;", $WBAD/$NUM);
       };
      $Q1 =~ s/granule/aggregate/g;
      $Q2 = join ('=', (split(/=/, $Q2))[0], ' '.$NUM.' ;');
      $Q3 = join ('=', (split(/=/, $Q3))[0], $OPT);
      $Q4 = join ('=', (split(/=/, $Q4))[0], $BAD);
      $varatt{$k1} = $Q1;
      $varatt{$k2} = $Q2;
      $varatt{$k3} = $Q3;
      $varatt{$k4} = $Q4;
    };
    return %varatt;
  }

# ----------------------
  sub aggregate_varnames
# ======================
# @varnames = aggregate_varnames(%headers);
  {
    # @varnames is a list of variable names taken from just the
    # first file whose header has been read into %headers
    my (%headers) = @_;
    my $name = (caller(0))[3];
    $name =~ s/main/$MAIN/g;
    my @varnames = ();
    foreach my $key (sort keys %headers) {
      my @buffer = @{$headers{$key}};
      foreach (@buffer) {
        ( /global attributes:/ ) and last;
        ( /\=/ ) and next;    # rejects dimensions
        ( /\t\t/ ) and next;  # rejects attributes
        ( /\t/ ) and do {     # they start with one tab
          my $value = (split(/ /, $_))[1];
          $value  = (split(/\(/, $value))[0];
          ($value eq 'Scan_Line') and next;
          ($value eq 'CrIS_FORs') and next;
          push(@varnames, $value);
        };
      }
      last; # Only need to pass through one header for this task
    }
    return @varnames;
  }

# ------------------------------------
  sub count_done_and_close_filehandles
# ====================================
# $k = count_done_and_close_filehandles(@fh);
  {
    # This is called from within the loop over granules within a swath
    # (overpass) in MAIN. @fh is the array of filehandles, the index of
    # which refers to the granules in the loop (this is 1-based indexing).
    # I do not have to close a filehandle once its granule has completed,
    # but this accumulates defunct processes that clear only when the main
    # script completes. The danger is that a process does not complete, then
    # the script will hang here waiting.
    my $k = 0;
    foreach (glob('granule_???/done.flag')) {
      $k++; my $j = int(substr($_, 8, 3));
      fileno($_[$j]) and close($_[$j]);
    }
    return($k);
  }


# -------------------
  sub cspp_preprocess
# ===================
# %options = cspp_preprocess(%options);
  {
    # On input, %options comprises some command line parameters plus other
    # processing options, including defaults for unspecified command line
    # values and extra stuff defined in main to guide processing. On output,
    # %options includes a field $options{process_dir} that points to swath
    # directories.  In this function I make swath directories and populate
    # with input data, or links to the input data, as appropriate.  The reason
    # I do not simply link to the user supplied input directory is that the
    # input data may not be grouped into swaths and/or may not be fomatted
    # as I require, either internally or in terms of file naming convention.
    my (%options) = @_;
    my $name = (caller(0))[3];
    $name =~ s/main/$MAIN/g;
    my $sat = $options{sat};
    my @process_dir = ();
    # npp/j01 CrIS/ATMS processing - this is all we have right now !
    (grep {$_ eq $sat} qw/npp j01/) and do {
      mkpath('links') or die "$name\: WORK_DIR/links cannot be made.\nAbort";
      foreach (@{$options{input_files}}) {
        my $target = File::Spec -> catfile($options{input_dir}, $_);
        my $linkname = File::Spec -> catfile('links', $_);
        symlink($target, $linkname) or die "$name\: Cannot make link to $target\nAbort";
      }
      my @fields = split(/\_/, basename(@{$options{input_files}}));
      my $origin = $fields[7];
      my $domain = (split(/\./, basename($fields[8])))[0];
      # If $options{break}, which is the default, we run file through nagg
      # We do this to be sure of presenting 32-second granules to NUCAPS
      if ($options{break}) {
        mkpath('input') or die "$name\: WORK_DIR/input cannot be made.\nAbort";
        # This OK because @{$options{input_files}} already globbed with correct mask!
        foreach my $sensor (qw/TATMS SCRIS SCRIF/) {
          my %nagg = ();
          my $globber = File::Spec -> catfile('links', "$sensor*.h5");
          foreach (sort(glob($globber))) {
            my $mask = substr(basename($_), 10, 34);
            ($sensor eq 'TATMS') and $mask = "GATMO_$sat\_$mask*.h5";
            ($sensor eq 'SCRIS') and $mask = "GCRSO_$sat\_$mask*.h5";
            ($sensor eq 'SCRIF') and $mask = "GCRSO_$sat\_$mask*.h5";
            my @geo = glob("links/$mask");
            ($#geo > -1) and $nagg{$_} = $geo[0];
          }
          foreach my $key (sort keys %nagg) {
            my $cmd = "$H5NAGG -S -t $sensor -n 1 -d input -O $origin -D $domain $key $nagg{$key}";
            ($options{verbose} < 3) and $cmd = $cmd." > /dev/null";
            system($cmd) == 0 or die "$name\: Could not run command $cmd\nAbort";
          }
        }
      } else {
        # Instead of loading directory 'input' with the 32-second granule outputs of
        # nagg opersating on the linked .h5 files in 'links', we assume those linked
        # files are already 32s and make the 'input' directory a link to them.
        symlink('links', 'input') or die "$name\: Cannot make link to links\nAbort";
        # NOTE: This may have repercussion for handling the VIIRS data, directly below,
        # but that has not yet been implemented so you can review when it is.
      }
      # If --jrr='JRR' the I will have JRR-CloudHeight, JRR-CloudMask and GMTCO or GMODO
      # files in links/ that need to come across disaggregated to inputs/.  I have no
      # expectation that nagg is going to work with JRR files, so I am going to have to
      # assume that they are already presented as 32s granules. The GEO file (whether it
      # is GMTCO or GMODO is stored in $options{geo_viirs}) can be disaggreated into input/
      # where it needs to be or not.
      ($options{jrr} eq 'JRR') and do {
        my %nagg = ();
        my $globber = File::Spec -> catfile('links', "$options{geo_viirs}*.h5");
        foreach (sort(glob($globber))) {
          my $mask = substr(basename($_), 10, 34);
          $mask = "$options{geo_viirs}_$sat\_$mask*.h5";
          my @geo = glob("links/$mask");
          ($#geo > -1) and $nagg{$_} = $geo[0];
        }
        foreach my $key (sort keys %nagg) {
          my $cmd = "$H5NAGG -g $options{geo_viirs} -n 1 -d input -O $origin -D $domain $key $nagg{$key}";
          ($options{verbose} < 3) and $cmd = $cmd." > /dev/null";
          system($cmd) == 0 or die "$name\: Could not run command $cmd\nAbort";
        }
        $globber = File::Spec -> catfile('links', "JRR-Cloud*.nc");
        foreach (sort(glob($globber))) {
          copy($_, 'input') or do {
            ($options{verbose} >= 1) and warn "WARNING: Copy failed: $!"; };
        }
      };
      my $globber = File::Spec -> catfile('input', '*.h5');
      my @input_files = ();
      foreach (sort(glob($globber))) { push(@input_files, basename($_)); }
      # I want to identify the granules which make up continuous swaths and have
      # them (or rather links to them) in separate, numbered swath directories under
      # the temporary working directory (which is currently, and by design, $PWD).
      my %swaths = npp_granules_to_swaths(@input_files);
      ($options{verbose} > 2) and pretty_dump('%swaths', %swaths);
      my $iswath = 0;
      foreach my $key (sort keys %swaths) {
        my $granules = sprintf("swath_%03d", ++$iswath);
        mkpath($granules) or die "$name\: WORK_DIR $granules cannot be made.\nAbort";
        foreach (@{$swaths{$key}}) {
          my $target = File::Spec -> catfile('../input', $_);
          if (0==1) { # This cannot occur - just a repack file logic placeholder
            my $cmd = "$H5REPACK -n -f NONE $target $granules/$_";
            system($cmd) == 0 or die "$name\: Could not run command $cmd\nAbort";
          } else {
            my $linkname = File::Spec -> catfile($granules, $_);
            symlink($target, $linkname) or die "$name\: Cannot make link to $target\nAbort";
          }
        }
        (grep {$_ eq $options{mode}} qw/HR_IRMW LR_IRMW/) and do {
          my @gatmo = sort(glob("$granules/GATMO*"));
          ($#gatmo > -1) or do {
            warn "WARNING: No GATMO found to process in /$granules";
            next; };
          my @tatms = sort(glob("$granules/TATMS*"));
          ($#tatms == $#gatmo) or do {
            warn "WARNING: Mis-match in TATMS/GATMO file count in /$granules";
            next; };
        };
        my @gcrso = sort(glob("$granules/GCRSO*"));
        ($#gcrso > -1) or do {
          warn "WARNING: No GCRSO found to process in /$granules";
          next; };
        my @scris = sort(glob("$granules/SCRI*"));
        ($#scris == $#gcrso) or do {
          warn "WARNING: Mis-match in SCRI/GCRSO file count in /$granules";
          next; };
        push(@process_dir, $granules);
      }
    };
    (grep {$_ eq $sat} qw/npp j01/) or do {
      die "$name\: No support for MISSION $sat\nAbort";
    };
    @{$options{process_dir}} = @process_dir;
    return %options;
  }

# -------------------
  sub get_commandline
# ===================
# %options = get_commandline($revision, $pwd);
  {
    # If things go bad in here, jump to usage() which terminates with exit(9).
    # Note that I had to hack in the Python-like -vv & -vvv support.
    # This routine assumes standard module Getopt::Long has been loaded.
    # abs_path() might be dangerous for the purpose of setting the work (and,
    # equivalently, the output) directory - see this post:
    # http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=257568
    my ($rev, $pwd) = (shift, shift);
    my $name = (caller(0))[3];
    $name =~ s/main/$MAIN/g;
    my $codeset = langinfo(CODESET);
    @ARGV = map{decode $codeset, $_} @ARGV;
    foreach (@ARGV) { s/\?/-\?/g; }
    my %options = ();
    GetOptions(\%options,
      'version|V',
      'help|?',
      'nothing|n',
      'input_files|i=s',
      'sat|s=s',
      'mode|m=s',
      'work|w=s',
      'dynanc|d=s',
      'jrr|j=s',
      'geo_viirs|g=s',
      'reproc|r=i',
      'fovs|f=i',
      'processors|p=i',
      'verbose|v+', 'vv', 'vvv',
      'xfiles|x',
      'subset|z!',
      'aggregate|a!',
      'break|b!',
      'quarantine|q!',
      'skipretrieval|k!',
      'clean|c!') or usage();
    defined($options{version}) and die "$rev\n";
    defined($options{help}) and usage();
    $options{nothing} = defined($options{nothing}) ? $options{nothing} : 0;
    $options{xfiles} = defined($options{xfiles}) ? $options{xfiles} : 0;
    defined($options{input_files}) or usage();
    defined($options{sat}) or usage();
    (grep {$_ eq $options{sat}} qw/npp j01/) or do {
      print "\nERROR --- $name\: Unsupported MISSION\n"; usage(); };
    $options{mode} = defined($options{mode}) ? $options{mode} : 'LR_IRMW';
    (grep {$_ eq $options{mode}} qw/HR_IRMW LR_IRMW HR_IRO LR_IRO/) or do {
      print "\nERROR --- $name\: Unsupported MODE\n"; usage(); };
    (defined($options{work})) or $options{work} = $pwd;
    (defined($options{reproc})) or $options{reproc} = 0;
    (defined($options{fovs})) or do { 
      ($options{sat} eq 'npp') and $options{fovs} = 7;
      ($options{sat} eq 'j01') and $options{fovs} = 9; };
    (defined($options{dynanc})) or 
      $options{dynanc} = File::Spec -> catfile($ENV{CSPP_NUCAPS_HOME}, 
        'data', 'dynamic_ancillary');
    foreach (qw/dynanc work/) {
      $options{$_} = File::Spec -> catfile(abs_path(dirname($options{$_})),
        basename($options{$_}));
    }
    $options{jrr} = defined($options{jrr}) ? $options{jrr} : 'OFF';
    (grep {$_ eq $options{jrr}} qw/JRR VIIRS OFF/) or do {
      print "\nERROR --- $name\: Unsupported JRR_VIIRS\n"; usage(); };
    $options{geo_viirs} = defined($options{geo_viirs}) ? $options{geo_viirs} : 'GMTCO';
    (grep {$_ eq $options{geo_viirs}} qw/GMTCO GMODO/) or do {
      print "\nERROR --- $name\: Unsupported GEOLOC_VIIRS\n"; usage(); };
    (defined($options{processors})) or $options{processors} = 1;
    $options{verbose} = defined($options{verbose}) ? $options{verbose} : 0;
    $options{vv} = defined($options{vv}) ? $options{vv} : 0;
    $options{vvv} = defined($options{vvv}) ? $options{vvv} : 0;
    $options{verbose} = $options{verbose} + 2 * $options{vv};
    $options{verbose} = $options{verbose} + 3 * $options{vvv};
    $options{verbose} = min($options{verbose}, 3);
    delete @options{'vv', 'vvv'};
    $options{clean} = defined($options{clean}) ? $options{clean} : 1;
    $options{aggregate} = defined($options{aggregate}) ? $options{aggregate} : 0;
    $options{skipretrieval} = defined($options{skipretrieval}) ? $options{skipretrieval} : 0;
    $options{subset} = defined($options{subset}) ? $options{subset} : 0;
    $options{break} = defined($options{break}) ? $options{break} : 1;
    $options{quarantine} = defined($options{quarantine}) ? $options{quarantine} : 0;
    ($options{reproc} > 1 and $options{reproc} < 3) and $options{quarantine} = 1; # no LFTP fetch.
    return %options;
  }

# ----------------------------
  sub get_datetime_from_nppgeo
# ============================
# ($YYYY, $MM, $DD, $hh, $mm, $ss) = get_datetime_from_nppgeo($file, $sdr, $block);
  {
    # $file is the name of the npp/SDR file, full path if not in $PWD.
    # $sdr is one of 'ATMS' or 'CrIS'
    # $block is either 'Beginning' or 'Ending' - ok, you get the picture.
    # Uses global $H5DUMP as validated location of h5dump executable
    # This is a filename independent way of getting date/time from
    # GATMO and GCRSO h5 files. Is it better than using the filename? - maybe
    # not, since the filenaming convention for npp SDRs is so well framed.
    my ($file, $sdr, $block) = (shift, shift, shift);
    my $var = 'Data_Products/XXXX-SDR-GEO/XXXX-SDR-GEO_Gran_0';
    $var =~ s/XXXX/$sdr/g;
    my $block_Date = '"'.$block.'_Date"';
    my $block_Time = '"'.$block.'_Time"';
    my $name = (caller(0))[3];
    $name =~ s/main/$MAIN/g;
    my ($date, $time) = ('N/A', 'N/A');
    my $cmd = "$H5DUMP --onlyattr -d $var $file |";
    open(my $fh, $cmd) or die "$name\: Could not run command $cmd\nAbort";
    while (<$fh>) { chomp;
      /$block_Date/ and do {
        while (<$fh>) { chomp;
          /0\)\:/ and do {
            my $tmp = (split(/0\)\:/, $_))[1];
            $tmp =~ s/\"//g;
            $date = trim_string($tmp); 
            last; };
        } };
      /$block_Time/ and do {
        while (<$fh>) { chomp;
          /0\)\:/ and do {
            my $tmp = (split(/0\)\:/, $_))[1];
            $tmp =~ s/\"//g;
            $time = trim_string($tmp); 
            last; };
        } };
    }
    close($fh);
    my $YYYY = substr($date, 0, 4);
    my $MM = substr($date, 4, 2);
    my $DD = substr($date, 6, 2);
    my $hh = substr($time, 0, 2);
    my $mm = substr($time, 2, 2);
    my $ss = substr($time, 4, 2);
    return ($YYYY, $MM, $DD, $hh, $mm, $ss);
  }

# -----------------------
  sub gmtkeys_from_nppgeo
# =======================
# @gmtkeys = gmtkeys_from_nppgeo($file, $sdr);
  {
    # $file is the name of the npp/SDR file, full path if not in $PWD.
    # $sdr is one of 'ATMS' or 'CrIS'
    # returns start & end as seconds since 1970, useful as hash keys
    my ($file, $sdr) = (shift, shift);
    my @ret=();
    foreach my $qual (qw/Beginning Ending/) {
      my @datetime = get_datetime_from_nppgeo($file, $sdr, $qual);
      my ($YYYY, $MM, $DD, $hh, $mm, $ss) = @datetime;
      my $gmtkey = timegm_nocheck($ss, $mm, $hh, $DD, $MM-1, $YYYY);
      push(@ret, $gmtkey);
    }
    return @ret;
  }

# -----------------------------------------------------------
  sub is_integer { defined $_[0] and $_[0] =~ /^[+-]?\d+$/; }
# ===========================================================

# -------------------------
  sub is_plausible_datetime
# =========================
# $status = is_plausible_datetime($YYYY, $MM, $DD, $hh, $mm);
  {
    # NOTE: Year < 2000 is considered implausible in this function.
    my ($YYYY, $MM, $DD, $hh, $mm) = (@_);
    my $implausible_year = 2000;
    is_integer($YYYY) or return 0;
    $YYYY < $implausible_year and return 0;
    is_integer($MM) or return 0;
    $MM >= 1 and $MM <= 12 or return 0;
    is_integer($DD) or return 0;
    $DD >= 1 and $DD <= 31 or return 0;
    is_integer($hh) or return 0;
    $hh >= 0 and $hh <= 23 or return 0;
    is_integer($mm) or return 0;
    $mm >= 0 and $mm <= 59 or return 0;
    return 1;
  }

# --------------------------
  sub job_coverage_start_end
# ==========================
# ($jcs, $jce) = job_coverage_start_end($cris_geo);
  {
    # Compute the job coverage start and end strings based on CrIS GEO
    # granule name, $cris_geo.  The problem I encountered is that this
    # name includes the YearMonthDay only for the start time, so that if
    # end time is in next day, just prepending the YearMonthDay string will
    # be wrong.  So I need to check if end time (Hour,Min,Sec) precedes start
    # time, and if so, add a day using some Perl date arithmetic (because it
    # could be end of month or end of year, and I need to roll-over correctly).
    my $cris_geo = shift;
    my $tag = substr($cris_geo, 6, 31);
    my $sdate = substr($tag, 5, 8);
    my $edate = $sdate;
    my $stime = substr($tag, 15, 7);
    my $etime = substr($tag, 24, 7);
    if ($etime < $stime) {
      my $YYYY = substr($sdate, 0, 4);
      my $MM = substr($sdate, 4, 2);
      my $DD = substr($sdate, 6, 2);
      # $MM-1 because month is zero-based; $DD+1 because we want next day.
      gmtime(timegm_nocheck(0, 0, 0, $DD+1, $MM-1, $YYYY));
      $YYYY = sprintf("%04d", $tm_year + 1900);
      $MM = sprintf("%02d", $tm_mon + 1);
      $DD = sprintf("%02d", $tm_mday);
      $edate = $YYYY.$MM.$DD;
    }
    my $jcs = $sdate.$stime;
    my $jce = $edate.$etime;
    return ($jcs, $jce);
  }

# ---------------
  sub lftp_mirror
# ===============
# $status = lftp_mirror($site, $file, $local, %options);
  {
    # executes lftp in mirror mode, but if NOT $options{quarantine}
    my ($site, $file, $local) = (shift, shift, shift);  
    my (%options) = @_;
    my $name = (caller(0))[3];
    $name =~ s/main/$MAIN/g;
    my $v = '';
    ($options{verbose} == 3) and $v = '-v';
    -e "$local/$file" and do {
      ($options{verbose} == 3) and print "$name\: $file FOUND in local repo\n";
      return 1;
    };
    ($options{quarantine}) and do {
      ($options{verbose} >= 1) and 
        warn "$name\: WARNING: $file NOT FOUND in local repo AND --quarantine";
      return 0;
    };
    (-d $local) or do {
      ($options{verbose} >=2) and print "Making $local\n";
      mkpath($local) or
        die "$name\: $local cannot be made.\nAbort"; };
    open(my $fh, '>script_file.lftp') or die "$name\: Cannot open script_file.lftp $!";
    print $fh "set dns:order inet inet6\n";            # default is 'inet6 inet'
    #print $fh "set ssl:verify-certificate no\n";       # default is ‘yes'
    #print $fh "set net:timeout 1\n";                   # default is '5'
    #print $fh "set net:max-retries 5\n";               # default is '10'
    #print $fh "set net:persist-retries 1\n";           # default is '5'
    #print $fh "set net:reconnect-interval-base 6\n";   # default is '60'
    print $fh "mirror $v -L -n --no-empty-dirs $site -I $file $local\n";
    close($fh);
    my $cmd = "flock $local $LFTP -f script_file.lftp";
    system($cmd) == 0 or do {
      ($options{verbose} >= 1) and warn "$name\: WARNING: $cmd failed";
      return 0; };
    return 1;
  }

# ------------------------------------
  sub make_nucaps_preproc_pcf_template
# ====================================
# make_nucaps_preproc_pcf_template('NUCAPS_Preproc.pl.PCF.template', MODE);
  {
    # The name of the PCF $pcf is written in $PWD, assumed to be $tmpdir
    # MODE is 'HR_IRMW', 'LR_IRMW', 'HR_IRO' or 'LR_IRO'
    # <stuff in angle brackets> is to be replaced in runtime PCF
    my ($pcf, $sat, $mode, $jrr) = (shift, shift, shift, shift);
    my ($res, $sens) = split(/\_/, $mode);
    my $name = (caller(0))[3];
    $name =~ s/main/$MAIN/g;
    open(my $fh, ">$pcf") or die "$name\: Cannot open template $pcf for write.\nAbort";
    print $fh "job_coverage_start=<JCS>\n";
    print $fh "job_coverage_end=<JCE>\n";
    print $fh "SATELLITE_ID=$sat\n";
    print $fh "OPS_BIN=$HOME/Common_Bin\n";
    print $fh "SCRIPT_OPS=$HOME/scripts\n";
    print $fh "UUIDGEN_LOC=$UUIDGEN\n";
    print $fh "CRIS_SDR=<CRIS_SDR>\n";
    print $fh "CRIS_GEO=<CRIS_GEO>\n";
    if ($sens eq 'IRO') {
      print $fh "ATMS_PROCESSING_FLAG=OFF\n";
    } else {
      print $fh "ATMS_PROCESSING_FLAG=ON\n";
      print $fh "ATMS_TDR_1=<ATMS_TDR_1>\n";
      print $fh "ATMS_TDR_2=<ATMS_TDR_2>\n";
      print $fh "ATMS_TDR_3=<ATMS_TDR_3>\n";
      print $fh "ATMS_GEO_TARGET_1=<ATMS_GEO_TARGET_1>\n";
      print $fh "ATMS_GEO_TARGET_2=<ATMS_GEO_TARGET_2>\n";
      print $fh "ATMS_GEO_TARGET_3=<ATMS_GEO_TARGET_3>\n";
    }
    ($jrr eq 'VIIRS') and print $fh "VIIRS_PROCESSING_FLAG=ON\n";
    ($jrr ne 'VIIRS') and print $fh "VIIRS_PROCESSING_FLAG=OFF\n";
    ($jrr eq 'JRR') and print $fh "JRR_PROCESSING_FLAG=ON\n";
    ($jrr ne 'JRR') and print $fh "JRR_PROCESSING_FLAG=OFF\n";
    print $fh "APOD=yes\n";
    print $fh "DEM_FILE=$HOME/DEM/Global_DEM.bin\n";
    print $fh "DEM_LAT_PTS=21600\n";
    print $fh "DEM_LON_PTS=43200\n";
    support_file($fh, 'preproc', 'RESAMPLE_DATA', "$HOME/resample/resample.data");
    print $fh "H5DUMP_LOC=$H5DUMP\n";
    print $fh "CRIS_STATUS_CHECK_FLAG=OFF\n";
    print $fh "ATMS_STATUS_CHECK_FLAG=OFF\n";
    print $fh "OLR_PROCESSING_FLAG=ON\n";
    print $fh "OLR_TEMPLATE=$HOME/CDLFILES/cris_olr.nc\n";
    print $fh "PRODUCTION_SITE=CSPP\n";
    print $fh "PRODUCTION_ENVIRONMENT=DEV\n";
    # HR resolution
    if ($res eq 'HR') {
      print $fh "NUCAPS_ALL_HR_TEMPLATE=$HOME/CDLFILES/nucaps_all_HR.nc\n";
      print $fh "OLR_DIR_HR=$HOME/olr/HR\n";
      print $fh "OLR_COR_HR=$HOME/olr/HR/rad_corr_reg_coef_HSR_17boxcar_airsv10ab_1.asc\n";
      print $fh "OLR_REG_HR=$HOME/olr/HR/olr_reg_coef_cv005_17boxcar_2.asc\n";
    }
    # LR resolution
    if ($res eq 'LR') {
      print $fh "NUCAPS_ALL_LR_TEMPLATE=$HOME/CDLFILES/nucaps_all.nc\n";
      print $fh "OLR_DIR_LR=$HOME/olr/LR\n";
      print $fh "OLR_COR_LR=$HOME/olr/LR/rad_corr_reg_coef_17boxcar_airsv10ab_3.asc\n";
      print $fh "OLR_REG_LR=$HOME/olr/LR/olr_reg_coef_cv005_17boxcar_2.asc\n";
    }
    # The section below will need extra attention once turned ON - variable # of files !!!
    print $fh "VIIRS_COUNT=<VIIRS_COUNT>\n";
    print $fh "VIIRS_CF_X=<VIIRS_CF_X>\n";
    print $fh "VIIRS_CF_GEO_X=<VIIRS_CF_GEO_X>\n";
    print $fh "VIIRS_CH_X=<VIIRS_CH_X>\n";
    print $fh "VIIRS_CH_GEO_X=<VIIRS_CH_GEO_X>\n";
    print $fh "VIIRS_COLOCATION_LUT_DIR=$HOME/viirs_colocation\n";
    # The section above will need extra attention once turned ON - variable # of files !!!
    close($fh) or die "$name\: Cannot close template $pcf.\nAbort";
    return;
  }
  #
  # Keep for reference purposes, at least until you have implemented VIIRS_PROCESSING
  # `````````````````````````````````````````````````````````````````````````````````
  # CRIS_SDR=SCRIS_npp_d20170821_t1446239_e1446537_b30137_c20170821155017179501_nobc_ops.h5
  # CRIS_GEO=GCRSO_npp_d20170821_t1446239_e1446537_b30137_c20170821155017137714_nobc_ops.h5
  # 
  # ATMS_TDR_1=TATMS_npp_d20170821_t1445466_e1446183_b30137_c20170821152824320731_nobc_ops.h5
  # ATMS_TDR_2=TATMS_npp_d20170821_t1446186_e1446503_b30137_c20170821152839378457_nobc_ops.h5
  # ATMS_TDR_3=TATMS_npp_d20170821_t1446506_e1447223_b30137_c20170821152852385569_nobc_ops.h5
  # 
  # ATMS_GEO_TARGET_1=GATMO_npp_d20170821_t1445466_e1446183_b30137_c20170821152824260751_nobc_ops.h5
  # ATMS_GEO_TARGET_2=GATMO_npp_d20170821_t1446186_e1446503_b30137_c20170821152839325819_nobc_ops.h5
  # ATMS_GEO_TARGET_3=GATMO_npp_d20170821_t1446506_e1447223_b30137_c20170821152852336889_nobc_ops.h5
  # VIIRS_PROCESSING_FLAG=OFF
  # JRR_PROCESSING_FLAG=ON
  # VIIRS_COUNT=2
  # VIIRS_CF_1=JRR-CloudMask_v1r1_npp_s201708211445234_e201708211446476_c201708211542520.nc
  # VIIRS_CF_GEO_1=GMTCO_npp_d20170821_t1445234_e1446476_b30137_c20170821153544624516_nobc_ops.h5
  # VIIRS_CH_1=JRR-CloudHeight_v1r1_npp_s201708211445234_e201708211446476_c201708211544540.nc
  # VIIRS_CH_GEO_1=GMTCO_npp_d20170821_t1445234_e1446476_b30137_c20170821153544624516_nobc_ops.h5
  # VIIRS_CF_2=JRR-CloudMask_v1r1_npp_s201708211446488_e201708211448130_c201708211542560.nc
  # VIIRS_CF_GEO_2=GMTCO_npp_d20170821_t1446488_e1448130_b30137_c20170821153555336974_nobc_ops.h5
  # VIIRS_CH_2=JRR-CloudHeight_v1r1_npp_s201708211446488_e201708211448130_c201708211545030.nc
  # VIIRS_CH_GEO_2=GMTCO_npp_d20170821_t1446488_e1448130_b30137_c20170821153555336974_nobc_ops.h5
  # VIIRS_COLOCATION_LUT_DIR=/data/data311/letitias/NUCAPS/CrISOPS/viirs_colocation

# --------------------------------------
  sub make_nucaps_retrieval_pcf_template
# ======================================
# make_nucaps_retrieval_pcf_template('NUCAPS_Retrieval.pl.PCF.template', $sat, $mode, $fovs, $grib);
  {
    # The name of the PCF $pcf is written in $PWD, assumed to be $tmpdir
    # $sat is 'npp' or 'j01'
    # $mode is 'HR_IRMW', 'LR_IRMW', 'HR_IRO' or 'LR_IRO'
    # $fovs is taken from $options{fovs}
    # $grib is 0 for grib2 (normal), 1 for grib1 (reproc. mode), 2 for grib2 (reproc. mode).
    # <stuff in angle brackets> is to be replaced in runtime PCF
    my ($pcf, $sat, $mode, $fovs, $grib) = (shift, shift, shift, shift, shift);
    my ($res, $sens) = split(/\_/, $mode);
    my $name = (caller(0))[3];
    $name =~ s/main/$MAIN/g;
    open(my $fh, ">$pcf") or die "$name\: Cannot open template $pcf for write.\nAbort";
    print $fh "job_coverage_start=<JCS>\n";
    print $fh "job_coverage_end=<JCE>\n";
    print $fh "SATELLITE_ID=$sat\n";
    print $fh "OPS_BIN=$HOME/Common_Bin\n";
    print $fh "SCRIPT_OPS=$HOME/scripts\n";
    print $fh "NUCAPS_ALL_PRODUCT=<NUCAPS_ALL_PRODUCT>\n";
    print $fh "BOUNDING_BOX_PRODUCT=<BOUNDING_BOX_PRODUCT>\n";
    if ($grib == 1) {
      print $fh "WGRIB_LOC=$WGRIB\n";
      print $fh "WGRIB_COMMAND=$HOME/scripts/run_wgrib.pl\n";
    } else {
      print $fh "WGRIB_LOC=$WGRIB2\n";
      print $fh "WGRIB_COMMAND=$HOME/scripts/run_wgrib2.pl\n";
    }
    print $fh "H5DUMP_LOC=$H5DUMP\n";  # use $H5DUMP, but that has to be full path!
    print $fh "UUIDGEN_LOC=$UUIDGEN\n";
    print $fh "BYTESWAP_FLAG=ON\n";
    print $fh "NUM_OF_FOVS=$fovs\n";
    print $fh "PRODUCTION_SITE=CSPP\n";
    print $fh "PRODUCTION_ENVIRONMENT=DEV\n";
    support_file($fh, 'retrieval', 'L2ERROR_IN', "$HOME/retrieval/errest/jpl_100.inp");
    support_file($fh, 'retrieval', 'OLRCOEFFILE', "$HOME/retrieval/RTA/airs_olr.dat");
    support_file($fh, 'retrieval', 'HSBWEIGHTFILE', "$HOME/retrieval/mit_ret/L2.M.weight.hsb.v1.0.0.anc");
    support_file($fh, 'retrieval', 'ECOFFILE', "$HOME/retrieval/mit_ret/L2.M.ecof_705.v1.0.0.anc");
    support_file($fh, 'retrieval', 'MWCOVFILE', "$HOME/retrieval/mit_ret/L2.M.cov100av.v1.0.0.anc");
    support_file($fh, 'retrieval', 'UARSCLIMFILE', "$HOME/retrieval/mit_ret/L2.uars_clim.v1.0.3.anc");
    support_file($fh, 'retrieval', 'NCEPCLIMFILE', "$HOME/retrieval/mit_ret/ncep_clim.bin");
    support_file($fh, 'retrieval', 'MASUDAFILE', "$HOME/retrieval/surface/L2.masuda.v2.0.0.anc");
    support_file($fh, 'retrieval', 'CO_CLIMFILE', "$HOME/retrieval/climatology/mopitt_v4clim_nhsh_20180605.dat");
    print $fh "AVN_FILE1=<AVN_FILE1>\n";
    print $fh "AVN_FILE2=<AVN_FILE2>\n";
    ($grib  < 3) and print $fh "AVN_GRID_SIZE=1\n";
    ($grib == 3) and print $fh "AVN_GRID_SIZE=0.5\n";
    print $fh "LAC_FILENAME=none\n";
    print $fh "NUCAPS_L2_TEMPLATE=$HOME/CDLFILES/nucaps_l2.nc\n";
    # HR resolution
    if ($res eq 'HR') {
      support_file($fh, 'retrieval', 'TC_AMSU_HR', "$HOME/retrieval/RTA/170406atms_rta_planck.asc");
      print $fh "NUCAPS_CCR_HR_TEMPLATE=$HOME/CDLFILES/nucaps_ccr_HR.nc\n";
      print $fh "NUCAPS_CCR_AR_HR_TEMPLATE=$HOME/CDLFILES/nucaps_ccr_archive_HR.nc\n";
      support_file($fh, 'retrieval', 'TUNINGMASKFILE_HR', "$HOME/retrieval/tuning/170214_hr_cris_tuning_mask_guard.asc");
      support_file($fh, 'retrieval', 'NOAAFRQFILE_HR', "$HOME/retrieval/regress/rt_noaa/cris_fsr_wavenumber.asc");
      support_file($fh, 'retrieval', 'SOLARFILE_HR', "$HOME/retrieval/solar/cris_solar_v11a.txt");
      support_file($fh, 'retrieval', 'TC_AIRS_HR', "$HOME/retrieval/RTA/RTA_cris_fsr_v11a_20161222.bin");
      ($sat eq 'npp') and support_file($fh, 'retrieval', 'IRNOISEFILE_HR', "$HOME/retrieval/noise/161101cris_fsr_guard.dat");
      ($sat eq 'j01') and support_file($fh, 'retrieval', 'IRNOISEFILE_HR', "$HOME/retrieval/noise/j01_cris_fsr_noise.dat");
      support_file($fh, 'retrieval', 'CLDAVGFILE_HR', "$HOME/retrieval/avgcld/cris_888g.t1");
      # IMRW sensors
      if ($sens eq 'IRMW') {
        print $fh "IR_ONLY_FLAG=OFF\n";
        support_file($fh, 'retrieval', 'TUNINGCOEFFILE_HR_IRMW', "$HOME/retrieval/tuning/180608_cris_irv11_tdr.asc");
        support_file($fh, 'retrieval', 'NOAAEIGFILE_HR_IRMW', "$HOME/retrieval/regress/rt_noaa/reg_eigenvec_cris_fsr_20150115_20170224.asc");
        support_file($fh, 'retrieval', 'NOAAREGFILE_HR_IRMW', "$HOME/retrieval/regress/rt_noaa/reg_coef_ccr_cris_fsr_20170726.asc");
        support_file($fh, 'retrieval', 'NOAAANGFILE_HR_IRMW', "$HOME/retrieval/regress/rt_noaa/reg_coef_all_cris_fsr_20170225.asc");
        support_file($fh, 'retrieval', 'RTAERRFILE_HR_IRMW', "$HOME/retrieval/rtaerr/180608_rtaerr_npp_cris_atms_tdr.asc");
        ($sat eq 'npp') and support_file($fh, 'retrieval', 'MWNOISEFILE_HR_IRMW', "$HOME/retrieval/noise/atms_20141212.dat");
        ($sat eq 'j01') and support_file($fh, 'retrieval', 'MWNOISEFILE_HR_IRMW', "$HOME/retrieval/noise/j01_atms_20180418.dat");
        support_file($fh, 'retrieval', 'IO_NL_HR_IRMW', "$HOME/retrieval/namelists/$sat/hr_irmw_io_cris.nl");
        support_file($fh, 'retrieval', 'IO_NL_7FOV_HR_IRMW', "$HOME/retrieval/namelists/$sat/hr_irmw_io_7fov_cris.nl");
        support_file($fh, 'retrieval', 'IO_NL_8FOV_HR_IRMW', "$HOME/retrieval/namelists/$sat/hr_irmw_io_8fov_cris.nl");
        support_file($fh, 'retrieval', 'CLOUDS_NL_HR_IRMW', "$HOME/retrieval/namelists/$sat/hr_irmw_clouds_cris.nl");
        support_file($fh, 'retrieval', 'MICROW_NL_HR_IRMW', "$HOME/retrieval/namelists/$sat/hr_irmw_microw_cris.nl");
        support_file($fh, 'retrieval', 'OZONE_NL_HR_IRMW', "$HOME/retrieval/namelists/$sat/hr_irmw_ozone_cris.nl");
        support_file($fh, 'retrieval', 'PRO_NL_HR_IRMW', "$HOME/retrieval/namelists/$sat/hr_irmw_pro_cris.nl");
        support_file($fh, 'retrieval', 'TEMP_NL_HR_IRMW', "$HOME/retrieval/namelists/$sat/hr_irmw_temp_cris.nl");
        support_file($fh, 'retrieval', 'WATER_NL_HR_IRMW', "$HOME/retrieval/namelists/$sat/hr_irmw_water_cris.nl");
      }
      # IRO sensors
      if ($sens eq 'IRO') {
        print $fh "IR_ONLY_FLAG=ON\n";
        support_file($fh, 'retrieval', 'TUNINGCOEFFILE_HR_IRO', "$HOME/retrieval/tuning/180608_cris_irv11_tdr.asc");
        support_file($fh, 'retrieval', 'NOAAEIGFILE_HR_IRO', "$HOME/retrieval/regress/rt_noaa/reg_eigenvec_cris_fsr_20150115_20170224.asc");
        support_file($fh, 'retrieval', 'NOAAREGFILE_HR_IRO', "$HOME/retrieval/regress/rt_noaa/reg_coef_ccr_cris_fsr_noatms_20170726.asc");
        support_file($fh, 'retrieval', 'NOAAANGFILE_HR_IRO', "$HOME/retrieval/regress/rt_noaa/reg_coef_all_cris_fsr_noatms_20170310.asc");
        support_file($fh, 'retrieval', 'RTAERRFILE_HR_IRO', "$HOME/retrieval/rtaerr/180608_rtaerr_npp_cris_atms_tdr.asc");
        ($sat eq 'npp') and support_file($fh, 'retrieval', 'MWNOISEFILE_HR_IRO', "$HOME/retrieval/noise/atms_20141212.dat");
        ($sat eq 'j01') and support_file($fh, 'retrieval', 'MWNOISEFILE_HR_IRO', "$HOME/retrieval/noise/j01_atms_20180418.dat");
        support_file($fh, 'retrieval', 'IO_NL_HR_IRO', "$HOME/retrieval/namelists/$sat/hr_iro_io_cris.nl");
        support_file($fh, 'retrieval', 'IO_NL_7FOV_HR_IRO', "$HOME/retrieval/namelists/$sat/hr_iro_io_7fov_cris.nl");
        support_file($fh, 'retrieval', 'IO_NL_8FOV_HR_IRO', "$HOME/retrieval/namelists/$sat/hr_iro_io_8fov_cris.nl");
        support_file($fh, 'retrieval', 'CLOUDS_NL_HR_IRO', "$HOME/retrieval/namelists/$sat/hr_iro_clouds_cris.nl");
        support_file($fh, 'retrieval', 'MICROW_NL_HR_IRO', "$HOME/retrieval/namelists/$sat/hr_iro_microw_cris.nl");
        support_file($fh, 'retrieval', 'OZONE_NL_HR_IRO', "$HOME/retrieval/namelists/$sat/hr_iro_ozone_cris.nl");
        support_file($fh, 'retrieval', 'PRO_NL_HR_IRO', "$HOME/retrieval/namelists/$sat/hr_iro_pro_cris.nl");
        support_file($fh, 'retrieval', 'TEMP_NL_HR_IRO', "$HOME/retrieval/namelists/$sat/hr_iro_temp_cris.nl");
        support_file($fh, 'retrieval', 'WATER_NL_HR_IRO', "$HOME/retrieval/namelists/$sat/hr_iro_water_cris.nl");
      }
    }
    # LR resolution
    if ($res eq 'LR') {
      support_file($fh, 'retrieval', 'TC_AMSU_LR', "$HOME/retrieval/RTA/tr_atms_new.dat");
      print $fh "NUCAPS_CCR_LR_TEMPLATE=$HOME/CDLFILES/nucaps_ccr.nc\n";
      print $fh "NUCAPS_CCR_AR_LR_TEMPLATE=$HOME/CDLFILES/nucaps_ccr_archive.nc\n";
      support_file($fh, 'retrieval', 'TUNINGMASKFILE_LR', "$HOME/retrieval/tuning/120214_cris_tuning_mask.asc");
      support_file($fh, 'retrieval', 'NOAAFRQFILE_LR', "$HOME/retrieval/regress/rt_noaa/cris_v03.frq");
      support_file($fh, 'retrieval', 'SOLARFILE_LR', "$HOME/retrieval/solar/cris_solar_v10a.txt");
      support_file($fh, 'retrieval', 'TC_AIRS_LR', "$HOME/retrieval/RTA/binary.trcoef.cris.v10a");
      support_file($fh, 'retrieval', 'IRNOISEFILE_LR', "$HOME/retrieval/noise/tobin120120.dat");
      support_file($fh, 'retrieval', 'CLDAVGFILE_LR', "$HOME/retrieval/avgcld/cris_v10a.t1");
      # IMRW sensors
      if ($sens eq 'IRMW') {
        print $fh "IR_ONLY_FLAG=OFF\n";
        support_file($fh, 'retrieval', 'TUNINGCOEFFILE_LR_IRMW', "$HOME/retrieval/RTA/2013_0115_0415_0715_1015_30_4.asc");
        support_file($fh, 'retrieval', 'NOAAEIGFILE_LR_IRMW', "$HOME/retrieval/regress/rt_noaa/eigvec.cris_1305_05122015");
        support_file($fh, 'retrieval', 'NOAAREGFILE_LR_IRMW', "$HOME/retrieval/regress/rt_noaa/cris_reg_coef_ccr_1305_05142015.asc");
        support_file($fh, 'retrieval', 'NOAAANGFILE_LR_IRMW', "$HOME/retrieval/regress/rt_noaa/cris_reg_coef_cld_1305_05122015.asc");
        support_file($fh, 'retrieval', 'RTAERRFILE_LR_IRMW', "$HOME/retrieval/RTA/151218_rtaerr_irv10_tdr.asc");
        support_file($fh, 'retrieval', 'MWNOISEFILE_LR_IRMW', "$HOME/retrieval/noise/atms_20141212.dat");
        support_file($fh, 'retrieval', 'IO_NL_LR_IRMW', "$HOME/retrieval/namelists/$sat/lr_irmw_io_cris.nl");
        support_file($fh, 'retrieval', 'IO_NL_7FOV_LR_IRMW', "$HOME/retrieval/namelists/$sat/lr_irmw_io_7fov_cris.nl");
        support_file($fh, 'retrieval', 'IO_NL_8FOV_LR_IRMW', "$HOME/retrieval/namelists/$sat/lr_irmw_io_8fov_cris.nl");
        support_file($fh, 'retrieval', 'CLOUDS_NL_LR_IRMW', "$HOME/retrieval/namelists/$sat/lr_irmw_clouds_cris.nl");
        support_file($fh, 'retrieval', 'MICROW_NL_LR_IRMW', "$HOME/retrieval/namelists/$sat/lr_irmw_microw_cris.nl");
        support_file($fh, 'retrieval', 'OZONE_NL_LR_IRMW', "$HOME/retrieval/namelists/$sat/lr_irmw_ozone_cris.nl");
        support_file($fh, 'retrieval', 'PRO_NL_LR_IRMW', "$HOME/retrieval/namelists/$sat/lr_irmw_pro_cris.nl");
        support_file($fh, 'retrieval', 'TEMP_NL_LR_IRMW', "$HOME/retrieval/namelists/$sat/lr_irmw_temp_cris.nl");
        support_file($fh, 'retrieval', 'WATER_NL_LR_IRMW', "$HOME/retrieval/namelists/$sat/lr_irmw_water_cris.nl");
      }
      # IRO sensors
      if ($sens eq 'IRO') {
        print $fh "IR_ONLY_FLAG=ON\n";
        support_file($fh, 'retrieval', 'TUNINGCOEFFILE_LR_IRO', "$HOME/retrieval/RTA/2013_0115_0415_0715_1015_30_4.asc");
        support_file($fh, 'retrieval', 'NOAAEIGFILE_LR_IRO', "$HOME/retrieval/regress/rt_noaa/eigvec.cris_1305_10262016");
        support_file($fh, 'retrieval', 'NOAAREGFILE_LR_IRO', "$HOME/retrieval/regress/rt_noaa/cris_reg_coef_ccr_less2k_noatms_11032016.asc");
        support_file($fh, 'retrieval', 'NOAAANGFILE_LR_IRO', "$HOME/retrieval/regress/rt_noaa/cris_reg_coef_cld_less2k_noatms_10262016.asc");
        support_file($fh, 'retrieval', 'RTAERRFILE_LR_IRO', "$HOME/retrieval/rtaerr/cris_rtaerr_v10a.asc");
        support_file($fh, 'retrieval', 'MWNOISEFILE_LR_IRO', "$HOME/retrieval/noise/atms_20141212.dat");
        support_file($fh, 'retrieval', 'IO_NL_LR_IRO', "$HOME/retrieval/namelists/$sat/lr_iro_io_cris.nl");
        support_file($fh, 'retrieval', 'IO_NL_7FOV_LR_IRO', "$HOME/retrieval/namelists/$sat/lr_iro_io_7fov_cris.nl");
        support_file($fh, 'retrieval', 'IO_NL_8FOV_LR_IRO', "$HOME/retrieval/namelists/$sat/lr_iro_io_8fov_cris.nl");
        support_file($fh, 'retrieval', 'CLOUDS_NL_LR_IRO', "$HOME/retrieval/namelists/$sat/lr_iro_clouds_cris.nl");
        support_file($fh, 'retrieval', 'MICROW_NL_LR_IRO', "$HOME/retrieval/namelists/$sat/lr_iro_microw_cris.nl");
        support_file($fh, 'retrieval', 'OZONE_NL_LR_IRO', "$HOME/retrieval/namelists/$sat/lr_iro_ozone_cris.nl");
        support_file($fh, 'retrieval', 'PRO_NL_LR_IRO', "$HOME/retrieval/namelists/$sat/lr_iro_pro_cris.nl");
        support_file($fh, 'retrieval', 'TEMP_NL_LR_IRO', "$HOME/retrieval/namelists/$sat/lr_iro_temp_cris.nl");
        support_file($fh, 'retrieval', 'WATER_NL_LR_IRO', "$HOME/retrieval/namelists/$sat/lr_iro_water_cris.nl");
      }
    }
    close($fh) or die "$name\: Cannot close template $pcf.\nAbort";
    return;
  }

# -----------------------------------
  sub make_nucaps_subset_pcf_template
# ===================================
# make_nucaps_subset_pcf_template('NUCAPS_Subset.pl.PCF.template', SAT, MODE);
  {
    # The name of the PCF $pcf is written in $PWD, assumed to be $tmpdir
    # SAT is 'npp' or 'j01'
    # MODE is 'HR_IRMW', 'LR_IRMW', 'HR_IRO' or 'LR_IRO'
    # <stuff in angle brackets> is to be replaced in runtime PCF
    my ($pcf, $sat, $mode) = (shift, shift, shift);
    my ($res, $sens) = split(/\_/, $mode);
    my $name = (caller(0))[3];
    $name =~ s/main/$MAIN/g;
    open(my $fh, ">$pcf") or die "$name\: Cannot open template $pcf for write.\nAbort";
    print $fh "job_coverage_start=<JCS>\n";
    print $fh "job_coverage_end=<JCE>\n";
    print $fh "SATELLITE_ID=$sat\n";
    print $fh "OPS_BIN=$HOME/Common_Bin\n";
    print $fh "SCRIPT_OPS=$HOME/scripts\n";
    print $fh "NUCAPS_ALL_PRODUCT=<NUCAPS_ALL_PRODUCT>\n";
    print $fh "NUCAPS_C0300_ALLFOVS_FLAG=ON\n";
    print $fh "NUCAPS_C0300_ALLFOVS_PCS3B_FLAG=ON\n";
    print $fh "H5DUMP_LOC=$H5DUMP\n";  # use $H5DUMP, but that has to be full path!
    if ($res eq 'HR') {
      print $fh "NUCAPS_C0300_ALLFOVS_HR_TEMPLATE=$HOME/CDLFILES/nucaps_c0300_allfovs_HR.nc\n";
      print $fh "NUCAPS_C0300_ALLFOVS_PCS3B_HR_TEMPLATE=$HOME/CDLFILES/nucaps_c0300_allfovs_pcs3b_HR.nc\n";
      support_file($fh, 'subset', 'PC_1BAND_COEFF_HR', "$HOME/pc_coeffs/eigvec.cris_HSR_713_band1.01042017");
      support_file($fh, 'subset', 'PC_2BAND_COEFF_HR', "$HOME/pc_coeffs/eigvec.cris_HSR_865_band2.01052017");
      support_file($fh, 'subset', 'PC_3BAND_COEFF_HR', "$HOME/pc_coeffs/eigvec.cris_HSR_633_band3.01052017");
    }
    if ($res eq 'LR') {
      print $fh "NUCAPS_C0300_ALLFOVS_LR_TEMPLATE=$HOME/CDLFILES/nucaps_c0300_allfovs.nc\n";
      print $fh "NUCAPS_C0300_ALLFOVS_PCS3B_LR_TEMPLATE=$HOME/CDLFILES/nucaps_c0300_allfovs_pcs3b.nc\n";
      support_file($fh, 'subset', 'PC_1BAND_COEFF_LR', "$HOME/pc_coeffs/eigvec_20160129_band1_ascii_real");
      support_file($fh, 'subset', 'PC_2BAND_COEFF_LR', "$HOME/pc_coeffs/eigvec_20160129_band2_ascii_real");
      support_file($fh, 'subset', 'PC_3BAND_COEFF_LR', "$HOME/pc_coeffs/eigvec_20160129_band3_ascii_real");
    }
    close($fh) or die "$name\: Cannot close template $pcf.\nAbort";
    return;
  }

# --------------------------
  sub npp_granules_to_swaths
# ==========================
# %swaths = npp_granules_to_swaths(@files);
  {
    # %swaths are key:value pairs $timegm:@filename_list
    # Break list of files into groups that comprise contiguous overpass swaths
    # to assist cspp_preprocess() in staging data. This is not a complicated
    # routine - all it does is group files together that fall within a time
    # window and ASSUMES them to belong to the same overpass; a reasonable
    # assumption for DB users, but details like dropped granules are wholly
    # ignored. The times are extracted from the well-specified NPP filenaming
    # convention.  ATMS granule-to-granule contiguity will need to be checked
    # later once you have built candidate CrIS granules to process and chosen
    # the fore-, mid- and aft ATMS granules.
    my @files = @_;
    my $name = (caller(0))[3];
    $name =~ s/main/$MAIN/g;
    my $time_window = 60 * $AGG_WINDOW_MINUTES;
    my $atms_beg = 2 * 60; # 2 mins of ATMS overlap
    my $atms_end = $time_window + $atms_beg;
    my %swaths = ();
    my ($YYYY, $MM, $DD, $hh, $mm, $found) = undef;
    # First loop distributes CrIS files uniquely to swaths
    foreach (sort @files) {
      (substr($_, 1, 3) eq 'ATM') and next;
      $YYYY = substr($_, 11, 4);
      $MM = substr($_, 15, 2);
      $DD = substr($_, 17, 2);
      $hh = substr($_, 21, 2);
      $mm = substr($_, 23, 2);
      is_plausible_datetime($YYYY, $MM, $DD, $hh, $mm) or
        die "$name\: Date from npp file $_ implausible\nAbort";
      my $t = timegm_nocheck(0, $mm, $hh, $DD, $MM-1, $YYYY);
      $found = 0;
      foreach my $key (sort keys %swaths) {
        (abs($key - $t) < $time_window) and do {
          push(@{$swaths{$key}}, $_);
          $found = 1; last; };
      }
      ($found) or @{$swaths{$t}} = ($_);
    }
    # Second loop distributes ATMS non-uniquely to swaths 
    foreach (sort @files) {
      (substr($_, 1, 3) ne 'ATM') and next;
      $YYYY = substr($_, 11, 4);
      $MM = substr($_, 15, 2);
      $DD = substr($_, 17, 2);
      $hh = substr($_, 21, 2);
      $mm = substr($_, 23, 2);
      is_plausible_datetime($YYYY, $MM, $DD, $hh, $mm) or
        die "$name\: Date from npp file $_ implausible\nAbort";
      my $t = timegm_nocheck(0, $mm, $hh, $DD, $MM-1, $YYYY);
      foreach my $key (sort keys %swaths) {
        (($key - $t) > $atms_beg) and next;
        (($t - $key) > $atms_end) and next;
        push(@{$swaths{$key}}, $_);
      }
    }
    return %swaths;
  }

# --------------------
  sub nucaps_avn_files
# ====================
# %avn = nucaps_avn_files($gmtbeg, %dynanc_loc);
  {
    # $gmtbeg is the beginning time of the CrIS SDR
    # %dynanc_loc is hash of dynamic ancillary data locations, local & remote
    # %avn is a hash of URL, LOCAL repo directory plus AVN file pairs, keyed
    # 'table' and 'spare', where 'table' is original AVN truth table as defined
    # by Chris Barnet, and 'spare' is the fallback possition if 'table' options
    # are unavailable. This fact cannot be known now, only when we go to the
    # ancillary data server or, if --quarantine is invoked, when we check the
    # local repo. Because the date is in both the URL and the LOCAL directory
    # name, and this could be different in the fallback positions, we have to
    # build all of $url, $local, $avn1 and $avn2 and return them in the hash;
    # I am going to just put then in 4 element vector in that order. The
    # bounding gfs files to use are for NUCAPS processing of CrIS SDR with
    # input $gmtbeg.
    my $gmtbeg = shift;
    my (%dynanc_loc) = @_;
    my $name = (caller(0))[3];
    $name =~ s/main/$MAIN/g;
    # AVN truth table as defined by Chris Barnet
    my %avn_truth_table = ('00z' => [('18z', 'F06', 'F09', 'yesterday')],
                           '03z' => [('00z', 'F03', 'F06', 'today')],
                           '06z' => [('00z', 'F06', 'F09', 'today')],
                           '09z' => [('06z', 'F03', 'F06', 'today')],
                           '12z' => [('06z', 'F06', 'F09', 'today')],
                           '15z' => [('12z', 'F03', 'F06', 'today')],
                           '18z' => [('12z', 'F06', 'F09', 'today')],
                           '21z' => [('18z', 'F03', 'F06', 'today')]);
    # AVN truth table fallback position 
    my %avn_truth_spare = ('00z' => [('12z', 'F12', 'F15', 'yesterday')],
                           '03z' => [('18z', 'F09', 'F12', 'yesterday')],
                           '06z' => [('18z', 'F12', 'F15', 'yesterday')],
                           '09z' => [('00z', 'F09', 'F12', 'today')],
                           '12z' => [('00z', 'F12', 'F15', 'today')],
                           '15z' => [('06z', 'F09', 'F12', 'today')],
                           '18z' => [('06z', 'F12', 'F15', 'today')],
                           '21z' => [('12z', 'F09', 'F12', 'today')]);
    gmtime($gmtbeg);
    my $YYYY = sprintf("%04d", $tm_year + 1900);
    my $ddd = sprintf("%03d", $tm_yday + 1);
    my $hh = sprintf("%02d", $tm_hour);
    my $key = sprintf("%02dz" , $hh - $hh % 3);
    my %avn = ();
    foreach (qw/table spare/) {
      my ($anal, $fct1, $fct2, $relday) = @{$avn_truth_table{$key}};
      /spare/ and ($anal, $fct1, $fct2, $relday) = @{$avn_truth_spare{$key}};
      my $avn1 = $dynanc_loc{GFSTEMPLATE};
      my $avn2 = $dynanc_loc{GFSTEMPLATE};
      $hh = substr($anal, 0, 2);
      my ($ff1, $ff2) = (substr($fct1, 1, 2), substr($fct2, 1, 2));
      $avn1 =~ s/%f/$ff1/g;
      $avn2 =~ s/%f/$ff2/g;
      ($relday eq 'yesterday') and $ddd--;
      gmtime(timegm_nocheck(0, 0, $hh, $ddd, 0, $YYYY));
      $YYYY = sprintf("%04d", $tm_year + 1900);
      my $MM = sprintf("%02d", $tm_mon + 1);
      my $DD = sprintf("%02d", $tm_mday);
      $ddd = sprintf("%03d", $tm_yday + 1);
      $hh = sprintf("%02d", $tm_hour);
      my $YY = substr($YYYY, 2 , 2);
      my @tokens = qw/ %Y   %y   %m    %j   %d   %H/;
      my @values = ($YYYY, $YY, $MM, $ddd, $DD, $hh);
      my $site = $dynanc_loc{'GFSSITE'};
      my $proto = $dynanc_loc{'GFSPROTO'};
      my $local = $dynanc_loc{'GFSLOCAL'};
      my $remote = $dynanc_loc{'GFSREMOTE'};
      for (my $j=0; $j<=$#tokens; $j++) {
        $local =~ s/$tokens[$j]/$values[$j]/g;
        $remote =~ s/$tokens[$j]/$values[$j]/g; 
        $avn1 =~ s/$tokens[$j]/$values[$j]/g; 
        $avn2 =~ s/$tokens[$j]/$values[$j]/g; 
      }
      # ********** MAKE PATH SO THAT WARNINGS CAN BE REPORTED **********
      if (! -d $local) {
        eval { mkpath($local) };
        if ($@) { warn "$name\: WARNING $local cannot be made"; }
      }
      my $url = "$proto$site/$remote";
      @{$avn{$_}} = ($url, $local, $avn1, $avn2);
    }
    return (%avn);
  }

# ---------------------------
  sub populate_npp_swath_info
# ===========================
# @swath = populate_npp_swath_info(\%dynanc_loc, %options);
  {
    # %dynanc_loc is remote/local information for GFS dynamic ancillary data
    # %options are the command line & default general processing options
    # @swath is an array of hash references, each reference points to granule
    # processing information, and this will finally be expressed as changes
    # to the PCF templates in $tmpdir. During the execution of this function,
    # the $PWD is assumed to be $tmpdir/swath_nnn.
    my %dynanc_loc = %{(shift)};
    my (%options) = @_;
    my $name = (caller(0))[3];
    $name =~ s/main/$MAIN/g;
    # push each valid (& thus fully populated) %granule in here
    my (@atms_geo, @atms_beg, @atms_end, @atms_tdr) = ((), (), (), ());
    my @swath = ();
    (grep {$_ eq $options{mode}} qw/HR_IRMW LR_IRMW/) and do {
      # Make co-arrays of ATMS geo filenames and their data beg/end times
      @atms_tdr = (sort(glob('TATMS*')));
      foreach my $atms_geo (sort(glob('GATMO*'))) {
        my ($beg, $end) = gmtkeys_from_nppgeo($atms_geo, 'ATMS');
        push(@atms_geo, $atms_geo);
        push(@atms_beg, $beg); push(@atms_end, $end);
      }
    };
    # Loop over the candidate CrIS data in $PWD
    my $waitawhile = $SLEEP_GFS_MINUTES;
    my @cris_sdr = (sort(glob('SCRI*')));
    foreach my $cris_geo (sort(glob('GCRSO*'))) {
      # Check for companion SCRI file for this GCRSO file
      my $tag = substr($cris_geo, 6, 31);
      my ($jcs, $jce) = job_coverage_start_end($cris_geo);
      my %granule = (JCS => $jcs, JCE => $jce, CRIS_GEO => $cris_geo);
      my $cris_sdr = 'N/A';
      foreach (@cris_sdr) {
        (substr($_, 6, 31) eq $tag) and $cris_sdr = $_;
      }
      ($cris_sdr eq 'N/A') and do {
        warn "$name\: WARNING: No SCRIS match to $cris_geo\n";
        next;
      };
      $granule{CRIS_SDR} = $cris_sdr;
      # Get the beg/end data times for this CrIS granule
      my ($beg, $end) = gmtkeys_from_nppgeo($cris_geo, 'CrIS');
      my $jdx = 0;
      (grep {$_ eq $options{mode}} qw/HR_IRMW LR_IRMW/) and do {
        # Use a mid point CrIS time to get the closest ATMS file
        my $mid = ($beg + $end) / 2;
        # Do we have trio of fore-, mid-, and aft- ATMS geo files?
        for (my $j=0; $j<=$#atms_geo; $j++) {
          (($mid > $atms_beg[$j]) and ($mid < $atms_end[$j])) and $jdx = $j;
        }
        (($jdx < 1) or ($jdx >= $#atms_geo)) and next;
        $granule{ATMS_GEO_TARGET_1} = $atms_geo[$jdx-1];
        $granule{ATMS_GEO_TARGET_2} = $atms_geo[$jdx];
        $granule{ATMS_GEO_TARGET_3} = $atms_geo[$jdx+1];
        # Check for companion TATMS file for each of the above GATMO files
        my @tdr = ();
        for (my $j=$jdx-1; $j<=$jdx+1; $j++) {
          $tag = substr($atms_geo[$j], 6, 31);
          my $tdr = 'N/A';
          foreach (@atms_tdr) {
            (substr($_, 6, 31) eq $tag) and $tdr = $_;
          }
          ($tdr eq 'N/A') and do {
            ($options{verbose} >= 1) and
              warn "$name\: WARNING No TATMS match to $atms_geo[$j]";
            next;
          };
          push(@tdr, $tdr);
        }
        ($#tdr == 2) or next;
        $granule{ATMS_TDR_1} = $tdr[0];
        $granule{ATMS_TDR_2} = $tdr[1];
        $granule{ATMS_TDR_3} = $tdr[2];
      };
      # Fields for retrieval phase PCF : With DAP v4-1_20170822-DB comes the change
      # that BOUNDING_BOX_PRODUCT and NUCAPS_ALL_PRODUCT have creation times in their
      # filenames, so I cannot know the name until after I run NUCAPS_Preproc.pl.  So
      # I put in the retrieval PCF the wildcard that I will later use to replace itself.
      my @parts = (substr($jcs, 0, 8), substr($jcs, 8, 7), substr($jce, 8, 7));
      my $globit = sprintf("s%s%s_e*%s", $parts[0], $parts[1], $parts[2]);
      $granule{BOUNDING_BOX_PRODUCT} = sprintf("NUCAPS-Bounding-Box*%s*.txt", $globit);
      $granule{NUCAPS_ALL_PRODUCT} = sprintf("NUCAPS-ALL*%s*.nc", $globit);
      # Before accepting this granule, check ATMS file contiguity
      my $contiguous = 1;
      (grep {$_ eq $options{mode}} qw/HR_IRMW LR_IRMW/) and do {
        $end = $atms_end[$jdx-1];
        for (my $j=$jdx; $j<=$jdx+1; $j++) {
          $beg = $atms_beg[$j];
          my $delta = $beg - $end;
          ($delta > 2) and $contiguous = 0;
          ($delta < 0) and $contiguous = 0;
          $end = $atms_end[$j];
        }
      };
      ($contiguous) or next;
      # Load AVN_FILE1/2 with dummy values in case we are skipping the retrieval
      $granule{AVN_FILE1} = 'DUMMY_AVN1';
      $granule{AVN_FILE2} = 'DUMMY_AVN2';
      ($options{skipretrieval}) or do {
        # Use the beginning time of CrIS to define the AVN gfs files
        my %avn = nucaps_avn_files($beg, %dynanc_loc);
        ($options{verbose} == 3) and pretty_dump('%avn', %avn);
        # Final check is that we have available the GFS data required
        my ($url, $local, $avn1, $avn2) = ('N/A', 'N/A', 'N/A', 'N/A');
        foreach (qw/table spare/) {
          ($url, $local, $avn1, $avn2) = @{$avn{$_}};
          foreach my $avn (($avn1, $avn2)) {
            my $target = File::Spec -> catfile($local, $avn);
            -e $avn or do {
              lftp_mirror($url, $avn, $local, %options);
              -e $target and symlink($target, $avn) or
                warn "$name\: WARNING: Cannot link to $target";
              ($options{repo}) and next; # don't need multiple attempts at local repo
              ($options{quarantine}) and next; # don't need multiple attempts at local repo
              -e $avn and next; # we have $target and link, so we are done for this $avn
              my $j = $waitawhile; # so $j is still in scope after the following loop
              for ($j=$waitawhile; $j>0; $j--) {
                ($options{verbose} >= 1) and warn "$name\: SLEEPING at minute $j waiting for $avn\n";
                sleep 60;
                lftp_mirror($url, $avn, $local, %options);
                -e $target or next;
                symlink($target, $avn) or warn "$name\: WARNING: Cannot link to $target";
                last;
              }
              $waitawhile = $j;
            };
          }
          ((-e $avn1) and (-e $avn2)) and last;
        }
        -e $avn1 or next; # no AVN_FILE1 so we bail on this granule
        -e $avn2 or next; # no AVN_FILE2 so we bail on this granule
        # All checks complete, push it onto swath processing stack
        $granule{AVN_FILE1} = $avn1;
        $granule{AVN_FILE2} = $avn2;
      };
      ($options{verbose} == 3) and pretty_dump('%granule', %granule);
      push (@swath, \%granule);
    }
    return @swath;
  }

# ---------------
  sub pretty_dump
# ===============
# pretty_dump($title, %hash);
  {
    # Do a "pretty print" of %hash by effectively grepping Dumper().
    # Assumes standard module Data::Dumper has been loaded.
    my $title = 'DEBUG '.shift;
    my (%hash) = @_;
    print '-' x length($title)."\n";
    print "$title\n";
    print '=' x length($title)."\n";
    foreach (split('\n', Dumper(\%hash))) {
      chomp;
      ((/VAR1/) or (/\}\;/)) and next;
      $_ =~ s/  //g; # shorten up the whitespace
      print "$_\n";
    }
    print "$title _END_\n";
  }

# ----------------
  sub support_file
# ================
# support_file($fh, $phase, $keyName, $fileName);
  {
    my ($fh, $phase, $keyName, $fileName) = (shift, shift, shift, shift);
    my $name = (caller(0))[3];
    $name =~ s/main/$MAIN/g;
    copy($fileName, "support/$phase") or die "$name\: $fileName $!";
    my $fn = basename($fileName);
    print $fh "$keyName=$fn\n";
  }

# ---------------
  sub token_value
# ===============
# $value = token_value($token, $file, $mask);  ...example follows...
# $value = token_value('%j', 't1.11332.1500.mod04.hdf', 't1.%y%j.%H%M.mod04.hdf');
  {
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
    #                                                                           #
    #   WARNING: This function not used in NUCAPS but kept as token reference   #
    #                                                                           #
    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
    #
    # Takes the filename $file (within which tokens have already been expanded),
    # and the $mask (within which tokens are unexpanded) and, for a given $token
    # return the $value extracted from $file. Here is an example ...
    #   $value = token_value('%j', 't1.11332.1500.mod04.hdf', 't1.%y%j.%H%M.mod04.hdf');
    # which would return $value as '332'.
    # These are the supported tokens, returns the first occurance if more than one:
    #   %Y  Year with century. (1582-9999)
    #   %y  Year without century (00–99).
    #   %m  Month as decimal number (01–12).
    #   %j  Day of year as decimal number (001–366).
    #   %d  Day of the month as decimal number (01–31).
    #   %H  Hours as decimal number (00–23).
    #   %M  Minute as decimal number (00–59).
    #   %S  Second as decimal number (00–59).
    #   %F  3-digit forecast hour (000-999).
    #   %f  2-digit forecast hour (00-99).
    my $token = shift;
    my $file = shift;
    my $mask = shift;
    my $value = undef;
    my $expand = $mask;
    my @tokens = qw/  %Y %y %m  %j %d %H %S %M  %F %f/;
    my @widths =   (   4, 2, 2,  3, 2, 2, 2, 2,  3, 2);
    my $width = 0;
    for (my $j=0; $j<=$#tokens; $j++) {
      ($token eq $tokens[$j]) and do { $width = $widths[$j]; next; };
      my $blank = 'X' x $widths[$j];
      $expand =~ s/$tokens[$j]/$blank/g; 
    }
    my $i = index($expand, $token);
    if ($i > -1) {
      $value = substr($file, $i, $width);
    }
    return $value;
  }

# ---------------
  sub trim_string
# ===============
# $new_string = trim_string($old_string);
  {
    my $string = shift;
    $string =~ s/^\s+//;
    $string =~ s/\s+$//;
    return $string;
  }

# ---------
  sub usage
# =========
# usage();
  {
    my $name = (caller(0))[3];
    $name =~ s/main/$MAIN/g;
    open(my $fh, "<$0");
    while (<$fh>) { chomp;
      if ( /START-HELP/ ) {
        while (<$fh>) { chomp;
          ( /END-HELP/ ) and do { close($fh); exit(9); };
          $_ =~ s/\$CSPP_NUCAPS_HOME/$ENV{CSPP_NUCAPS_HOME}/g;
          my $help = (split(/\#/, $_))[1]; # remove leading hash
          $help and print "$help\n" or print "\n";
        }
      }
    }
    die "$name\: ERROR! Have you tinkered with help in script header?\nAbort";
  }

# --------------------
  sub validate_options
# ====================
# %options = validate_options($filemask, %options);
  {
    # $filemask is for globbing the nominated satellite data.
    # %options are the command line options to "validate".
    # Fatal errors in here terminate with die without showing usage() message.
    # The most important task is to convert $options{input_files} to a scalar
    # $options{input_dir} and filename list put back in $options{input_files}.
    # This is also where the working and ancillary data directories are made
    # if they do not exist, and their writable status verified.
    my $filemask = shift;
    my (%options) = @_;
    my $name = (caller(0))[3];
    $name =~ s/main/$MAIN/g;
    my $globber = $options{input_files};
    my $altglob = $globber;
    (-d $options{input_files}) and do {
      my $base = $options{input_files};
      $globber = File::Spec -> catfile($base, $filemask);
      ($options{xfiles}) or do { # 'bait & switch' - well not really, but this
        my $mask = 'N/A';        #   supports a second DB filename format
        ($options{sat} eq 'aqua') and $mask = 'A*.{GEO,L1B_LAC}';
        ($options{sat} eq 'terra') and $mask = 'T*.{GEO,L1B_LAC}';
        $altglob = File::Spec -> catfile($base, $mask); }; };
    my @input_files = sort(glob($globber));
    ($#input_files < 0) and @input_files = sort(glob($altglob));
    ($#input_files < 0) and die "$name\: INPUTFILES not found.\nAbort";
    $options{input_dir} = dirname(abs_path($input_files[0]));
    $options{input_files} = ();
    foreach (@input_files) { push(@{$options{input_files}}, basename($_)); }
    ($options{reproc} > 3) and do {
      die "$name\: --reproc=GRIB $options{reproc} is not valid.\nAbort"; };
    ($options{sat} eq 'npp') and do {
      ($options{fovs} < 7 or $options{fovs} > 9) and do {
        die "$name\: --fovs=N_FOVS $options{fovs} is not valid.\nAbort"; }; };
    (-d $options{work}) or do {
      ($options{verbose} >=2) and print "Making WORK dir $options{work}\n";
      mkpath($options{work}) or
        die "$name\: WORK dir $options{work} cannot be made.\nAbort"; };
    (-w $options{work}) or
      die "$name\: WORK dir $options{work} not writable.\nAbort";
    ($options{skipretrieval}) and do {
      $options{subset} = 1;
      $options{quarantine} = 1; };
    ($options{quarantine}) or do {
      (-d $options{dynanc}) or do {
        ($options{verbose} >=2) and print "Making dynanc DIR $options{dynanc}\n";
        mkpath($options{dynanc}) or
          die "$name\: DYNANC dir $options{dynanc} cannot be made.\nAbort"; };
      (-w $options{dynanc}) or
        die "$name\: DYNANC dir $options{dynanc} not writable.\nAbort"; };
    return %options;
  }

